package edu.washington.cs.rfid.mstreams.data;

import edu.washington.cs.rfid.mstreams.exception.MStreamSyntaxException;

public class NodeTransition implements Comparable<NodeTransition> {

   private static final String TOKEN_DELIMETER = ";";
   
   //--//
   
   public long   node1;
   public long   node2;
   public double prob;
   
   //--//
   
   public NodeTransition(String triple) throws MStreamSyntaxException {
      
      try {
         // Trim the parentheses
         String   t      = triple.substring(1, triple.length() - 1);
         String[] tokens = t.split(NodeTransition.TOKEN_DELIMETER);
         
         this.node1 = Long  .parseLong  (tokens[0]);
         this.node2 = Long  .parseLong  (tokens[1]);
         this.prob  = Double.parseDouble(tokens[2]);

      } catch(Exception e){
         throw new MStreamSyntaxException("Error parsing transition: " + triple);
      }
   }
   
   
   public NodeTransition(long node1, long node2, double prob){
	   this.node1 = node1;
	   this.node2 = node2;
	   this.prob  = prob;
   }
   
   
   public boolean equals(NodeTransition n){
	   return this.node1 == n.node1 && this.node2 == n.node2 && this.prob == n.prob;
   }
   
   
   public int compareTo(NodeTransition t){
	   int result = 0;
	   
	   if(this.node1 > t.node1){
		   result = 1;
	   } else if(this.node1 < t.node1){
		   result = -1;
	   } else {
		   if(this.node2 > t.node2){
			   result = 1;
		   } else if(this.node2 < t.node2){
			   result = -1;
		   } else {
			   if(this.prob > t.prob){
				   result = 1;
			   } else if(this.prob < t.prob){
				   result = -1;
			   } else {
				   result = 0;
			   }
		   }
	   }
	   
	   return result;
   }
   
   
   public String toString(){
      return "(" + node1 + NodeTransition.TOKEN_DELIMETER + node2 + NodeTransition.TOKEN_DELIMETER + this.prob + ")";
   }
}
