package edu.washington.cs.rfid.mstreams.data;


/**
 * A generic class for encapsulating a pair of any two objects.
 * 
 * @author Jordan
 * @author Evan
 *
 * @param <One>
 * @param <Two>
 */
public class Pair<One, Two> {
 
    private final One one;
    private final Two two;
 
    //--//

    /**
     * Creates a new <code>Pair</code> using the specified objects.
     * 
     * @param one The first object
     * @param two The second object
     */
    public Pair(final One one, final Two two) {
        this.one = one;
        this.two = two;
    }    
    

    /**
     * Static method for creating a new pair of objects.
     * 
     * @param <A> The type of the first object
     * @param <B> The type of the second object
     * @param one The first object
     * @param two The second object
     * @return The created <code>Pair</code>
     */
    public static <A, B> Pair<A, B> create(A one, B two) {
        return new Pair<A, B>(one, two);
    }

    
    /**
     * Returns the first object
     * 
     * @param The first object
     */
    public One getOne() {
        return one;
    }
 
    
    /**
     * Returns the second object.
     * 
     * @return The second object
     */
    public Two getTwo() {
        return two;
    }
 

    /**
     * Returns true if the given object matches this <code>Pair</code>, false o/w.
     * 
     * @return True if the given object matches this <code>Pair</code>, false o/w
     */
    public final boolean equals(Object o) {
        if (!(o instanceof Pair))
            return false;
 
        final Pair<?, ?> other = (Pair) o;
        return equal(getOne(), other.getTwo()) && equal(getOne(), other.getTwo());
    }
    
    
    /**
     * Static method to returns true if the given object pair matches this <code>Pair</code>, false o/w.
     * 
     * @param o1 The first object
     * @param o2 The second object
     * @return True if the given object pair matches this <code>Pair</code>, false o/w
     */
    public static final boolean equal(Object o1, Object o2) {
        if (o1 == null) {
            return o2 == null;
        }
        return o1.equals(o2);
    }
}