package edu.washington.cs.rfid.mstreams.data;

import java.util.Vector;



public class ParticleFilterStream {

   public Vector<MarginalCPTPair> sequence;
   
   //--//
   
   /**
    * Creates a new ParticleFilterStream.
    */
   public ParticleFilterStream(){
      this.sequence = new Vector<MarginalCPTPair>();
   }
   
   
   /**
    * Adds a new marginal, CPT pair to this particle filter stream.
    * @param marginal
    * @param cpt
    */
   public void addPair(ParticleFilterStreamMarginalData marginal, ParticleFilterStreamCPTData cpt){
      this.sequence.add(new MarginalCPTPair(marginal, cpt));
   }

   
   /**
    * Determines whether or not this stream is identical in value to another stream.
    * @param file
    * @return
    */
   public boolean equals(ParticleFilterStream file){
	   boolean result = false;
	   
	   if(this.sequence.size() == file.sequence.size()){
			System.out.println("\n\n\nSequence sizes are equal!");
				
			// Now compare marginals timestep by timestep
		    boolean marginalsOk = true;
		    boolean cptsOk      = true;
		    for(int i = 0; i < this.sequence.size(); i++){
		    	MarginalCPTPair p1 = this.sequence.get(i);
		    	MarginalCPTPair p2 = file.sequence.get(i);
		    	
		    	if(!p1.marginal.equals(p2.marginal)){
		    		marginalsOk = false;
		    	}
		    	
		    	if(!p1.cpt.equals(p2.cpt)){
		    		cptsOk = false;
		    	}
		    }
		    if(marginalsOk){
		    	System.out.println("\tMarginals are equal!");
		    } else {
		    	System.out.println("\tMarginals are NOT equal!");
		    }
		    if(cptsOk){
		    	System.out.println("\tCPTs are equal!");
		    } else {
		    	System.out.println("\tCPTs are NOT equal!");
		    }
		    result = marginalsOk && cptsOk;
		}
	   return result;
   }
   
   
   /**
    * Returns a String representing this stream.
    */
   public String toString(){
      String str = new String();
      for(MarginalCPTPair pair : this.sequence){
         str += pair.toString();
      }
      return str;
   }
   
   
   /**
    * Gets a description of the stream using the tag ID, start and end times. 
    * 
    * @return
    */
   public String getStreamDescription() {
	   String streamKey = new String();
	   
	   if(this.sequence.size() == 0){
		   streamKey = "emptyStream";
	   } else if(this.sequence.size() == 1){
		   MarginalCPTPair pair = this.sequence.firstElement();
		   streamKey            = pair.marginal.tagId     + "_" + 
		                          pair.cpt.startTimestamp + "_" +
		                          pair.cpt.endTimestamp;
	   } else {
	      MarginalCPTPair firstPair = this.sequence.firstElement();
		  MarginalCPTPair lastPair  = this.sequence.lastElement();
		   
		  streamKey = firstPair.marginal.tagId     + "_" +
		              firstPair.cpt.startTimestamp + "_" +
		              lastPair.cpt.endTimestamp;
	   }
	   return streamKey;
   }

   
   //--//
   
   
   /**
    * An internal class that contains a marginal, CPT pair.
    * @author evan
    */
   public class MarginalCPTPair {
      public ParticleFilterStreamMarginalData marginal;
      public ParticleFilterStreamCPTData      cpt;
      
      public MarginalCPTPair(ParticleFilterStreamMarginalData marginal, ParticleFilterStreamCPTData cpt){
         this.marginal = marginal;
         this.cpt      = cpt;
      }
      
      public String toString(){
         return marginal.toString() + "\n" + cpt.toString() + "\n";
      }
   }
}
