package edu.washington.cs.rfid.mstreams.data;

import java.text.DecimalFormat;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import storageManager.schema.ITuple;
import edu.washington.cs.rfid.mstreams.exception.MStreamSyntaxException;

public class ResultStreamCPTData {

   private static final String TOKEN_DELIMETER    = ",";
   
   //--//
   
   public long                                     startTimestamp;
   public long                                     endTimestamp;
   public Vector<StateTransition>                  stateTransitions;
   public HashMap<Integer,HashMap<Integer,Double>> transitionMap;
   
   //--//
   
   public ResultStreamCPTData(long                           startTimestamp,
                              long                           endTimestamp  ,
                              Map<ITuple,Map<ITuple,Double>> map){
      this.startTimestamp   = startTimestamp;
      this.endTimestamp     = endTimestamp;
      this.stateTransitions = new Vector<StateTransition>();
      this.transitionMap    = new HashMap<Integer,HashMap<Integer,Double>>();
      
      for(ITuple state1Tuple : map.keySet()){
         Map<ITuple,Double> subMap = map.get(state1Tuple);
         
         for(ITuple state2Tuple : subMap.keySet()){
            int             state1 = ((Integer)state1Tuple.getAttributeValue(0)).intValue();
            int             state2 = ((Integer)state2Tuple.getAttributeValue(0)).intValue();
            double          prob   = subMap.get(state2Tuple);
            StateTransition trans  = new StateTransition(state1,state2,prob);
            
            this.stateTransitions.add(trans);
            
            if(null == this.transitionMap.get(state1)){
               this.transitionMap.put(state1, new HashMap<Integer,Double>());
            }
            this.transitionMap.get(state1).put(state2, prob);
         }
      }
   }
   
   
   public Vector<Double> getTotalProbs(){
      
      Vector<Double> totalProbs = new Vector<Double>();
      
      for(Integer state1 : this.transitionMap.keySet()){
         HashMap<Integer,Double> subMap = this.transitionMap.get(state1);
         double                  totalP = 0.0;
         for(Double prob : subMap.values()){
            totalP += prob.doubleValue();
         }
         totalProbs.add(totalP);
      }
      return totalProbs;
   }
   
   
   public boolean equals(ResultStreamCPTData c){
      boolean result = false;
      
      if(this.startTimestamp == c.startTimestamp && this.endTimestamp == c.endTimestamp){
         
         // Compare list of transitions
         if(this.stateTransitions.size() == c.stateTransitions.size()){
            Collections.sort(this.stateTransitions);
            Collections.sort(c.stateTransitions);
            
            result = true;
            
            for(int i = 0; i < this.stateTransitions.size(); i++){
               if(!this.stateTransitions.get(i).equals(c.stateTransitions.get(i))){
                  result = false;
                  break;
               }
            }
         } else {
            System.out.println("CPT: State transitions are not the same size!");
            System.out.println("file1 transitions " + this.stateTransitions.size() + ":");
            for(StateTransition t : this.stateTransitions){
               System.out.print(t.toString() + " ");
            }
            System.out.println("\nfile2 transitions " + c.stateTransitions.size() + ":");
            for(StateTransition t : c.stateTransitions){
               System.out.print(t.toString() + " ");
            }
         }
         
         // Now compare maps
         Set<Integer> myKeys    = this.transitionMap.keySet();
         Set<Integer> otherKeys = c.transitionMap.keySet();
         
         if(myKeys.size() == otherKeys.size()){
            System.out.println("CPT: map keys sets same length!");
            
            if(myKeys.equals(otherKeys)){
               System.out.println("CPT: map keys sets are equal!");
               
               // Now compare map contents
               for(Integer key : myKeys){
                  if(!this.transitionMap.get(key).keySet().equals(c.transitionMap.get(key).keySet())){
                     result = false;
                     System.out.println("CPT: map values not equal for key: " + key);
//                   System.out.println("CPT: 1) " + key + " --> " +  );
                     break;
                  }
               }
            } else {
               System.out.println("CPT: map key sets not equal!");
               result = false;
            }
         } else {
            System.out.println("CPT: maps key set sizes not equal!");
            result = false;
         }
      }
      return result;
   }
   
   
   public String toPrettyString(){
      
      String       str = this.startTimestamp + ResultStreamCPTData.TOKEN_DELIMETER + 
                         this.endTimestamp   + ": "; 
      
      for(StateTransition transition : this.stateTransitions){
         str += transition.toPrettyString() + ResultStreamCPTData.TOKEN_DELIMETER;
      }
      str = str.substring(0, str.length()-1);
      return str;
   }
   
   
   public String toString(){
      String str = this.startTimestamp + ResultStreamCPTData.TOKEN_DELIMETER + 
                   this.endTimestamp   + ResultStreamCPTData.TOKEN_DELIMETER; 
      
      for(StateTransition transition : this.stateTransitions){
         str += transition.toString() + ResultStreamCPTData.TOKEN_DELIMETER;
      }
      str = str.substring(0, str.length()-2);
      return str;
   }
}
