package edu.washington.cs.rfid.mstreams.data;

import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Map;
import java.util.Vector;

import storageManager.schema.ITuple;

public class ResultStreamMarginalData {
   
   private static final String TOKEN_DELIMETER    = ",";
   
   //--//
   
   public long                 timestamp;
   public Vector<MaskProbPair> maskProbPairs;
   
   //--//   
   
   public ResultStreamMarginalData(long timestamp, Map<ITuple,Double> map){
      this.timestamp     = timestamp;
      this.maskProbPairs = new Vector<MaskProbPair>();
      
      for(ITuple tuple : map.keySet()){
         int    mask = ((Integer)tuple.getAttributeValue(0)).intValue();
         double prob = map.get(tuple);
         
         this.maskProbPairs.add(new MaskProbPair(mask,prob));
      }
   }
   
   
   public double getTotalProb(){
      double totalP = 0.0;
      for(MaskProbPair pair : this.maskProbPairs){
         totalP += pair.prob;
      }
      return totalP;
   }
   
   
   public boolean equals(ResultStreamMarginalData m){
      boolean result = false;
      
      if(this.timestamp == m.timestamp){
         if(this.maskProbPairs.size() == m.maskProbPairs.size()){
            Collections.sort(this.maskProbPairs);
            Collections.sort(m.maskProbPairs);
            
            result = true;
            
            for(int i = 0; i < this.maskProbPairs.size(); i++){
               MaskProbPair p1 = this.maskProbPairs.get(i);
               MaskProbPair p2 = this.maskProbPairs.get(i);
               
               if(p1.mask != p2.mask || p1.prob != p2.prob){
                  result = false;
                  break;
               }
            }
         }
      }
      return result;
   }
   
   
   public String toPrettyString(){
      String str = this.timestamp + ": ";
      
      for(int i = 0; i < this.maskProbPairs.size() - 1; i++){
         MaskProbPair pair  = this.maskProbPairs.get(i);
         str               += "(" + pair.toString() + "),";
      }
      str += " (" + this.maskProbPairs.get(this.maskProbPairs.size() - 1).toString() + ")";
      return str;
   }
   
   
   public String toString(){
      String str = this.timestamp + ResultStreamMarginalData.TOKEN_DELIMETER;
      
      for(int i = 0; i < this.maskProbPairs.size() - 1; i++){
         MaskProbPair pair  = this.maskProbPairs.get(i);
         str               += pair.toString() + ResultStreamMarginalData.TOKEN_DELIMETER;
      }
      str += this.maskProbPairs.get(this.maskProbPairs.size() - 1).toString();
      return str;
   }
   
   
   //--//
   
   
   public class MaskProbPair implements Comparable<MaskProbPair>{
      
      public int    mask;
      public double prob;
      
      public MaskProbPair(int mask, double prob){
         this.mask = mask;
         this.prob = prob;
      }
      
      public int compareTo(MaskProbPair p){
        int result = 0;
        
        if(this.mask > p.mask){
           result = 1;
        } else if(this.mask < p.mask){
           result = -1;
        } else {
           if(this.prob > p.prob){
              result = 1;
           } else if(this.prob < p.prob){
              result = -1;
           } else {
              result = 0;
           }
        }
        return result;
      }
      
      public String toString(){
         DecimalFormat df = new DecimalFormat("#.####");
         return this.mask + ResultStreamMarginalData.TOKEN_DELIMETER + df.format(this.prob);
      }
   }
}
