package edu.washington.cs.rfid.mstreams.data.db;

import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Vector;

import edu.washington.cs.rfid.data.db.DbParameters;
import edu.washington.cs.rfid.data.db.GenericDataConnectionPool;
import edu.washington.cs.rfid.data.db.SimpleDbConnection;

public class MStreamDataConnectionPool extends GenericDataConnectionPool {

   protected Vector<MStreamDataConnection> connections;      // List of connections in the pool 

   //--//
   
   /**
    * Creates a new <code>SimpleDbConnectionPool</code> for connections with the given database parameters.
    * @param dbParams The database parameters for the connections.
    */
   public MStreamDataConnectionPool(DbParameters dbParams){
      this.connections = new Vector<MStreamDataConnection>();
      this.dbParams    = dbParams;
      this.reaper      = new ConnectionReaper(this);
      
      this.reaper.start();
   }
   
   
   /**
    * Reaps connections which have not been used for a specified timeout.
    */
   @Override
   public synchronized void reapConnections(){
      
      long                               stale    = System.currentTimeMillis() - MStreamDataConnectionPool.REAP_TIMEOUT;
      Enumeration<MStreamDataConnection> connlist = this.connections.elements();
    
      while((connlist != null) && (connlist.hasMoreElements())) {
 
         MStreamDataConnection conn = (MStreamDataConnection)connlist.nextElement();

          if((conn.inUse()) && (stale > conn.getLastUse()) && (!conn.isConnected())) {
              this.removeConnection(conn);
         }
      }
   }
   
   
   public synchronized void closeConnections() {
      
      Enumeration<MStreamDataConnection> connlist = this.connections.elements();

      while((connlist != null) && (connlist.hasMoreElements())) {
         MStreamDataConnection conn = (MStreamDataConnection)connlist.nextElement();
         removeConnection(conn);
      }
   }


   public synchronized MStreamDataConnection getConnection() throws SQLException, ClassNotFoundException {

      MStreamDataConnection c = null;
      
      for(int i = 0; i < this.connections.size(); i++) {
         c = (MStreamDataConnection)this.connections.elementAt(i);
         if (c.lease()) {
            return c;
         }
      }
      c = new MStreamDataConnection(new SimpleDbConnection(this.dbParams));
      
      c.lease();
      this.connections.addElement(c);
      
      return c;
   } 

   //--//
   
   protected synchronized void removeConnection(MStreamDataConnection conn) {
      this.connections.removeElement(conn);
   }
}
