package edu.washington.cs.rfid.mstreams.data.db;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import edu.washington.cs.rfid.mstreams.data.StreamKey;


public class MStreamFileConnection {

   public static final String MOCK_DB_DIR  = "/data/MOCK-SQL-SERVER/";
   public static final String MOCK_DB_FILE = "stream-key.tbl";
   
   //--//
   
   protected HashMap<Long,String> streamKeyTbl;
   protected HashMap<String,Long> streamKeyTblRev;
   protected String               filename;
   
   //--//

   /**
    * Creates a new MStreamFileConnection.
    * @throws IOException
    */
   public MStreamFileConnection() {
      this.streamKeyTbl    = new HashMap<Long,String>();
      this.streamKeyTblRev = new HashMap<String,Long>();
      this.filename        = MStreamFileConnection.MOCK_DB_DIR + MStreamFileConnection.MOCK_DB_FILE;
      this.readInTable();
   }
   
   
   public StreamKey getStreamKey(String description){
      StreamKey sk  = null;
      Long      key = this.streamKeyTblRev.get(description);
      
      if(null != key){
         sk = new StreamKey(key,description);
      }
      return sk;
   }
   
   
   public StreamKey getStreamKey(long key){
      StreamKey sk   = null;
      String    desc = this.streamKeyTbl.get(key);
      
      if(null != desc){
         sk = new StreamKey(key,desc);
      }
      return sk;

   }
   
   
   public boolean addStreamKey(StreamKey sk){
      boolean success = true;
      
      this.streamKeyTbl   .put(sk.key, sk.description);
      this.streamKeyTblRev.put(sk.description, sk.key);
      
      return success;
   }
   
   
   public boolean removeStreamKey(StreamKey sk){
      boolean success = true;
      
      this.streamKeyTbl   .remove(sk.description);
      this.streamKeyTblRev.remove(sk.key);
      
      return success;
   }
   
   
   public void writeTable() throws IOException {
      BufferedWriter writer = null;
      
      try {
         writer = new BufferedWriter(new FileWriter(filename));
         
         for(Long key : streamKeyTbl.keySet()){
            String desc = streamKeyTbl.get(key);
            writer.write(key + "," + desc);
            writer.newLine();
         }
      } catch(FileNotFoundException e){
         e.printStackTrace();
      } catch(IOException e){
         e.printStackTrace();
      } finally {
         try {
            if(null != writer){
               writer.close();
            }
         } catch(IOException e){
            e.printStackTrace();
         }
      }
   }
   
   
   
   
   //-------------------------------------------------------------------------------//
   
   private void readInTable() {
      BufferedReader reader = null;
      
      try {
         reader      = new BufferedReader(new FileReader(this.filename));
         String line = null;

         while(null != (line = reader.readLine())){
            String[] pair = line.split(",");
            long     key  = Long.parseLong(pair[0]);
            String   desc = pair[1];

            this.streamKeyTbl   .put(key, desc);
            this.streamKeyTblRev.put(desc, key);
         }
      } catch(FileNotFoundException e){
         e.printStackTrace();
      } catch(IOException e){
         e.printStackTrace();
      } finally {
         if(null != reader){
            try {
               reader.close();
            } catch(IOException e){
               e.printStackTrace();
            }
         }
      }
   }
}
