package edu.washington.cs.rfid.mstreams.data.db;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import edu.washington.cs.rfid.data.Node;
import edu.washington.cs.rfid.data.db.DbParameters;
import edu.washington.cs.rfid.data.db.DeploymentDataConnection;
import edu.washington.cs.rfid.data.db.SimpleDbConnection;

/**
 * Class that maintains information on the location nodes in the
 * system, connections between then, alternate names for each node,
 * and distances between nodes.
 * 
 * @author evan
 */
public class NodeManager {
   
   protected static NodeManager instance;
   
   //--//
   
   protected DeploymentDataConnection   deployDb;
   protected Map<Long,Node>             nodeTable;
   protected Map<Long,Map<Long,Double>> pathLengths;
   
   //--//
   
   /**
    * Gets the singleton instance of NodeManager.
    * @return
    */
   public static NodeManager getInstance(){
      if(null == NodeManager.instance){
         NodeManager.instance = new NodeManager();
      }
      return NodeManager.instance;
   }
   
   
   /**
    * Creates a new NodeManager with data pulled from SQL Server.
    */
   protected NodeManager(){
      DbParameters dbParams = new DbParameters(DbParameters.TYPE_SQLSERVER,
                                                "myserver" ,
                                                1433                       ,
                                                "mydb"          , 
                                                "cascadia_user"              ,
                                                "mypasswd".toCharArray());
      
      try {
         this.deployDb = new DeploymentDataConnection(new SimpleDbConnection(dbParams));
         
         // Populate the node table
         Vector<Node> nodes = this.deployDb.getNodes();
         this.nodeTable     = new HashMap<Long,Node>();
         
         for(Node n : nodes){
            this.nodeTable.put(n.nid, n);
         }
         
         // Populate the path length table
         this.pathLengths = this.deployDb.getAllPathLengths();
         
      } catch(Exception e){
         e.printStackTrace();
      }
   }
   
   
   //--//
   
   
   /**
    * Gets the length of the shortest path between two nodes.
    * 
    * @param nid1
    * @param nid2
    * @return
    */
   public double getShortestPathLength(long nid1, long nid2){
      double len    = -1.0;
      Double lenObj = this.getShortestPathLength(nid1, nid2);
      
      if(null != lenObj){
         len = lenObj.doubleValue();
      }
      return len;
   }
   
   
   /**
    * Gets a Node by its node ID if one exists.
    * 
    * @param nid
    * @return
    */
   public Node getNode(long nid){
      return this.nodeTable.get(nid);
   }
}
