package edu.washington.cs.rfid.mstreams.event;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;

import markovianStream.ITimestep;
import api.queryResults.QueryResult;
import edu.washington.cs.rfid.mstreams.data.StreamKey;
import edu.washington.cs.rfid.mstreams.data.db.BdbConnection;
import edu.washington.cs.rfid.mstreams.data.db.LaharConnection;
import edu.washington.cs.rfid.mstreams.data.db.StreamKeyManager;
import edu.washington.cs.rfid.mstreams.util.StreamConverter;

public class FarEvent extends ScenicEvent {

   public List<AtEvent> atStreams;
   public double        threshold;
   public NearEvent     nearEvent;
   
   //--//
   
   /**
    * Creates a new FarEvent
    * 
    * @param entities
    * @param threshold
    */
   public FarEvent(List<AtEvent> atStreams, double threshold){
      super();
      this.atStreams = atStreams;
      this.threshold = threshold;
      this.nearEvent = new NearEvent(atStreams,threshold);
   }
   
   
   /**   
    * Gets the stream description for this event.
    */
   @Override
   public String getStreamDescription(){
      StringBuilder description = new StringBuilder();
      
      description.append("_FAR");
      
      Collections.sort(this.atStreams);
      for(AtEvent atEvent : this.atStreams){
         description.append("_");
         description.append(atEvent.getStreamDescription());
      }
      return description.toString();
   }
   
   
   /**
    * Executes the event using the given Lahar instance and returns a query result.
    * 
    * @param lahar
    * @return
    */
   @Override
   public QueryResult executeQuery(final BdbConnection    bdb  ,
                                   final LaharConnection  lahar, 
                                   final StreamKeyManager keyManager) throws IOException,
                                                                             ClassNotFoundException,
                                                                             SQLException {
      QueryResult result = null;
      StreamKey   sk     = this.getStreamKey(keyManager);
      
      if(lahar.streamKeyExists(sk, this.bitmaskSchema)){
         result = lahar.loadCachedResult(sk, this.bitmaskSchema);
      } else {
         QueryResult     nearResult       = nearEvent.executeQuery(bdb, lahar, keyManager);
         List<ITimestep> nearResultStream = StreamConverter.convertToMStream(nearResult);
         List<ITimestep> farResultStream  = StreamConverter.negateResultStream(nearResultStream);         
         
         if(bdb.storeMStream(farResultStream, sk.getKey(), this.bitmaskSchema, false)){
            lahar.moveStream(sk, this.bitmaskSchema);
            
            result = lahar.loadCachedResult(sk, this.bitmaskSchema);
         }         
      }
      return result;
   }
}