package edu.washington.cs.rfid.mstreams.event;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import markovianStream.ITimestep;
import api.queryResults.QueryResult;
import edu.washington.cs.rfid.mstreams.data.StreamKey;
import edu.washington.cs.rfid.mstreams.data.db.BdbConnection;
import edu.washington.cs.rfid.mstreams.data.db.LaharConnection;
import edu.washington.cs.rfid.mstreams.data.db.StreamKeyManager;
import edu.washington.cs.rfid.mstreams.util.ParticleFilterStreamMerger;

public class NearEvent extends ScenicEvent {
   
   public List<AtEvent> atStreams;
   public double        threshold;
   
   //--//
   
   /**
    * Creates a new NearEvent.
    * 
    * @param entities The entities in question
    * @param threshold The near-ness threshold in meters
    */
   public NearEvent(List<AtEvent> atStreams, double threshold){
      this.atStreams = atStreams;
      this.threshold = threshold;
   }
   
   
   /**
    * Gets the description of this event stream
    * @return
    */
   @Override
   public String getStreamDescription(){
      StringBuilder description = new StringBuilder();
      
      description.append("_NEAR");
      
      Collections.sort(this.atStreams);
      for(AtEvent atEvent : this.atStreams){
         description.append("_");
         description.append(atEvent.getStreamDescription());
      }
      return description.toString();
   }
   
   
   /**
    * Executes the event using the given Lahar instance and returns a query result.
    * 
    * @param lahar
    * @return
    */
   @Override
   public QueryResult executeQuery(final BdbConnection    bdb,
                                   final LaharConnection  lahar, 
                                   final StreamKeyManager keyManager) throws IOException,
                                                                             ClassNotFoundException,
                                                                             SQLException {
      QueryResult result = null;
      StreamKey   sk     = this.getStreamKey(keyManager);
      
      if(lahar.streamKeyExists(sk, this.bitmaskSchema)){
         result = lahar.loadCachedResult(sk, this.bitmaskSchema);
      } else { 
         // 1) load all the At MStreams
         List<List<ITimestep>> streams = new ArrayList<List<ITimestep>>();
         
         for(AtEvent atEvent : this.atStreams){
            StreamKey       atSk     = atEvent.getStreamKey(keyManager);
            List<ITimestep> stream = lahar.loadMStream(atSk, this.particleFilterSchema);
            streams.add(stream);
         }
         
         // 2) Create the "Near" stream by merging, storing and re-querying
         if(0 < streams.size()){
            List<ITimestep> nearStream = ParticleFilterStreamMerger.createNearStream(streams, this.threshold);
                        
            if(bdb.storeMStream(nearStream, sk.getKey(), this.bitmaskSchema, false)){
               lahar.moveStream(sk, this.bitmaskSchema);
               result = lahar.loadCachedResult(sk, this.bitmaskSchema);
            }
         }
      }
      return result;
   }
}
