package edu.washington.cs.rfid.mstreams.event;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;

import markovianStream.ITimestep;
import parser.UnsupportedQueryException;
import api.queryResults.QueryResult;
import edu.washington.cs.rfid.mstreams.data.StreamKey;
import edu.washington.cs.rfid.mstreams.data.db.BdbConnection;
import edu.washington.cs.rfid.mstreams.data.db.LaharConnection;
import edu.washington.cs.rfid.mstreams.data.db.StreamKeyManager;
import edu.washington.cs.rfid.mstreams.util.StreamConverter;

public class OutsideEvent extends ScenicEvent {

   public AtEvent       atStream;
   public List<Integer> nodeList;
   public InsideEvent   insideEvent;
   
   
   /**
    * Creates a new OutsideEvent.
    * @param tag
    * @param nodeList
    */
   public OutsideEvent(final AtEvent atStream, final List<Integer> nodeList){
      super();
      this.atStream    = atStream;
      this.nodeList    = nodeList;
      this.insideEvent = new InsideEvent(atStream,nodeList);
   }
   
   
   /**
    * Gets the stream key for this event.
    */
   @Override
   public String getStreamDescription(){
      StringBuilder desc = new StringBuilder();
      
      desc.append(this.atStream.getStreamDescription());
      desc.append("_OUTSIDE");
      
      Collections.sort(this.nodeList);
      for(Integer node : this.nodeList){
         desc.append("_");
         desc.append(node);
      }
      return desc.toString();
   }
   
   
   /**
    * Executes the event using the given Lahar instance and returns a query result.
    * 
    * @param lahar
    * @return
    */
   @Override
   public QueryResult executeQuery(final BdbConnection    bdb,
                                   final LaharConnection  lahar, 
                                   final StreamKeyManager keyManager) throws IOException,
                                                                             ClassNotFoundException,
                                                                             SQLException,
                                                                             UnsupportedQueryException {
      QueryResult result = null;
      StreamKey   sk     = this.getStreamKey(keyManager);
      
      if(lahar.streamKeyExists(sk, this.bitmaskSchema)){
         result = lahar.loadCachedResult(sk, this.bitmaskSchema);
      } else {
         QueryResult     insideResult    = insideEvent.executeQuery(bdb, lahar, keyManager);
         List<ITimestep> inResultStream  = StreamConverter.convertToMStream(insideResult);
         List<ITimestep> outResultStream = StreamConverter.negateResultStream(inResultStream);         
         
         if(bdb.storeMStream(outResultStream, sk.getKey(), this.bitmaskSchema, false)){
            lahar.moveStream(sk, this.bitmaskSchema);
            
            result = lahar.loadCachedResult(sk, this.bitmaskSchema);
         }         
      }
      return result;
   }
}
