package edu.washington.cs.rfid.mstreams.event;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;

import markovianStream.ITimestep;
import api.queryResults.QueryResult;
import edu.washington.cs.rfid.mstreams.data.StreamKey;
import edu.washington.cs.rfid.mstreams.data.db.BdbConnection;
import edu.washington.cs.rfid.mstreams.data.db.LaharConnection;
import edu.washington.cs.rfid.mstreams.data.db.StreamKeyManager;
import edu.washington.cs.rfid.mstreams.util.StreamConverter;

public class WithoutEvent extends ScenicEvent {
   
   public List<AtEvent> atStreams;
   public WithEvent     withEvent;
   
   //--//
   
   /**
    * Creates a new WithoutEvent with the specified entities.
    * 
    * @param entities
    */
   public WithoutEvent(final List<AtEvent> atStreams){
      super();
      this.atStreams = atStreams;
      this.withEvent = new WithEvent(atStreams);
   }
   
   
   /**   
    * Gets the stream description for this event.
    */
   @Override
   public String getStreamDescription(){
      StringBuilder description = new StringBuilder();
      
      description.append("_WITHOUT");
      
      Collections.sort(this.atStreams);
      for(AtEvent atEvent : this.atStreams){
         description.append("_");
         description.append(atEvent.getStreamDescription());
      }
      return description.toString();
   }
   
   
   /**
    * Executes the event using the given Lahar instance and returns a query result.
    * 
    * @param lahar
    * @return
    */
   @Override
   public QueryResult executeQuery(final BdbConnection    bdb  ,
                                   final LaharConnection  lahar, 
                                   final StreamKeyManager keyManager) throws IOException,
                                                                             ClassNotFoundException,
                                                                             SQLException {
      QueryResult result = null;
      StreamKey   sk     = this.getStreamKey(keyManager);
      
      if(lahar.streamKeyExists(sk, this.bitmaskSchema)){
         result = lahar.loadCachedResult(sk, this.bitmaskSchema);
      } else {
         QueryResult     withResult          = withEvent.executeQuery(bdb, lahar, keyManager);
         List<ITimestep> withResultStream    = StreamConverter.convertToMStream(withResult);
         List<ITimestep> withoutResultStream = StreamConverter.negateResultStream(withResultStream);         
         
         if(bdb.storeMStream(withoutResultStream, sk.getKey(), this.bitmaskSchema, false)){
            lahar.moveStream(sk, this.bitmaskSchema);
            
            result = lahar.loadCachedResult(sk, this.bitmaskSchema);
         }         
      }
      return result;
   }
}