package edu.washington.cs.rfid.mstreams.tools;

import java.io.File;

import storageManager.schema.IStreamSchema;
import accessPaths.IStorageLayer.StorageType;
import api.Lahar;
import api.LaharConfiguration;
import edu.washington.cs.rfid.mstreams.schema.ParticleFilterStreamSchema;
import edu.washington.cs.rfid.mstreams.schema.BitmaskStreamSchema;

public class LaharDbCreator {

	public static String DEFAULT_DB_LOC = "/data/db_loc/RfidDB";
	
	//--//
	
	/**
	 * Prints usage information.
	 */
	public static void usage(){
	   System.out.println("Usage: LaharDbCreator [DIR]\n" +
	                      "Creates a Lahar database which resides under a particular directory, DIR " +
	                      "(or " + LaharDbCreator.DEFAULT_DB_LOC + " by default).");
	}
	
	
	/**
	 * Creates a new Lahar DB for the RFID data.
	 * @param args
	 */
	public static void main(String[] args) {
		
	   if(args.length > 1 || (args.length == 1 && args[0].equalsIgnoreCase("--help"))){
	      LaharDbCreator.usage();
	      System.exit(1);
	   }
	   
	   String dbLocDir = null;
	   
	   if(args.length == 1){
	      dbLocDir = args[0];
	   } else {
	      dbLocDir = LaharDbCreator.DEFAULT_DB_LOC;
	   }
	   
	   
	   System.out.println("Setting database location to " + dbLocDir + "...");
		File dbLoc = new File(dbLocDir);
		
		
		System.out.println("Creating schema objects for particle filter and result streams...");
		IStreamSchema pfSchema     = new ParticleFilterStreamSchema();
		IStreamSchema resultSchema = new BitmaskStreamSchema();

		
		System.out.println("Creating new Lahar database of type BDB...");
		StorageType        storageType = StorageType.BDB;
		LaharConfiguration config      = new LaharConfiguration(dbLoc, storageType);
		Lahar              lahar       = Lahar.createNewDatabase(config);
		
		
		System.out.println("Creating stream tables for schema in Lahar...");
		lahar.createStreamTable(pfSchema    );
		lahar.createStreamTable(resultSchema);

		
		System.out.println("Database and stream tables created successfully.");
	}
}
