package edu.washington.cs.rfid.mstreams.util;

import java.security.MessageDigest;


/**
 * Implements algorithms to hash String keys to longs.
 * 
 * @author Evan
 */
public class KeyHash {
	
	/**
	 * Hashes a String to a 64-bit number.
	 * 
	 * @param k The String key
	 * @param initval An initial value
	 * @return The 64-bit hash
	 */
	public static long hash64(String k, long initval){
		
		byte[] kb = k.getBytes();
		long   a, b, c;
		int    len;
		
		a   = Long.parseLong("7f4a7c13", 16); 
		b   = a;
		c   = initval;
		
		len = kb.length;
		
		if(len < 24){
			byte[] temp = new byte[24];
			
			for(int i = 0; i < temp.length; i++){
				if(i < kb.length){
					temp[i] = kb[i];
				} else {
					temp[i] = 0;
				}
			}
		}
		
		int i = 0;
		
		while (len >= 24){
			a = a + ((long)kb[i] + ((long)kb[i + 1] <<  8) + ((long)kb[i + 2] << 16) + ((long)kb[i + 3] << 24)
			                     + ((long)kb[i + 4] << 32) + ((long)kb[i + 5] << 40) + ((long)kb[i + 6] << 48) 
			                     + ((long)kb[i + 7] << 56)
			         );
			      
			b = b + ((long)kb[i + 8] + ((long)kb[i + 9] <<  8) + ((long)kb[i + 10] << 16) + ((long)kb[i + 11] << 24)
                    			     + ((long)kb[i + 12]<< 32) + ((long)kb[i + 13] << 40) + ((long)kb[i + 14] << 48) 
                    			     + ((long)kb[i + 15]<< 56)
					);
			
			c = c + ((long)kb[i + 16] + ((long)kb[i + 17] <<  8) + ((long)kb[i + 18] << 16) + ((long)kb[i + 19] << 24)
                    			      + ((long)kb[i + 20] << 32) + ((long)kb[i + 21] << 40) + ((long)kb[i + 22] << 48) 
                    			      + ((long)kb[i + 23] << 56)
					);
			
			c = KeyHash.mix(a,b,c);
			
			i   = i   + 24; 
			len = len - 24;
		}
		
		c = c + len;

		switch(len){
		case 23: c = c + ((long)kb[22] << 56);
		case 22: c = c + ((long)kb[21] << 48);
		case 21: c = c + ((long)kb[20] << 40);
		case 20: c = c + ((long)kb[19] << 32);
		case 19: c = c + ((long)kb[18] << 24);
		case 18: c = c + ((long)kb[17] << 16);
		case 17: c = c + ((long)kb[16] << 8);
		
		case 16: b = b + ((long)kb[15] << 56);
		case 15: b = b + ((long)kb[14] << 48);
		case 14: b = b + ((long)kb[13] << 40);
		case 13: b = b + ((long)kb[12] << 32);
		case 12: b = b + ((long)kb[11] << 24);
		case 11: b = b + ((long)kb[10] << 16);
		case 10: b = b + ((long)kb[9]  << 8 );
		case 9 : b = b + ((long)kb[8]       );

		case 8: a = a + ((long)kb[7] << 56);
		case 7: a = a + ((long)kb[6] << 48);
		case 6: a = a + ((long)kb[5] << 40);
		case 5: a = a + ((long)kb[4] << 32);
		case 4: a = a + ((long)kb[3] << 24);
		case 3: a = a + ((long)kb[2] << 16);
		case 2: a = a + ((long)kb[1] << 8 );
		case 1: a = a + ((long)kb[0]      );
		}

		c = KeyHash.mix(a, b, c);
		
		return c;
	}
	

	/**
	 * The mixing part of the hash function.
	 * 
	 * @param a Internal state
	 * @param b Internal state
	 * @param c The key / internal state
	 * @return The result of mixing
	 */
	private static long mix(long a, long b, long c){

	  a = a - b;  a = a - c;  a = a^(c >> 43);
	  b = b - c;  b = b - a;  b = b^(a << 9 );
	  c = c - a;  c = c - b;  c = c^(b >> 8 );
	  a = a - b;  a = a - c;  a = a^(c >> 38);
	  b = b - c;  b = b - a;  b = b^(a << 23);
	  c = c - a;  c = c - b;  c = c^(b >> 5 );
	  a = a - b;  a = a - c;  a = a^(c >> 35);
	  b = b - c;  b = b - a;  b = b^(a << 49);
	  c = c - a;  c = c - b;  c = c^(b >> 11);
	  a = a - b;  a = a - c;  a = a^(c >> 12);
	  b = b - c;  b = b - a;  b = b^(a << 18);
	  c = c - a;  c = c - b;  c = c^(b >> 22);

	  return c;
	}
	
	
	/**
	 * Returns the long result of the first 64 bits of an MD5 hash
	 * XOR'd with the last 64 bits.
	 *  
	 * @param k The key to hash
	 * @return The XOR'd hash as a long
	 */
	public static long hashMD5(String k){
	
		long   hash;
		
		byte[] kb     = k.getBytes();
		byte[] hash64 = new byte[8];
		
		try{
		     MessageDigest md5 = MessageDigest.getInstance("MD5");
		     
		     md5.reset (  );
		     md5.update(kb);
		     
		     byte[] hash128 = md5.digest();
		     hash64  = new byte[8];
		     
		     for(int i = 0, j = hash128.length - 1; i < hash64.length; i++, j--){
		    	 hash64[i] = (byte) ((hash128[j]) ^ (hash64[i]));
		     }
		     
		} catch(Exception e){}
		
		hash = ((long)hash64[0]) + ((long)hash64[1] << 8 ) + ((long)hash64[2] << 16) + ((long)hash64[3] << 24)
								 + ((long)hash64[4] << 32) + ((long)hash64[5] << 40) + ((long)hash64[6] << 48)
								 + ((long)hash64[7] << 56 );
		
		return hash;
	}	
}