package edu.washington.cs.rfid.mstreams.util;

import java.util.HashMap;
import java.util.Map;

import markovianStream.CPT;
import markovianStream.ITimestep;
import markovianStream.Timestep_Imprecise;
import storageManager.schema.ITuple;
import edu.washington.cs.rfid.mstreams.data.ScenicEventExtraData;

public class MStreamCreatorUtil {
      
   /**
    * Gets a new ITimestep given the index, timestamp, marginal and CPT.
    * 
    * @param index
    * @param timestamp
    * @param marginalMap
    * @param cptMap
    * @return
    */
   public static ITimestep getITimestep(int                            index      , 
                                        long                           timestamp  , 
                                        Map<ITuple,Double>             marginalMap, 
                                        Map<ITuple,Map<ITuple,Double>> cptMap){
      ITimestep            ts    = null;
      CPT                  cpt   = new CPT(cptMap);
      ScenicEventExtraData extra = new ScenicEventExtraData(timestamp);
      
      ts = new Timestep_Imprecise(index, marginalMap, cpt);
      
      ts.setExtraData(extra);
      
      return ts;
   }
   
   
   /**
    * Helper method for converting a query result into a result stream.
    * Generates a default CPT to follow a given marginal.
    * @param marginal
    * @return
    */
   public static Map<ITuple, Map<ITuple, Double>> getDefaultCPT(Map<ITuple, Double> marginal){
      Map<ITuple, Map<ITuple, Double>> cpt     = new HashMap<ITuple, Map<ITuple, Double>>();
      int                              numKeys = marginal.keySet().size();
      ITuple[]                         keys    = marginal.keySet().toArray(new ITuple[numKeys]);
      double                           keyProb = 1.0 / (double)numKeys;
      
      for(int i = 0; i < keys.length; i++){
         HashMap<ITuple,Double> map = new HashMap<ITuple,Double>();
         ITuple tuple1 = keys[i];
         for(int j = 0; j < keys.length; j++){
            ITuple tuple2 = keys[j];
            map.put(tuple2, keyProb);
         }
         cpt.put(tuple1, map);
      }
      return cpt;
   }
   
   
   /**
    * Helper method for converting a query result into a result stream.
    * Generates a default CPT for the transition between two consecutive marginals.
    * @param marginal1
    * @param marginal2
    * @return
    */
   public static Map<ITuple, Map<ITuple, Double>> getDefaultCPT(Map<ITuple, Double> marginal1, 
                                                                Map<ITuple, Double> marginal2){
      Map<ITuple, Map<ITuple, Double>> cpt = new HashMap<ITuple, Map<ITuple, Double>>();
      
      for(ITuple tuple1 : marginal1.keySet()){
         //HashMap<ITuple,Double> map = marginal2.clone(); //TODO: Can we just clone the second marginal here??
         HashMap<ITuple,Double> map = new HashMap<ITuple,Double>();
         for(ITuple tuple2 : marginal2.keySet()){
            map.put(tuple2, marginal2.get(tuple2));
         }
         cpt.put(tuple1, map);
      }
      return cpt;
   }
}
