package edu.washington.cs.rfid.mstreams.util;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import markovianStream.CPT;
import markovianStream.ITimestep;
import storageManager.schema.ITuple;
import edu.washington.cs.rfid.mstreams.data.ParticleFilterStream;
import edu.washington.cs.rfid.mstreams.data.ParticleFilterStreamCPTData;
import edu.washington.cs.rfid.mstreams.data.ParticleFilterStreamMarginalData;
import edu.washington.cs.rfid.mstreams.data.ScenicEventExtraData;
import edu.washington.cs.rfid.mstreams.exception.MStreamSyntaxException;

public class ParticleFilterStreamParser {
   
   public static ParticleFilterStream parseSmoothedPF(String filename) throws IOException, 
                                                                              MStreamSyntaxException {

      ParticleFilterStream file   = null;
      BufferedReader       reader = null;

      try {
         reader                                    = new BufferedReader(new FileReader(filename));
         String                           line     = null;
         file                                      = new ParticleFilterStream();
         ParticleFilterStreamMarginalData marginal = null;
         ParticleFilterStreamCPTData      cpt      = null;
         boolean                parsingMarginal = true;

         while(null != (line = reader.readLine())){
            if(parsingMarginal){
               marginal = new ParticleFilterStreamMarginalData(line);
            } else {
               cpt = new ParticleFilterStreamCPTData(line);
               file.addPair(marginal, cpt);
            }
            parsingMarginal = !parsingMarginal;
         }
      } catch(FileNotFoundException e){
         e.printStackTrace();
      } catch(IOException e){
         e.printStackTrace();
      } finally {
         try {
            if(null != reader){
               reader.close();
            }
         } catch(IOException e){
            e.printStackTrace();
         }
      }
      return file;
    }
    
    
    public static ParticleFilterStream parseTimestepList(List<ITimestep> list){
       ParticleFilterStream file = new ParticleFilterStream();

       for(ITimestep ts : list){
          Map<ITuple,Double>             marginalMap = ts.getMarginal();
          CPT                            cpt         = ts.getCPT();
          Map<ITuple,Map<ITuple,Double>> cptMap      = cpt.getValues();
          ScenicEventExtraData           extra       = (ScenicEventExtraData)ts.getExtraData();

          ParticleFilterStreamMarginalData marginal = new ParticleFilterStreamMarginalData(extra.timestamp,-1,marginalMap);
          ParticleFilterStreamCPTData      cptData  = new ParticleFilterStreamCPTData(-1,extra.timestamp,extra.timestamp + 1000L,cptMap);

          file.addPair(marginal, cptData);
       }
       return file;
    }
}
