package edu.washington.cs.rfid.cascadia.playback.client.ui;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.playback.client.data.PlaybackOptions;
import edu.washington.cs.rfid.cascadia.playback.client.event.FastForwardEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.PauseEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.PlayEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.RestartEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.SlowDownEvent;
import edu.washington.cs.rfid.cascadia.playback.client.handler.FastForwardHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.PauseHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.PlayHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.RestartHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.SlowDownHandler;
import edu.washington.cs.rfid.map.client.data.LocationData;

/**
 * A panel that holds zero or more PlaybackMap panels for playing back
 * a trace using multiple views.
 * @author evan
 */
public class PlaybackPanel extends VerticalPanel {

   protected static final String DEFAULT_ICON_URL     = "http://rfid.cs.washington.edu/map-icons";
   
   protected static final String STYLE_PLAYBACK_PANEL = "playback-panel";
   protected static final String DEFAULT_SCROLL_WIDTH = "950px";//"420px";
   protected static final String DEFAULT_WIDTH        = "920px";//"400px";
   protected static final String DEFAULT_HEIGHT       = "750px";
   protected static final String DEFAULT_ROW_WIDTH    = "920px";

   //--//
   
   // Title label for this PlaybackPanel
   protected HTML                     playbackLabel;
   
   // Playback information and control panel
   protected VerticalPanel            playbackControlPanel;
   protected HTML                     playbackTimeLabel;
   protected HorizontalPanel          legendPanel;
   protected HorizontalPanel          playbackControlStrip;
   protected Button                   restartButton;
   protected Button                   rewindButton;
   protected Button                   playButton;
   protected Button                   pauseButton;
   protected Button                   slowDownButton;
   protected Button                   ffwdButton;
   
   protected HTML                     playbackSpeedLabel;
   
   // Handlers for the playback events
   protected List<SlowDownHandler>    slowDownHandlers;
   protected List<FastForwardHandler> fastForwardHandlers;
   protected List<PauseHandler>       pauseHandlers;
   protected List<PlayHandler>        playHandlers;
   protected List<RestartHandler>     restartHandlers;
   
   // Widgets and information for scroll panel containing PlaybackMaps
   protected ScrollPanel              scrollPanel;
   protected VerticalPanel            mapContainer;
   protected List<PlaybackMap>        playbackMaps;
   protected List<PlaybackOptions>    playbackOptions;

   // TODO: woye, get rid of this...
   // HACK HACK HACK
//   protected PlaybackMap jamesNotesWidget = null;
   // HACK HACK HACK
   
   //--//
   
   /**
    * Creates the PlaybackPanel, initially with no PlaybackMaps.
    */
   public PlaybackPanel(){
      
      this.playbackLabel = new HTML("Playback");
      this.playbackLabel.setStyleName("gwt-HTML-big-label");
      
      this.initPlaybackControls();
      
      this.scrollPanel  = new ScrollPanel();
      this.mapContainer = new VerticalPanel();
      this.mapContainer.setWidth(PlaybackPanel.DEFAULT_WIDTH);
      this.mapContainer.setHorizontalAlignment(HorizontalPanel.ALIGN_CENTER);
      this.mapContainer.setStyleName(PlaybackPanel.STYLE_PLAYBACK_PANEL);
      this.initWidgetContainer();
      
      this.playbackMaps    = new ArrayList<PlaybackMap>();
      this.playbackOptions = new ArrayList<PlaybackOptions>();
      this.scrollPanel.setWidget(this.mapContainer);
      this.scrollPanel.setSize(PlaybackPanel.DEFAULT_SCROLL_WIDTH, 
                               PlaybackPanel.DEFAULT_HEIGHT);
      
      this.setHorizontalAlignment(VerticalPanel.ALIGN_CENTER);
      this.add(this.playbackLabel);
      this.add(this.playbackControlPanel);
      this.setCellHorizontalAlignment(this.playbackControlPanel, VerticalPanel.ALIGN_CENTER);
      this.add(this.scrollPanel  );
   }
   
   
   /**
    * Generates the PlaybackMaps using a list of PlaybackOptions. 
    * @param playbackOptions
    */
   public void generatePlaybackWidgets(List<PlaybackOptions> playbackOptions){

      this.playbackOptions = playbackOptions;
      this.legendPanel.clear();
      this.mapContainer.clear();
      this.playbackMaps.clear();
      
      this.legendPanel.add(new HTML("Now Showing:&nbsp;&nbsp;"));
      
      // Step through the PlaybackOptions list and create each PlaybackMap accordingly
      for(int i = 0; i < this.playbackOptions.size(); i++){
         PlaybackOptions a = this.playbackOptions.get(i);
         
         // Create the legend
         this.legendPanel.add(new Image(PlaybackPanel.DEFAULT_ICON_URL + "/" + a.trackedEntityImgUrl));
         this.legendPanel.add(new HTML(a.trackedEntityName)); //a.traceName));
         this.legendPanel.add(new HTML("&nbsp;&nbsp;&nbsp;"));
         
         if(i + 1 < this.playbackOptions.size()){
            PlaybackOptions b = this.playbackOptions.get(i + 1);
            
            // HACKHACKHACK below
//            if(b.traceName.equals("James' Notes")){
//               
//               this.legendPanel.add(new HTML("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Also available at this time:</b>&nbsp;&nbsp;"));
//               
//               CheckBox cb = new CheckBox();
//               cb.addValueChangeHandler(new ValueChangeHandler<Boolean>(){
//                  public void onValueChange(ValueChangeEvent<Boolean> e){
//                     if(e.getValue()){
//                        if(null != jamesNotesWidget){
//                           jamesNotesWidget.setVisible(true);
//                        } else {
//                           jamesNotesWidget.setVisible(false);
//                        }
//                     }
//                  }
//               });
//               this.legendPanel.add(cb);
//               this.legendPanel.add(new Image(PlaybackPanel.DEFAULT_ICON_URL + "/" + b.trackedEntityImgUrl));
//               this.legendPanel.add(new HTML(b.traceName));
//               
//               this.widgetContainer.add(this.generateDoublePlaybackRow(a,b,playbackOptions));
//               i++;
//               
//            } else {
            
               this.legendPanel.add(new Image(PlaybackPanel.DEFAULT_ICON_URL + "/" + b.trackedEntityImgUrl));
               this.legendPanel.add(new HTML(b.trackedEntityName));
               this.legendPanel.add(new HTML("&nbsp;&nbsp;&nbsp;"));
               
               this.mapContainer.add(this.generateDoublePlaybackRow(a,b,playbackOptions));
               i++;
//            }
         } else {
            this.mapContainer.add(this.generateSinglePlaybackRow(a,playbackOptions));
         }
         this.mapContainer.add(this.generateSpacerRow());
      }
   }
   
   
   /**
    * Updates the time label at the top of the playback panel.
    * @param time
    */
   public void updateTimeLabel(String time){
      this.playbackTimeLabel.setHTML("( <b>"+time+"</b> )");
   }
   
   
   /**
    * Updates the maps with the next batch of location data.
    * @param locationMap
    */
   public void updateLocation(Map<String,LocationData> locationMap){
      for(PlaybackMap w : this.playbackMaps){
         w.updateLocation(locationMap);
      }
   }
   
   
   public void setPlaybackSpeedLabel(String html){
      this.playbackSpeedLabel.setHTML(html);
   }
   
   
   public void clearAllMaps(){
      if(null != this.playbackMaps){
         for(PlaybackMap map : this.playbackMaps){
            map.clearOverlays();
         }
      }
   }
   
   
   public void addSlowDownHandler(SlowDownHandler h){
      this.slowDownHandlers.add(h);
   }
   
   
   public void addFastForwardHandler(FastForwardHandler h){
      this.fastForwardHandlers.add(h);
   }
   
   
   public void addPauseHandler(PauseHandler h){
      this.pauseHandlers.add(h);
   }

   
   public void addPlayHandler(PlayHandler h){
      this.playHandlers.add(h);
   }
   

   public void addRestartHandler(RestartHandler h){
      this.restartHandlers.add(h);
   }
   
   
   public void removeSlowDownHandler(SlowDownHandler h){
      this.slowDownHandlers.remove(h);
   }
   
   
   public void removeFastForwardHandler(FastForwardHandler h){
      this.fastForwardHandlers.remove(h);
   }
   
   
   public void removePauseHandler(PauseHandler h){
      this.pauseHandlers.remove(h);
   }

   
   public void removePlayHandler(PlayHandler h){
      this.playHandlers.remove(h);
   }
   

   public void removeRestartHandler(RestartHandler h){
      this.restartHandlers.remove(h);
   }
   
   
   //--//
   
   
   /**
    * Initializes the widget container panel.
    */
   private void initWidgetContainer(){
      this.mapContainer.add(new HTML("<br><br><br><br><br><br>No Playback Cued Yet<br><br><br><br><br><br>"));
   }
   
   
   /**
    * Gets the playback options for a particular  
    * @param key
    * @return
    */
   private PlaybackOptions getOptions(String key){
      PlaybackOptions options = null;
      
      for(PlaybackOptions o : this.playbackOptions){
         if(o.followedKey.equals(key)){
            options = o;
         }
      }
      return options;
   }
   
   
   /**
    * Initialize the playback controls.
    */
   protected void initPlaybackControls(){
      
      this.playbackControlPanel = new VerticalPanel();
      this.playbackTimeLabel    = new HTML("<br>");
      this.legendPanel          = new HorizontalPanel();
      this.playbackControlStrip = new HorizontalPanel();
      this.restartButton        = new Button("Restart");
      this.rewindButton         = new Button("Rewind");
      this.playButton           = new Button("Play");
      this.pauseButton          = new Button("Pause");
      this.slowDownButton         = new Button("Slower");
      this.ffwdButton           = new Button("Faster");
      
      // Init legend
      this.playbackControlStrip.add(this.restartButton);
      this.playbackControlStrip.add(new HTML("&nbsp;&nbsp;"));
      this.playbackControlStrip.add(this.rewindButton);
      this.playbackControlStrip.add(new HTML("&nbsp;&nbsp;"));
      this.playbackControlStrip.add(this.playButton);
      this.playbackControlStrip.add(new HTML("&nbsp;&nbsp;"));
      this.playbackControlStrip.add(this.pauseButton);
      this.playbackControlStrip.add(new HTML("&nbsp;&nbsp;"));
      this.playbackControlStrip.add(this.slowDownButton);
      this.playbackControlStrip.add(new HTML("&nbsp;&nbsp;"));
      this.playbackControlStrip.add(this.ffwdButton);
      this.playbackSpeedLabel   = new HTML("( stopped )");
      
      this.playbackControlPanel.setHorizontalAlignment(VerticalPanel.ALIGN_CENTER);
      this.playbackControlPanel.add(this.playbackTimeLabel);
      this.playbackControlPanel.add(this.legendPanel);
      this.playbackControlPanel.add(this.playbackControlStrip);
      this.playbackControlPanel.add(this.playbackSpeedLabel);
      
      this.slowDownHandlers    = new ArrayList<SlowDownHandler>();
      this.fastForwardHandlers = new ArrayList<FastForwardHandler>();
      this.pauseHandlers       = new ArrayList<PauseHandler>();
      this.playHandlers        = new ArrayList<PlayHandler>();
      this.restartHandlers     = new ArrayList<RestartHandler>();
      
      this.restartButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            // Fire all restart handlers
            RestartEvent re = new RestartEvent();
            for(RestartHandler handler : restartHandlers){
               handler.onRestart(re);
            }
         }
      });
      
      this.playButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            // Fire all play handlers
            PlayEvent pe = new PlayEvent();
            for(PlayHandler handler : playHandlers){
               handler.onPlay(pe);
            }
         }
      });
      
      this.pauseButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            // Fire all pause handlers
            PauseEvent pe = new PauseEvent();
            for(PauseHandler handler : pauseHandlers){
               handler.onPause(pe);
            }
         }
      });
      
      this.slowDownButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            // Fire all slow down handlers
            SlowDownEvent se = new SlowDownEvent();
            for(SlowDownHandler handler : slowDownHandlers){
               handler.onSlowDown(se);
            }
         }
      });
      
      this.ffwdButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            // Fire all fast forward handlers
            FastForwardEvent fe = new FastForwardEvent();
            for(FastForwardHandler handler : fastForwardHandlers){
               handler.onFastForward(fe);
            }
         }
      });
   }
   
   
   /**
    * Creates a new row in the PlaybackPanel containing 2 PlaybackMaps initialized with the
    * given PlaybackOptions.
    * @param a PlaybackOptions for the first PlaybackMap
    * @param b PlaybackOptions for the second PlaybackMap
    * @param playbackOptions The list of all PlaybackOptions
    * @return
    */
   protected HorizontalPanel generateDoublePlaybackRow(PlaybackOptions       a, 
                                                       PlaybackOptions       b,
                                                       List<PlaybackOptions> playbackOptions){
      HorizontalPanel row = new HorizontalPanel();
      PlaybackMap     wa  = new PlaybackMap(a,playbackOptions);
      PlaybackMap     wb  = new PlaybackMap(b,playbackOptions);
      this.playbackMaps.add(wa);
      this.playbackMaps.add(wb);
      row.setHorizontalAlignment(HorizontalPanel.ALIGN_CENTER);
      row.setWidth(PlaybackPanel.DEFAULT_ROW_WIDTH);
      row.add(wa);
      row.add(wb);
      row.setCellHorizontalAlignment(wa,HorizontalPanel.ALIGN_LEFT );
      row.setCellHorizontalAlignment(wb,HorizontalPanel.ALIGN_RIGHT);

      // HACKHACKHACK
//      if(b.traceName.equals("James' Notes")){
//         this.jamesNotesWidget = wb;
//         this.jamesNotesWidget.setVisible(false);
//      }
      
      return row;
   }
   
   
   /**
    * Creates a new row in the PlaybackPanel containing a PlaybackMap initialized with the
    * given PlaybackOption.
    * @param a PlaybackOptions for the PlaybackMap 
    * @param playbackOptions The list of all PlaybackOptions
    * @return
    */
   protected HorizontalPanel generateSinglePlaybackRow(PlaybackOptions       a,
                                                       List<PlaybackOptions> playbackOptions){
      HorizontalPanel row = new HorizontalPanel();
      PlaybackMap     wa  = new PlaybackMap(a,playbackOptions);
      this.playbackMaps.add(wa);
      row.setHorizontalAlignment(HorizontalPanel.ALIGN_CENTER);
      row.add(wa);
      
      return row;
   }
   
   
   /**
    * Generates a HorizontalPanel as a spacer.
    * @return
    */
   protected HorizontalPanel generateSpacerRow(){
      HorizontalPanel spacer = new HorizontalPanel();
      spacer.add(new HTML("&nbsp;"));
      return spacer;
   }
}
