package edu.washington.cs.rfid.cascadia.scenic.client.grammar;

import java.util.ArrayList;

import com.google.gwt.user.client.rpc.IsSerializable;

import edu.washington.cs.rfid.cascadia.scenic.client.xml.Node;

public class ScenicConnector implements IsSerializable {
   
   protected static final String SCENIC_ACTOR      = "scenicactor";
   protected static final String SCENIC_PERSON     = "scenicperson";
   protected static final String SCENIC_PERSON_GRP = "scenicpersongrp";
   protected static final String SCENIC_THING      = "scenicthing";
   protected static final String SCENIC_THING_GRP  = "scenicthinggrp";
   protected static final String SCENIC_PLACE      = "scenicplace";
   
   protected static final String SCENIC_PRIMITIVE  = "scenicprimitive";
   protected static final String SCENIC_WITH       = "scenicwith";
   protected static final String SCENIC_WITHOUT    = "scenicwithout";
   protected static final String SCENIC_NEAR       = "scenicnear";
   protected static final String SCENIC_FAR        = "scenicfar";
   protected static final String SCENIC_INSIDE     = "scenicinside";
   protected static final String SCENIC_OUTSIDE    = "scenicoutside";
   protected static final String SCENIC_LASTS      = "sceniclasts";
   
   protected final static String ACTOR_X           = "actor-x";
   protected final static String ACTOR_Y           = "actor-y";
   protected final static String PRIMITIVE_X       = "primitive-x";
   protected final static String PRIMITIVE_Y       = "primitive-y";
   
   
   //--//
   
   
   // Actor end of the connector
   protected ScenicActor     actor;
   protected int             actorX;
   protected int             actorY;
   
   // Primitive end of the connector
   protected ScenicPrimitive primitive;
   protected int             primitiveX;
   protected int             primitiveY;
   
   //--//
   
   /**
    * Creates a default ScenicConnector.
    */
   public ScenicConnector(){
      this.actor      = null;
      this.actorX     = -1;
      this.actorY     = -1;
      this.primitive  = null;
      this.primitiveX = -1;
      this.primitiveY = -1;
   }
   
   
   public ScenicConnector(ArrayList<Node> kids){
 
      for(int i = 0; i < kids.size(); i++){        
         Node   n    = kids.get(i);
         String name = n.getName();
         
         if(name.equals(ScenicConnector.SCENIC_ACTOR)){
            this.actor = new ScenicActor(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_PERSON)){
            this.actor = new ScenicPerson(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_PERSON_GRP)){
            this.actor = new ScenicPersonGrp(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_THING)){
            this.actor = new ScenicThing(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_THING_GRP)){
            this.actor = new ScenicThingGrp(n.getChildNodes());
         } else if(name.equals(ScenicSequence.SCENIC_PLACE)){
            this.actor = new ScenicPlace(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_PRIMITIVE)){
            this.primitive = new ScenicPrimitive(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_WITH)){
            this.primitive = new ScenicWith(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_WITHOUT)){
            this.primitive = new ScenicWithout(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_NEAR)){
            this.primitive = new ScenicNear(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_FAR)){
            this.primitive = new ScenicFar(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_INSIDE)){
            this.primitive = new ScenicInside(n.getChildNodes());
         } else if(name.equals(ScenicConnector.SCENIC_OUTSIDE)){
            this.primitive = new ScenicOutside(n.getChildNodes());
         } else if(name.equals(ScenicConnector.ACTOR_X)){
             this.actorX = Integer.parseInt(((Node)n.getChildNodes().get(0)).getValue());
         } else if(name.equals(ScenicConnector.ACTOR_Y)){
            this.actorY = Integer.parseInt(((Node)n.getChildNodes().get(0)).getValue());
         } else if(name.equals(ScenicConnector.PRIMITIVE_X)){
            this.primitiveX = Integer.parseInt(((Node)n.getChildNodes().get(0)).getValue());
         } else if(name.equals(ScenicConnector.PRIMITIVE_Y)){
            this.primitiveY = Integer.parseInt(((Node)n.getChildNodes().get(0)).getValue());
         }
      }
   }
   
   
   public ScenicConnector(ScenicActor     actor, 
                          int             actorX,
                          int             actorY,
                          ScenicPrimitive primitive,
                          int             primitiveX,
                          int             primitiveY){
      this.actor      = actor;
      this.actorX     = actorX;
      this.actorY     = actorY;
      this.primitive  = primitive;
      this.primitiveX = primitiveX;
      this.primitiveY = primitiveY;
   }
   
   
   public ScenicActor getActor(){
      return this.actor;
   }
   
   public int getActorX(){
      return this.actorX;
   }
   
   public int getActorY(){
      return this.actorY;
   }
   
   public ScenicPrimitive getPrimitive(){
      return this.primitive;
   }
   
   public int getPrimitiveX(){
      return this.primitiveX;
   }
   
   public int getPrimitiveY(){
      return this.primitiveY;
   }
   
   public void setActor(ScenicActor actor){
      this.actor = actor;
   }
   
   public void setActorX(int actorX){
      this.actorX = actorX;
   }
   
   public void setActorY(int actorY){
      this.actorY = actorY;
   }
   
   public void setPrimitive(ScenicPrimitive primitive){
      this.primitive = primitive;
   }
   
   public void setPrimitiveX(int primitiveX){
      this.primitiveX = primitiveX;
   }
   
   public void setPrimitiveY(int primitiveY){
      this.primitiveY = primitiveY;
   }
   
   public String toXml(){
      return this.toXml(0);
  }
  
  public String toXml(int level){
      
      String indent = "";
      
      for(int i = 0; i < level; i++){
         indent += "\t";
      }
      
      String xml = indent + "<scenicconnector>\n";
      
      if(null != this.actor){
         xml += this.actor.toXml(level + 1);
      }
      xml += indent + "\t<actor-x>" + this.actorX + "</actor-x>\n";
      xml += indent + "\t<actor-y>" + this.actorY + "</actor-y>\n";

      if(null != this.primitive){
         xml += this.primitive.toXml(level + 1);
      }
      xml += indent + "\t<primitive-x>" + this.primitiveX + "</primitive-x>\n"; 
      xml += indent + "\t<primitive-y>" + this.primitiveY + "</primitive-y>\n";      
      xml += indent + "</scenicconnector>\n";
      
      return xml;
  }
}
