package edu.washington.cs.rfid.cascadia.scenic.client.grammar;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.user.client.rpc.IsSerializable;

import edu.washington.cs.rfid.cascadia.scenic.client.xml.Node;

public class ScenicScene implements IsSerializable {

   protected static final String SCENIC_ACTOR      = "scenicactor";
   protected static final String SCENIC_PERSON     = "scenicperson";
   protected static final String SCENIC_PERSON_GRP = "scenicpersongrp";
   protected static final String SCENIC_THING      = "scenicthing";
   protected static final String SCENIC_THING_GRP  = "scenicthinggrp";
   protected static final String SCENIC_PLACE      = "scenicplace";
   
   protected static final String SCENIC_PRIMITIVE  = "scenicprimitive";
   protected static final String SCENIC_WITH       = "scenicwith";
   protected static final String SCENIC_WITHOUT    = "scenicwithout";
   protected static final String SCENIC_NEAR       = "scenicnear";
   protected static final String SCENIC_FAR        = "scenicfar";
   protected static final String SCENIC_INSIDE     = "scenicinside";
   protected static final String SCENIC_OUTSIDE    = "scenicoutside";
   protected static final String SCENIC_LASTS      = "sceniclasts";
   
   protected static final String SCENIC_CONNECTOR  = "scenicconnector";
   
   protected static final String DURATION          = "duration";
    
   //--//

   protected static final long DEFAULT_DURATION_MS = 0;
   
   //--//
   
   protected long                    duration;
   protected Vector<ScenicActor>     actors;
   protected Vector<ScenicPrimitive> primitives;
//   protected Vector<ScenicConnector> connectors;
   
   //--//
   
   public ScenicScene(){
      this.duration   = ScenicScene.DEFAULT_DURATION_MS;
      this.actors     = new Vector<ScenicActor>();
      this.primitives = new Vector<ScenicPrimitive>();
//      this.connectors = new Vector<ScenicConnector>();
   }
   
   public ScenicScene(ArrayList<Node> kids){

      this.actors     = new Vector<ScenicActor>();
      this.primitives = new Vector<ScenicPrimitive>();
       
      for(int i = 0; i < kids.size(); i++){
        
         Node   n    = kids.get(i);
         String name = n.getName();
         
         if(name.equals(ScenicScene.SCENIC_ACTOR)){
            this.actors.add(new ScenicActor(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_PERSON)){
             this.actors.add(new ScenicPerson(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_PERSON_GRP)){
             this.actors.add(new ScenicPersonGrp(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_THING)){
             this.actors.add(new ScenicThing(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_THING_GRP)){
             this.actors.add(new ScenicThingGrp(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_PLACE)){
             this.actors.add(new ScenicPlace(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_PRIMITIVE)){
            this.primitives.add(new ScenicPrimitive(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_WITH)){
            this.primitives.add(new ScenicWith(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_WITHOUT)){
            this.primitives.add(new ScenicWithout(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_NEAR)){
            this.primitives.add(new ScenicNear(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_FAR)){
            this.primitives.add(new ScenicFar(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_INSIDE)){
            this.primitives.add(new ScenicInside(n.getChildNodes()));
         } else if(name.equals(ScenicScene.SCENIC_OUTSIDE)){
            this.primitives.add(new ScenicOutside(n.getChildNodes()));
//         } else if(name.equals(ScenicScene.SCENIC_CONNECTOR)){
//            this.connectors.add(new ScenicConnector(n.getChildNodes()));
         } else if(name.equals(ScenicScene.DURATION)){
             this.duration = Long.parseLong(((Node)n.getChildNodes().get(0)).getValue());
         }
      }
   }
   
   public ScenicScene(long                    duration  , 
                      Vector<ScenicActor>     actors    , 
                      Vector<ScenicPrimitive> primitives){
      this.duration   = duration;
      this.actors     = actors;
      this.primitives = primitives;
   }
   
   public long getDuration(){
      return this.duration;
   }
   
   public Vector<ScenicActor> getActors(){
      return this.actors;
   }
   
   public Vector<ScenicPrimitive> getPrimitives(){
      return this.primitives;
   }
   
//   public Vector<ScenicConnector> getConnectors(){
//      return this.connectors;
//   }
   
   public void setDuration(long duration){
      this.duration = duration;
   }
   
   public void setActors(Vector<ScenicActor> actors){
      this.actors = actors;
   }
   
   public void setPrimitives(Vector<ScenicPrimitive> primitives){
      this.primitives = primitives;
   }
   
//   public void setConnectors(Vector<ScenicConnector> connectors){
//      this.connectors = connectors;
//   }
   
   public String toXml(){
       return this.toXml(0);
   }
   
   public String toXml(int level){

      String indent = "";

      for(int i = 0; i < level; i++){
         indent += "\t";
      }

      String xml = indent + "<scenicscene>\n" +
      indent + "\t<duration>" + this.duration + "</duration>\n";

      if(null != this.actors) {
         for(int i = 0; i < this.actors.size(); i++){
            xml += this.actors.get(i).toXml((level + 1));
         }
      }

      if(null != this.primitives){
         for(int i = 0; i < this.primitives.size(); i++){
            xml += this.primitives.get(i).toXml((level + 1));
         }
      }
      
      xml += indent + "</scenicscene>\n";

      return xml;
   }
}