package edu.washington.cs.rfid.cascadia.scenic.client.grammar;

import java.util.ArrayList;
import java.util.Vector;

import edu.washington.cs.rfid.cascadia.scenic.client.exception.InvalidArgumentException;
import edu.washington.cs.rfid.cascadia.scenic.client.xml.Node;

public class ScenicWithout extends ScenicPrimitive {

   public ScenicWithout(){
      super();
   }
   
   public ScenicWithout(ArrayList<Node> kids){
      super(kids);
   }
   
   public ScenicWithout(Vector<ScenicActor> actors, int left, int top) {
      super(actors, left, top);
   }

   /**
    * 
    * @precondition a1 and a2 must be contained by this primitive already
    * @param a1
    * @param a2
    */
   @Override
   public void addRelation(ScenicActor a1, ScenicActor a2) throws InvalidArgumentException {
      int a1Index = this.actors.indexOf(a1);
      int a2Index = this.actors.indexOf(a2);
      
      if(-1 == a1Index || -1 == a2Index || a1 instanceof ScenicPlace || a2 instanceof ScenicPlace){
         throw new InvalidArgumentException();
      }
      
      this.relations[a1Index][a2Index] = true;
      this.relations[a2Index][a1Index] = true;
   }
   
   public String toXml(){
       return this.toXml(0);
   }   
   
   /**
    * Generates an XML representation of this object.
    * @return
    */
   public String toXml(int level){
       
       String indent = "";
       
       for(int i = 0; i < level; i++){
          indent += "\t";
       }
       
       String xml = indent + "<scenicwithout>\n";
       
       if(null != this.actors){
          for(int i = 0; i < this.actors.size(); i++){
             xml += this.actors.get(i).toXml(level + 1);
          }
       }
          
       if(null != this.relations){
          for(int i = 0; i < this.relations.length; i++){
             for(int j = 0; j < this.relations[i].length; j++){
                if(this.relations[i][j]){
                    xml += indent + "\t<relation>" + 
                           this.actors.get(i).getName() + "," + this.actors.get(j).getName() +
                           "</relation>\n";
                }
             }
          }
       }
          
       xml += indent + "\t<left>"   + this.left   + "</left>\n"  + 
              indent + "\t<top>"    + this.top    + "</top>\n"   +
              indent + "</scenicwithout>\n";
       return xml;
   }
}