package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import com.allen_sauer.gwt.dnd.client.PickupDragController;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.IconWidget;

public class IconPanel extends HorizontalPanel {
   
   protected class IconCell extends HorizontalPanel {
      
      public IconCell(IconWidget w){
         
         this.setHorizontalAlignment(HorizontalPanel.ALIGN_CENTER);
         this.setVerticalAlignment  (HorizontalPanel.ALIGN_MIDDLE);
         
         this.add(w);
      }
      
      
      public void add(IconWidget w) {
         super.add(w);
         
         dragController.makeDraggable(w);
         dragController.addDragHandler(w.getDragHandler());      
         
         this.setCellHorizontalAlignment(w, HorizontalPanel.ALIGN_CENTER);
         this.setCellVerticalAlignment  (w, HorizontalPanel.ALIGN_MIDDLE);
      }
      
      
      /**
       * Removed widgets that are instances of {@link IconWidget}
       * and immediately replaces them with an incremented copy of the original.
       * 
       * @param w the widget to remove
       */
      @Override
      public boolean remove(Widget w) {
         
         int index = getWidgetIndex(w);
      
         if (index != -1 && w instanceof IconWidget) {
            ((IconWidget) w).wigdetPickedUp();

            IconWidget clone = ((IconWidget) w).nextWidget();
            this.add(clone);
          }
          return super.remove(w);
      }
   }
   
   //--//
   
   protected static final String DEFAULT_ICON_PANEL_STYLE = "iconPanel";
   protected static final int    DEFAULT_ICON_SPACING     = 10;
   protected static final int    MINIMUM_WIDTH            = 150;
   protected static final int    MINIMUM_HEIGHT           = 70;
   protected static final String DEFAULT_HEIGHT           = "70px";
   
   //--//
   
   protected final PickupDragController  dragController;
   protected final DefaultDropController dropController;

   //--//
   
   public IconPanel(PickupDragController dragController){
      this.dragController = dragController;
      this.dropController = new DefaultDropController( this );
      
      this.dragController.registerDropController( this.dropController );
      
      this.addStyleName          ( IconPanel.DEFAULT_ICON_PANEL_STYLE );
      //this.setSpacing            ( IconPanel.DEFAULT_ICON_SPACING     );
      this.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER  ); // WAS LEFT
      this.setVerticalAlignment  ( HasVerticalAlignment.ALIGN_MIDDLE  );

      this.setHeight(IconPanel.DEFAULT_HEIGHT);
   }
   
   
   /**
    * Overloaded method that makes widgets draggable.
    * 
    * @param w the widget to be added are made draggable
    */
   public void add(IconWidget w) {
      IconCell cell = new IconCell(w);
      
      this.add(cell);
      
      if(IconPanel.MINIMUM_WIDTH > w.getOffsetWidth()){
         this.setCellWidth(cell, IconPanel.MINIMUM_WIDTH + "px");
      } 
      if(IconPanel.MINIMUM_HEIGHT > w.getOffsetHeight()){
         this.setCellHeight(cell, IconPanel.MINIMUM_HEIGHT + "px");
      }
      this.setCellHorizontalAlignment(cell, HorizontalPanel.ALIGN_CENTER);
      this.setCellVerticalAlignment  (cell, HorizontalPanel.ALIGN_MIDDLE);
   }
   
   
   /**
    * Sets the size of this IconPanel.
    * 
    * @param width
    * @param height
    */
   public void setSize(String width, String height){
      this.setWidth (width );
      this.setHeight(height);
   }
}