package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.util.ImageUtility;


public class LabeledIcon extends VerticalPanel {

   protected static final String LABEL_STYLE    = "iconLabel";

   protected static final int    DEFAULT_MAX_WIDTH_IMG  = 60;
   protected static final int    DEFAULT_MAX_HEIGHT_IMG = 50;
   
   //--//
   
   public AbsolutePanel absPanel;
   public Image         img;
   public Image         defaultImg;
   public HTML          label;
   public int           maxWidthImg;
   public int           maxHeightImg;
   
   //--//
   
   public LabeledIcon(Image   img, 
                      Image   defaultImg,
                      int     maxWidthImg,
                      int     maxHeightImg,
                      HTML    label, 
                      boolean useDefault){
      this.maxWidthImg  = maxWidthImg;
      this.maxHeightImg = maxHeightImg;
      
      this.absPanel = new AbsolutePanel();
      this.absPanel.setWidth (this.maxWidthImg  + "px");
      this.absPanel.setHeight(this.maxHeightImg + "px");
      
      this.img        = img;
      this.defaultImg = defaultImg;
      this.label      = label;
      
      this.label.setStyleName( LabeledIcon.LABEL_STYLE );
      this.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      
      this.add(this.absPanel);
      this.add( this.label  );
      
      if(useDefault){
         this.rescaleImage(this.defaultImg);
      } else {
         this.rescaleImage(this.img);
      }
      this.setCellHorizontalAlignment(this.absPanel, VerticalPanel.ALIGN_CENTER);
      this.setCellHorizontalAlignment(this.label   , VerticalPanel.ALIGN_CENTER);
   }
   
   
   public LabeledIcon(Image   img, 
                      Image   defaultImg,
                      HTML    label, 
                      boolean useDefault){
      this(img,
           defaultImg,
           LabeledIcon.DEFAULT_MAX_WIDTH_IMG,
           LabeledIcon.DEFAULT_MAX_HEIGHT_IMG,
           label,
           useDefault);
   }
   
   
   
   public Image getImage(){
      return this.img;
   }
   
   public Label getLabel(){
      return this.label;
   }

   public boolean getLabelVisible(){
      return this.label.isVisible();
   }   
   
   public void setImage(Image img){
      this.img = img;
   }
   
   public void setLabel(String labelHtml){
      this.label.setHTML(labelHtml);
   }
   
   public void setLabelVisible(boolean visible){
      this.label.setVisible(visible);
   }

   
   public void switchFromDefault(){
      if(0 <= this.absPanel.getWidgetIndex(this.defaultImg)){
         this.absPanel.remove(this.defaultImg);
      }
      this.changeImage(this.img);
   }
   
   
   public void changeImage(Image img){      
      if(0 <= this.absPanel.getWidgetIndex(this.img)){
         this.absPanel.remove(this.img);
      } else if(0 <= this.absPanel.getWidgetIndex(this.defaultImg)){
         this.absPanel.remove(this.defaultImg);
      }
      this.img = img;
      this.rescaleImage(this.img);
      // Add a deferred command that will reposition the new image
      // based on its current center anchor points and new center anchor points
   }   
   
   //--//
   
   private void rescaleImage(Image img){
      this.absPanel.add(img, -999, 0);
      
      ImageUtility.rescaleImage(img             ,
                                this.maxWidthImg,
                                this.maxHeightImg,
                                this.absPanel   ,
                                0               ,
                                0);
   }
}
