package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import java.util.Vector;

import com.allen_sauer.gwt.dnd.client.PickupDragController;
import com.allen_sauer.gwt.dnd.client.drop.DropController;
import com.google.gwt.animation.client.Animation;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseMoveEvent;
import com.google.gwt.event.dom.client.MouseMoveHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Event.NativePreviewEvent;
import com.google.gwt.user.client.Event.NativePreviewHandler;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ActorWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ConnectorHandleWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.IconWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PrimitiveWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;

public class Scene extends FocusPanel {

   protected static final String STYLE_SCENE       = "scene";
   protected static final int    TIME_FADE_IN_FAST = 100;
   protected static final int    TIME_FADE_IN      = 200;
   
   //--//
   
   protected SequencePanel                 parentSequence;
   protected VerticalPanel                 containerPanel;
   protected AbsolutePanel                 sceneSyntaxSplitter;
   
   protected ScenePositioningPanel         positioningPanel;
   protected SceneInfoPanel                infoPanel;
   protected boolean                       includeInfo; 
   protected boolean                       thumbnail;
   
   protected SceneDropController           dropController;
   protected int                           width;
   protected int                           height;
   
   protected int                           id;
   protected int                           sequencePosition;
   protected Vector<ActorWidget>           actorWidgets;
   protected Vector<PrimitiveWidget>       primitiveWidgets;
   protected Vector<ConnectorHandleWidget> connectorWidgets;
   
   protected ContextMenu                   sceneMenu;
   
   protected boolean                       connecting;
   protected boolean                       copying;
   protected Vector<IconWidget>            potentialTargetList;
   protected IconWidget                    draggingIcon;
   protected IconWidget                    connectorSourceIcon;
   protected IconWidget                    closestTarget;
   
   //--//
      
   /**
    * Creates a new scene with the specified id, parent sequence and sizing 
    * parameters.
    * @param id
    * @param parentSequence
    * @param width
    * @param height
    * @param positioningHeight
    * @param infoHeight
    */
   public Scene(int           id               ,
                SequencePanel parentSequence   ,
                int           width            ,
                int           height           ,
                int           positioningHeight,
                int           infoHeight       ,
                boolean       includeInfo){

      this.id               = id;
      this.sequencePosition = 0;
      this.parentSequence   = parentSequence;
      this.containerPanel   = new VerticalPanel();
      this.width            = width;      
      this.actorWidgets     = new Vector<ActorWidget>();
      this.primitiveWidgets = new Vector<PrimitiveWidget>();
      this.connectorWidgets = new Vector<ConnectorHandleWidget>();
      
      this.addStyleName ( Scene.STYLE_SCENE  );
      
      this.addMouseUpHandler  (new SceneMouseUpHandler  ());
      Event.addNativePreviewHandler(new SceneGlobalMouseMoveHandler());
      this.addMouseMoveHandler(new SceneMouseMoveHandler());
      this.addMouseOutHandler (new SceneMouseOutHandler ());
      
      this.sceneMenu        = new ContextMenu();
      JavaScriptHelper.disableContextMenu(sceneMenu.getElement(), true);
      this.sceneMenu.addItem( "Clear scene" , new CmdClear (this) );
      this.sceneMenu.addItem( "Delete scene", new CmdDelete(this) );
      this.sceneMenu.hide();
      
      this.connecting          = false;
      this.copying             = false;
      this.potentialTargetList = new Vector<IconWidget>();
      this.draggingIcon        = null;
      this.connectorSourceIcon = null;
      this.closestTarget       = null;
      
      this.positioningPanel = new ScenePositioningPanel(this);
      this.positioningPanel.setWidth (this.width + "px", this.width);
      this.positioningPanel.setHeight(positioningHeight + "px", positioningHeight);
      
      this.dropController   = new SceneDropController(this, this.positioningPanel);

      this.containerPanel.add(this.positioningPanel);    
      
      this.includeInfo = includeInfo;
      this.thumbnail   = !includeInfo;
      if(this.includeInfo){
         this.height = height;
         this.setWidth     ( this.width + "px"  );
         this.setHeight    ( this.height + "px" );
         
         this.sceneSyntaxSplitter = new AbsolutePanel();
         this.sceneSyntaxSplitter.setPixelSize(this.width, 4);
         this.sceneSyntaxSplitter.setStyleName("scene-syntax-splitter");
         this.containerPanel.add(this.sceneSyntaxSplitter);
         
         this.infoPanel = new SceneInfoPanel(this);
         this.infoPanel.setWidth (this.width + "px", this.width);
         this.infoPanel.setHeight(infoHeight + "px", infoHeight);
         this.containerPanel.add(this.infoPanel);
      } else {
         this.height = height - infoHeight;
         this.setWidth( this.width + "px"  );
         this.setHeight( this.height + "px" );
      }
      this.add(this.containerPanel);
   }

   
   public void fadeIn(boolean fast){
      SceneFadeInAnimation animation = new SceneFadeInAnimation();
      if(fast)
         animation.run(Scene.TIME_FADE_IN_FAST);
      else
         animation.run(Scene.TIME_FADE_IN);
   }
   
   
   public DropController getDropController(){
      return this.dropController;
   }
   
   
   public int getWidth(){
      return this.width;
   }
   
   
   public int getHeight(){
      return this.height;
   }
   
   
   public int getWidgetLeft(Widget w){
       return this.positioningPanel.getWidgetLeft(w);
   }
   
   
   public int getWidgetTop(Widget w){
       return this.positioningPanel.getWidgetTop(w);
   }
   
   
   private void addIconWidget(IconWidget w){

      int left = w.getAbsoluteLeft() - this.positioningPanel.getAbsoluteLeft();
      int top  = w.getAbsoluteTop () - this.positioningPanel.getAbsoluteTop ();

      this.addIconWidget(w, left, top);
   }
   
   
   private void addIconWidget(IconWidget w, int left, int top){
      int widthBefore = w.getOffsetWidth();
      int diff        = 0;
      
      if(!w.getLabelVisible()){
         w.setLabelVisible(true);
         this.positioningPanel.add( w, left, top );
         diff = w.getOffsetWidth() - widthBefore;
      } else {
         this.positioningPanel.add( w, left, top );
      }
       
      if(diff > 0){
         left -= (diff / 2);
         this.positioningPanel.setWidgetPosition( w, left, top );
      }
      if(this.includeInfo) this.infoPanel.updateInfo();
   }
   
   
   public void addActorWidget(ActorWidget aw){
      if(!this.actorWidgets.contains(aw)){
         this.actorWidgets.add( aw );
      }
      this.addIconWidget( aw );
   }
   
   
   public void addActorWidget(ActorWidget aw, int left, int top){
      if(!this.actorWidgets.contains(aw)){
         this.actorWidgets.add( aw );
      }
      this.addIconWidget(aw, left, top);
   }

   
   public void addPrimitiveWidget(PrimitiveWidget pw){
      if(!this.primitiveWidgets.contains(pw)){
         this.primitiveWidgets.add( pw );
      }
      this.addIconWidget( pw );
   }
   
   
   public void addPrimitiveWidget(PrimitiveWidget pw, int left, int top){
      if(!this.primitiveWidgets.contains(pw)){
         this.primitiveWidgets.add( pw );
      }
      this.addIconWidget(pw, left, top);
   }
   
   
   public void addConnectorHandleWidget(ConnectorHandleWidget chw){
      if(!this.connectorWidgets.contains(chw)){
         this.connectorWidgets.add(chw);
         
         // Will be repositioned to middle of connector on redraw of connectors
         this.positioningPanel.add(chw, chw.actorX, chw.actorY);
 
         if(this.includeInfo) this.infoPanel.updateInfo();
         this.redrawConnectors();
      }
      
   }
   
   
   public void handleGlobalIconWidgetAddition(IconWidget w, Scene s){
      if(this.draggingIcon == w && this != s){
         this.draggingIcon = null;
         
         if(w instanceof ActorWidget){
            this.removeActorWidget((ActorWidget)w);
         } else if(w instanceof PrimitiveWidget){
            this.removePrimitiveWidget((PrimitiveWidget)w);
         }

         Vector<ConnectorHandleWidget> connectors = w.getConnectorHandles();
         for(int i = 0; i < connectors.size(); i++){
            this.removeConnectorHandleWidget(connectors.get(i));
            i--;
         }
      }
   }
   
   
   public void handleGlobalIconWidgetRemoval(IconWidget w){
      if(this.draggingIcon == w){
         this.draggingIcon = null;
         
         if(w instanceof ActorWidget){
            this.removeActorWidget((ActorWidget)w);
         } else if(w instanceof PrimitiveWidget){
            this.removePrimitiveWidget((PrimitiveWidget)w);
         }

         ScenicToolbar.getDragController().removeDragHandler(w.getDragHandler()); // NOTE: only call when icon is actually deleted
         
         Vector<ConnectorHandleWidget> connectors = w.getConnectorHandles();
         for(int i = 0; i < connectors.size(); i++){
            this.removeConnectorHandleWidget(connectors.get(i));
            i--;
         }
      }
   }
   
   
   public void removeIconWidget(IconWidget w){
      
      if(w instanceof ActorWidget){
         this.removeActorWidget((ActorWidget)w);
      } else if(w instanceof PrimitiveWidget){
         this.removePrimitiveWidget((PrimitiveWidget)w);
      }

      this.positioningPanel.remove(w); // Note that positioning panel may not contain icon
      ScenicToolbar.getDragController().removeDragHandler(w.getDragHandler());
      
      Vector<ConnectorHandleWidget> connectors = w.getConnectorHandles();
      for(int i = 0; i < connectors.size(); i++){
         this.removeConnectorHandleWidget(connectors.get(i));
         i--;
      }
   }
   
   
   public void removeActorWidget(ActorWidget aw){      
      this.actorWidgets.remove(aw);
      if(this.includeInfo) this.infoPanel.updateInfo();
   }
   
   
   public void removePrimitiveWidget(PrimitiveWidget pw){
      this.primitiveWidgets.remove(pw);
      if(this.includeInfo) this.infoPanel.updateInfo();
   }
   
   
   public void removeConnectorHandleWidget(ConnectorHandleWidget chw){
      this.positioningPanel.remove(chw);
      this.connectorWidgets.remove(chw);
      
      if(null != chw.actorWidget){
         chw.actorWidget.removeConnectorHandle(chw);
      }
      if(null != chw.primitiveWidget){
         chw.primitiveWidget.removeConnectorHandle(chw);
      }
      if(this.includeInfo) this.infoPanel.updateInfo();
      this.redrawConnectors();
   }
   
   
   public Vector<ActorWidget> getActorWidgets(){
      return this.actorWidgets;
   }
   
   
   public Vector<PrimitiveWidget> getPrimitiveWidgets(){
      return this.primitiveWidgets;
   }
   
   
   public Vector<ConnectorHandleWidget> getConnectorHandleWidgets(){
      return this.connectorWidgets;
   }
   
   
   public boolean getCopying(){
      return this.copying;
   }
   
   
   public void refreshInfoPanel(){
      if(this.includeInfo) this.infoPanel.updateInfo();
   }
   
   
   public void refreshConnectors(){
      for(ConnectorHandleWidget chw : this.connectorWidgets){
         ActorWidget     aw = chw.actorWidget;
         PrimitiveWidget pw = chw.primitiveWidget;
         
         if(null != aw && null != pw){
            int awLeft     = aw.getAbsoluteLeft() - this.positioningPanel.getAbsoluteLeft();
            int awTop      = aw.getAbsoluteTop () - this.positioningPanel.getAbsoluteTop ();
            int pwLeft     = pw.getAbsoluteLeft() - this.positioningPanel.getAbsoluteLeft();
            int pwTop      = pw.getAbsoluteTop () - this.positioningPanel.getAbsoluteTop ();
            chw.actorX     = awLeft + aw.getRelativeXConnectorAnchor();
            chw.actorY     = awTop  + aw.getRelativeYConnectorAnchor();
            chw.primitiveX = pwLeft + pw.getRelativeXConnectorAnchor();
            chw.primitiveY = pwTop  + pw.getRelativeYConnectorAnchor();
         }
      }
      this.redrawConnectors();
   }
   
   
   public void refreshConnectors(IconWidget w){
      int left    = w.getAbsoluteLeft() - this.positioningPanel.getAbsoluteLeft();
      int top     = w.getAbsoluteTop () - this.positioningPanel.getAbsoluteTop ();
      int centerX = left + w.getRelativeXConnectorAnchor();
      int centerY = top  + w.getRelativeYConnectorAnchor();
      
      for(ConnectorHandleWidget connector : w.getConnectorHandles()){
         connector.actorX = centerX;
         connector.actorY = centerY;
      }
      this.redrawConnectors();
   }
   
   
   /**
    * Returns a String that describes the contents of this Scene.
    * @return
    */
   public String getDescription(){
      String desc = new String();
      
      // For now, get the description of each Primitive
      for(PrimitiveWidget pw : this.primitiveWidgets){
         String d = pw.getPrimitiveDescription();
         if(0 < d.length()){
            desc += pw.getPrimitiveDescription() + "<br>\n";
         }
      }
      return desc;
   }
   
   
   public void clearScene(){
      
      for(int i = 0; i < this.positioningPanel.getWidgetCount(); i++){
         Widget w = this.positioningPanel.getWidget(i);
         
         if(w instanceof ActorWidget || w instanceof PrimitiveWidget || w instanceof ConnectorHandleWidget){
            this.positioningPanel.remove(i);
            i--;
         }
      }
      this.actorWidgets    .clear();
      this.primitiveWidgets.clear();
      this.connectorWidgets.clear();
      
      if(this.includeInfo) this.infoPanel.updateInfo();
      this.redrawConnectors();
      
      this.sceneMenu.hide();
   }
   
   
   public void delete(){
      this.parentSequence.deleteScene(this);
      this.sceneMenu.hide();
   }
   
   
   //--//
   
   /**
    * Checks whether the Scene is currently valid.
    */
   public boolean isValid(){
      boolean valid = true;
      
      if(this.actorWidgets.size() == 0 && this.primitiveWidgets.size() == 0){
         valid = false;
      } else {
         // For now, just check that all primitives are valid and all actors are connected
         for(ActorWidget aw : this.actorWidgets){
            if(0 == aw.getConnectorHandles().size()){
               valid = false;
               break;
            }
         }
         if(valid){
            for(PrimitiveWidget pw : this.primitiveWidgets){
               if(!pw.isValid()){
                  valid = false;
                  break;
               }
            }
         }
      }
      return valid;
   }
   
   
   public String getHint(){
      String hint = new String();
      
      if(this.actorWidgets.size() == 0 && this.primitiveWidgets.size() == 0){
         hint = "(Hint: Add icons to start defining this scene.)";
      } else if(this.actorWidgets.size() == 0){
         hint = "(Hint: Add people, objects or places.)";
      } else if(this.primitiveWidgets.size() == 0){
         hint = "(Hint: Add primitives to describe what's happening.)";
      } else {
         // Check all icons connected
         boolean connected = true;
         for(ActorWidget aw : this.actorWidgets){
            if(0 == aw.getConnectorHandles().size()){
               connected = false;
               break;
            }
         }
         for(PrimitiveWidget pw : this.primitiveWidgets){
            if(0 == pw.getConnectorHandles().size()){
               connected = false;
               break;
            }
         }
         if(!connected){
            hint = "(Hint: Add connections between icons.)";
         } else {
            // Check primitives are valid
            for(PrimitiveWidget pw : this.primitiveWidgets){
               if(!pw.isValid()){
                  // TODO: find out why!
                  hint = "";
                  break;
               }
            }
         }
         
         
      }
      
      return hint;
   }
   
   
   //---------------------------- CONNECTOR METHODS --------------------------------
  
   public void setConnectorMode(boolean on, IconWidget w){ 
      
      if(on){
         // Initialize the list of potential connection targets
         if(w instanceof ActorWidget){
            for(PrimitiveWidget pw : this.primitiveWidgets){
               if(w.checkPotentialTarget(pw)){
                  this.potentialTargetList.add(pw);
               }
            }
         } else if(w instanceof PrimitiveWidget){
            for(ActorWidget aw : this.actorWidgets){
               if(w.checkPotentialTarget(aw)){
                  this.potentialTargetList.add(aw);
               }
            }
         }
         
         // Only turn on connector mode if there are potential targets
         if(0 < this.potentialTargetList.size()){
            this.connecting          = true;
            this.connectorSourceIcon = w;
            
            
            // Make all widgets non-draggable
            PickupDragController dc = ScenicToolbar.getDragController();
            for(ActorWidget aw : this.actorWidgets){
               dc.makeNotDraggable(aw);
            }
            for(PrimitiveWidget pw : this.primitiveWidgets){
               dc.makeNotDraggable(pw);
            }
         }
      } else {
         this.connecting          = false;
         this.connectorSourceIcon = null;
         this.closestTarget       = null;
         this.potentialTargetList.clear();
         
         // Make all widgets draggable
         for(ActorWidget aw : this.actorWidgets){
            this.parentSequence.dragController.makeDraggable(aw);
         }
         for(PrimitiveWidget pw : this.primitiveWidgets){
            this.parentSequence.dragController.makeDraggable(pw);
         }
      }
      this.redrawConnectors();
   }

   
   public void setCopying(boolean copying){
      this.copying = copying;
   }
   
   
   /**
    * First make an exact copy of the specified icon and then remove it from this scene.
    * @param w
    * @pre Called on preview drag start
    */
   public void copyIcon(IconWidget w){
      
      // First clone, insert and remap any connectors
      IconWidget clone = w.cloneWidget();
      int        left  = this.positioningPanel.getWidgetLeft(w);
      int        top   = this.positioningPanel.getWidgetTop(w);

      if(clone instanceof ActorWidget){
         this.addActorWidget((ActorWidget)clone, left, top);
         
         for(ConnectorHandleWidget connector : clone.getConnectorHandles()){
            connector.actorWidget = (ActorWidget)clone;
         }
      } else if(clone instanceof PrimitiveWidget){
         this.addPrimitiveWidget((PrimitiveWidget)clone, left, top);
         
         for(ConnectorHandleWidget connector : clone.getConnectorHandles()){
            connector.primitiveWidget = (PrimitiveWidget)clone;
         }
      }
      clone.setScene(this);

      // Then remove old actor (currently being dragged) from Scene
      if(w instanceof ActorWidget){
         this.removeActorWidget((ActorWidget)w);
      } else if(w instanceof PrimitiveWidget){
         this.removePrimitiveWidget((PrimitiveWidget)w);
      }
      
      Vector<ConnectorHandleWidget> oldConnectors = w.getConnectorHandles();
      for(int i = 0; i < oldConnectors.size(); i++){
         w.removeConnectorHandle(oldConnectors.get(i));
         i--;
      }
      
      // Make the clone draggable OUTSIDE of the current call stack
      DeferredCommand.addCommand(new CmdPostCopyMakeDraggable(clone,this));
   }
   
   
   
   protected static class CmdPostCopyMakeDraggable implements Command {
      
      protected IconWidget w;
      protected Scene      s;
      
      public CmdPostCopyMakeDraggable(IconWidget w, Scene s){
         this.w = w;
         this.s = s;
      }
      
      public void execute(){
         PickupDragController dc = ScenicToolbar.getDragController();
         dc.makeDraggable (this.w                 );
         dc.addDragHandler(this.w.getDragHandler());
         
         this.s.copying = false;
      }
   }
   

   public void dragStartIcon(IconWidget w){
      this.draggingIcon = w;
   }
   
   
   public void dragEndIcon(IconWidget w){
      
      if(null != this.draggingIcon){
         for(ConnectorHandleWidget connector : this.draggingIcon.getConnectorHandles()){
            int centerX = getWidgetLeft(this.draggingIcon) + this.draggingIcon.getRelativeXConnectorAnchor();
            int centerY = getWidgetTop (this.draggingIcon) + this.draggingIcon.getRelativeYConnectorAnchor();
            
            if(this.draggingIcon instanceof ActorWidget){
               connector.actorX = centerX;
               connector.actorY = centerY;
            } else if(this.draggingIcon instanceof PrimitiveWidget){
               connector.primitiveX = centerX;
               connector.primitiveY = centerY;
            }
            // Set new position of the connector handle in the middle of the line segment
            int newX = ((connector.primitiveX + connector.actorX)/2) - (connector.getWidth () / 2);
            int newY = ((connector.primitiveY + connector.actorY)/2) - (connector.getHeight() / 2);
            this.positioningPanel.setWidgetPosition(connector, newX, newY);
         }
         this.draggingIcon = null;
         redrawConnectors();
      }
   }
   
   
   public void mouseUpIcon(IconWidget w){
      if(this.connecting){
         this.attemptConnection();
      }
   }
   
   //--//
   
   
   protected void attemptConnection(){
      boolean         addConnector = false;
      ActorWidget     aw           = null;
      PrimitiveWidget pw           = null;
      
      if(this.connectorSourceIcon instanceof ActorWidget){
         aw = (ActorWidget)     this.connectorSourceIcon;
         pw = (PrimitiveWidget) this.closestTarget;
      } else if(this.connectorSourceIcon  instanceof PrimitiveWidget){
         aw = (ActorWidget)     this.closestTarget;
         pw = (PrimitiveWidget) this.connectorSourceIcon;
      }
      
      // If closestTarget is non-null then make connection to it
      if(null != this.closestTarget){
         addConnector = true;
         
         for(ConnectorHandleWidget connector : this.connectorWidgets){
            if(connector.primitiveWidget == pw && connector.actorWidget == aw){
               addConnector = false;
               break;
            }
         }
      }
      
      if(addConnector){
         int actorX     = this.getWidgetLeft(aw) + aw.getRelativeXConnectorAnchor();
         int actorY     = this.getWidgetTop (aw) + aw.getRelativeYConnectorAnchor();
         int primitiveX = this.getWidgetLeft(pw) + pw.getRelativeXConnectorAnchor();
         int primitiveY = this.getWidgetTop (pw) + pw.getRelativeYConnectorAnchor();
         
         ConnectorHandleWidget connector = new ConnectorHandleWidget(aw        ,
                                                                     pw        , 
                                                                     primitiveX, 
                                                                     primitiveY, 
                                                                     actorX    ,
                                                                     actorY    ,
                                                                     true);
         
         this.connectorSourceIcon.addConnectorHandle(connector);
         this.closestTarget.addConnectorHandle(connector);
         this.connectorWidgets.add(connector);
         
         // Will be repositioned to middle of connector on redraw of connectors
         this.positioningPanel.add(connector, actorX, actorY);
                
         this.setConnectorMode(false, null);
         if(this.includeInfo) this.infoPanel.updateInfo();
      } else {
         this.setConnectorMode(false, null);
      }
      this.redrawConnectors();
   }
   
   
   public void redrawConnectors(){
      Vector<LineSegment> segments = new Vector<LineSegment>();

      for(ConnectorHandleWidget handle : this.connectorWidgets){
         
         double width = (thumbnail)? LineSegment.WIDTH_CONNECTED_THUMBNAIL : LineSegment.WIDTH_CONNECTED;
         String color = LineSegment.COLOR_CONNECTED;
         
         if(//(this.connecting && this.connectorHandle == handle) ||
            (null != draggingIcon && draggingIcon.getConnectorHandles().contains(handle))){
            width = LineSegment.WIDTH_CONNECTING;
            color = LineSegment.COLOR_CONNECTING;
         }
         
         LineSegment seg = new LineSegment(handle.primitiveX,
                                           handle.primitiveY,
                                           handle.actorX    ,
                                           handle.actorY    ,
                                           width            ,
                                           color            ,
                                           false);
         
         segments.add(seg);
         
         // Set new position of the connector handle in the middle of the line segment
         int newX = ((handle.primitiveX + handle.actorX)/2) - (handle.getWidth () / 2);
         int newY = ((handle.primitiveY + handle.actorY)/2) - (handle.getHeight() / 2);
         positioningPanel.setWidgetPosition(handle, newX, newY);
      }
      
      if(this.connecting){
         for(IconWidget w : this.potentialTargetList){
            double width   = LineSegment.WIDTH_TARGET;
            String color   = LineSegment.COLOR_TARGET;
            int actorX     = 0;
            int actorY     = 0;
            int primitiveX = 0;
            int primitiveY = 0;
            
            if(w instanceof ActorWidget){
               actorX     = this.getWidgetLeft(w) + w.getRelativeXConnectorAnchor();
               actorY     = this.getWidgetTop (w) + w.getRelativeYConnectorAnchor();
               primitiveX = this.getWidgetLeft(this.connectorSourceIcon) + this.connectorSourceIcon.getRelativeXConnectorAnchor();
               primitiveY = this.getWidgetTop (this.connectorSourceIcon) + this.connectorSourceIcon.getRelativeYConnectorAnchor();
            } else if(w instanceof PrimitiveWidget){
               actorX     = this.getWidgetLeft(this.connectorSourceIcon) + this.connectorSourceIcon.getRelativeXConnectorAnchor();
               actorY     = this.getWidgetTop (this.connectorSourceIcon) + this.connectorSourceIcon.getRelativeYConnectorAnchor();
               primitiveX = this.getWidgetLeft(w) + w.getRelativeXConnectorAnchor();
               primitiveY = this.getWidgetTop (w) + w.getRelativeYConnectorAnchor();
            }
            boolean dashed = (w != closestTarget);
            LineSegment seg = new LineSegment(primitiveX,
                                              primitiveY,
                                              actorX    ,
                                              actorY    ,
                                              width     ,
                                              color     ,
                                              dashed);
            segments.add(seg);
         }
      }
      this.positioningPanel.drawSegments(segments);
   }
   
   
   //---------------------------- COMMANDS --------------------------------
   
   
   protected static class CmdClear implements Command {
      
      protected Scene s;
      
      public CmdClear(Scene s){
         this.s = s;
      }
      
      public void execute(){
         this.s.clearScene();
      }      
   }
   
   
   protected static class CmdDelete implements Command {
      
      protected Scene s;
      
      public CmdDelete(Scene s){
         this.s = s;
      }
      
      public void execute(){
         this.s.delete();
      }
   }
   
   //---------------------------- HANDLERS --------------------------------
   
   
   protected class SceneGlobalMouseMoveHandler implements NativePreviewHandler {
      
      public SceneGlobalMouseMoveHandler(){}
      
      public void onPreviewNativeEvent(NativePreviewEvent event) {
         if (event.getNativeEvent().getType().equals("mousemove")){
            
            if(null != draggingIcon){

               for(ConnectorHandleWidget connector : draggingIcon.getConnectorHandles()){
                  
                  int left    = draggingIcon.getAbsoluteLeft() - positioningPanel.getAbsoluteLeft();
                  int top     = draggingIcon.getAbsoluteTop () - positioningPanel.getAbsoluteTop ();
                  int centerX = left + draggingIcon.getRelativeXConnectorAnchor();
                  int centerY = top  + draggingIcon.getRelativeYConnectorAnchor();
                  
                  if(draggingIcon instanceof ActorWidget){
                     connector.actorX = centerX;
                     connector.actorY = centerY;
                  } else if(draggingIcon instanceof PrimitiveWidget){
                     connector.primitiveX = centerX;
                     connector.primitiveY = centerY;
                  }
               }
               redrawConnectors();
            }
         }
      }
   }
   
   
   // Mouse Handlers
   protected class SceneMouseMoveHandler implements MouseMoveHandler {
      
      public SceneMouseMoveHandler(){}
      
      public void onMouseMove(MouseMoveEvent e){
         
         if(connecting && potentialTargetList.size() > 0){
            
            // Compute closest potential target
            IconWidget closestTemp   = potentialTargetList.get(0);
            int        x1            = e.getRelativeX(positioningPanel.getElement());
            int        x2            = getWidgetLeft(closestTemp) + closestTemp.getRelativeXConnectorAnchor();
            int        y1            = e.getRelativeY(positioningPanel.getElement());
            int        y2            = getWidgetTop(closestTemp)  + closestTemp.getRelativeYConnectorAnchor();
            double     minEuclidDist = Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
            
            for(IconWidget w : potentialTargetList){
               x2 = getWidgetLeft(w) + w.getRelativeXConnectorAnchor();
               y2 = getWidgetTop(w)  + w.getRelativeYConnectorAnchor();
               
               double euclid = Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
               
               if(euclid < minEuclidDist){
                  closestTemp   = w;
                  minEuclidDist = euclid;
               }
            }
            if(closestTemp != closestTarget){
               closestTarget = closestTemp;
               redrawConnectors();
            }
         }
      }
   }
   
   
   protected class SceneMouseUpHandler implements MouseUpHandler {
      
      public SceneMouseUpHandler(){}
      
      public void onMouseUp(MouseUpEvent e){
         Widget sender = (Widget)e.getSource();
         
         int x =   e.getClientX()
                 - DOM.getAbsoluteLeft(sender.getElement())
                 + DOM.getElementPropertyInt(sender.getElement(), "scrollLeft")
                 + Window.getScrollLeft();

         int y =   e.getClientY()
                 - DOM.getAbsoluteTop(sender.getElement())
                 + DOM.getElementPropertyInt(sender.getElement(), "scrollTop")
                 + Window.getScrollTop();

         if(e.getNativeButton() == NativeEvent.BUTTON_RIGHT){
            sceneMenu.setPopupPosition(x + getAbsoluteLeft(), y + getAbsoluteTop());
            sceneMenu.show();
         } else if(e.getNativeButton() == NativeEvent.BUTTON_LEFT && connecting){
            attemptConnection();
         }
         e.stopPropagation();
      }
   }
   
   
   protected class SceneMouseOutHandler implements MouseOutHandler {
      
      public SceneMouseOutHandler(){}
      
      public void onMouseOut(MouseOutEvent e){
         
         if(connecting){
            setConnectorMode(false,null);
            
         } else if(null != draggingIcon){
            // Remove connectors attached to the dragged icon
            for(ConnectorHandleWidget connector : draggingIcon.getConnectorHandles()){
               removeConnectorHandleWidget(connector);
            }
            draggingIcon = null;
         }
      }
   }
   
   
   //---------------------------- ANIMATIONS --------------------------------
   
   protected class SceneFadeInAnimation extends Animation {
      
      private Element sceneElement;
      private Element canvasElement;
      private Element infoElement;
      private Element splitterElement;
      
      public SceneFadeInAnimation(){
         this.sceneElement  = getElement();
         this.canvasElement = positioningPanel.canvas.getElement();
         this.infoElement   = infoPanel.getElement();
         this.splitterElement = sceneSyntaxSplitter.getElement();
         
         ScenicToolbar.setOpacity(this.sceneElement, 0);
         ScenicToolbar.setOpacity(this.canvasElement, 0);
         ScenicToolbar.setOpacity(this.infoElement, 0);
         ScenicToolbar.setOpacity(this.splitterElement, 0);
      }
      
      @Override
      protected void onUpdate(double progress){
         ScenicToolbar.setOpacity(this.sceneElement, progress);
         ScenicToolbar.setOpacity(this.canvasElement, progress);
         ScenicToolbar.setOpacity(this.infoElement, progress);
         ScenicToolbar.setOpacity(this.splitterElement, progress);
      }
   }
}