package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import java.util.Vector;

import com.allen_sauer.gwt.dnd.client.PickupDragController;
import com.google.gwt.animation.client.Animation;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.SceneFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.TransitionFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.IconWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;


public class SequencePanel extends HorizontalPanel implements ResizeHandler, MouseUpHandler {

   protected static final String SEQUENCE_PANEL_STYLE  = "sequencePanel";
   protected static final int    TIME_INSERT_ANIMATION = 250;
   protected static final int    TIME_DELETE_ANIMATION = 250;
   
   //--//
   
   protected final PickupDragController  dragController;
   protected final DefaultDropController dropController;
   
   //--//
   
   protected String             height;
   protected int                sceneSpacing;
   protected Vector<Scene>      scenes;               // List of Scenes in this sequence
   protected Vector<Transition> transitions;          // List of the transitions in the sequence
   protected int                currSceneId;          // Current Scene ID (next ID to be assigned)
   protected int                numScenes;            // The count of all scenes in the sequence
   protected int                currWidth;            // Current width of the SequencePanel
   protected HorizontalPanel    spacerPanel;          // Spacer panel to force Sequence Panel to draw   
   protected ContextMenu        sequenceMenu;         // Context menu for sequences
   
   protected boolean            thumbnail;
   
   //--//
   
   public SequencePanel(String  height      , 
                        int     sceneSpacing, 
                        boolean dragEnabled ,
                        boolean thumbnail) { 
      this.height         = height;
      this.sceneSpacing   = sceneSpacing;
      this.thumbnail      = thumbnail;
      
      if(dragEnabled){
         this.dragController = ScenicToolbar.getDragController();
         this.dropController = new DefaultDropController(this);
         
         this.dragController.registerDropController(this.dropController);
      } else {
         this.dragController = null;
         this.dropController = null;
      }
      this.scenes      = new Vector<Scene>();
      this.transitions = new Vector<Transition>();
      
      this.currSceneId = 0;
      this.currWidth   = Window.getClientWidth();
      this.numScenes   = 0;
      
      this.spacerPanel = new HorizontalPanel();
      this.spacerPanel.setSize("1px", "1px");
      this.spacerPanel.addStyleName(SequencePanel.SEQUENCE_PANEL_STYLE);
      this.add(this.spacerPanel);
      
      this.sequenceMenu = new ContextMenu();
      
      JavaScriptHelper.disableContextMenu(this.sequenceMenu.getElement(), true);
      this.sequenceMenu.addItem("Clear Sequence", new CmdClear(this));
      this.sequenceMenu.hide();
      
      this.addStyleName          ( SequencePanel.SEQUENCE_PANEL_STYLE );
      this.setSpacing            ( SequencePanel.this.sceneSpacing    );
      this.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_LEFT  );
      this.setVerticalAlignment  ( HasVerticalAlignment.ALIGN_MIDDLE  );

      if(!thumbnail){
         this.setWidth(Window.getClientWidth() + "px");
      }
      this.setHeight(this.height);
   }
   
   
   /**
    * Overloaded method that makes widgets draggable.
    * 
    * @param w the widget to be added is made draggable
    */
   public void add(Scene s) {
      
      if(0 == this.numScenes){
         this.remove(this.spacerPanel);
      }
      
      if(this.numScenes > 0){
         Transition t = (s.thumbnail)? TransitionFactory.createThumbnailTransition(this) : 
                                       TransitionFactory.createTransition(this);
         super.add(t);
         this.transitions.add(t);
      }
      
      super.add(s);
      this.scenes.add(s);
       
      if(!s.thumbnail){
         this.dragController.registerDropController(s.getDropController());
      }
      this.readjustCellWidths();
      
      s.sequencePosition = this.numScenes;
      this.numScenes++;
      
      s.refreshConnectors();
      s.refreshInfoPanel();
      s.fadeIn(true);
   }
   
   
   public void insertScene(Scene s, int i){
      
      if(0 == this.numScenes){
         this.remove(this.spacerPanel);
         this.add(s);
         s.fadeIn(true);
      } else {
         if(i == this.numScenes){
            this.add(s);
            s.fadeIn(true);
         } else {
            int insertIndex = i * 2;

            if(insertIndex == 0){
               // Then add a new transition at the beginning
               Transition t = (s.thumbnail)? TransitionFactory.createThumbnailTransition(this) :
                                             TransitionFactory.createTransition(this);
               super.add(t);
               this.transitions.insertElementAt(t,0);
               this.insert(t, 0);
            } else {
               // Then replace the existing transitions with two new ones
               Transition oldTransition = (Transition)this.getWidget(insertIndex - 1);
               int        oldIndex      = this.transitions.indexOf(oldTransition);
               this.transitions.remove(oldTransition);
               this.remove(insertIndex-1);

               Transition t1 = (s.thumbnail)? TransitionFactory.createThumbnailTransition(this) :
                                              TransitionFactory.createTransition(this);
               super.add(t1);
               this.transitions.insertElementAt(t1,oldIndex);
               this.insert(t1, insertIndex-1);

               Transition t2 = (s.thumbnail)? TransitionFactory.createThumbnailTransition(this) :
                                              TransitionFactory.createTransition(this);
               super.add(t2);
               this.transitions.insertElementAt(t2,oldIndex);
               this.insert(t2, insertIndex-1);
            }
            
            // Animate in the scene
            SceneInsertAnimation animation = new SceneInsertAnimation(s,i,insertIndex);
            animation.run(SequencePanel.TIME_INSERT_ANIMATION);
         }
      }
   }
   
   
   public void refreshConnectors(){
      for(Scene s : this.scenes){
         s.refreshConnectors();
      }
   }
   
   
   public void deleteScene(Scene s){
      if(!s.thumbnail){
         this.dragController.unregisterDropController(s.getDropController());
      }
      // Remove the widgets before and/or after the scene
      int widgetIndex = this.getWidgetIndex(s);
      if(widgetIndex == 0 && this.numScenes > 1){
         // Then remove the following transition
         Transition oldTransition = (Transition)this.getWidget(1);
         this.transitions.remove(oldTransition);
         this.remove(1);
      } else if(widgetIndex > 0 && widgetIndex == this.getWidgetCount() - 1){
         // Then remove the preceding transition
         Transition oldTransition = (Transition)this.getWidget(widgetIndex - 1);
         this.transitions.remove(oldTransition);
         this.remove(widgetIndex - 1);
      } else if(widgetIndex > 0){
         // Then remove the preceding and following transitions 
         // and insert a new blank transition before
         Transition following = (Transition)this.getWidget(widgetIndex+1);
         this.transitions.remove(following);
         this.remove(widgetIndex + 1);
         
         Transition preceding = (Transition)this.getWidget(widgetIndex-1);
         int precedingIndex = this.transitions.indexOf(preceding);
         this.transitions.remove(preceding);
         this.remove(widgetIndex-1);
         
         Transition t = TransitionFactory.createTransition(this);
         this.transitions.insertElementAt(t,precedingIndex);
         this.insert(t, widgetIndex-1);
      }
      
      
      int sceneIndex = this.scenes.indexOf(s);
      this.scenes.remove(s);
      this.numScenes--;
      
      for(int i = sceneIndex; i < this.scenes.size(); i++){
         this.scenes.get(i).sequencePosition = i;
         this.scenes.get(i).refreshInfoPanel();
      }
      
      SceneDeleteAnimation animation = new SceneDeleteAnimation(s);
      animation.run(SequencePanel.TIME_DELETE_ANIMATION);
   }
   
   
   public void clearScenes(){      
      if(0 < this.numScenes){
         for(int i = 0; i < this.getWidgetCount(); i++){
            Widget w = this.getWidget(i);
            
            if(w instanceof Scene){
               Scene s = (Scene)w;
               
               if(!s.thumbnail){
                  this.dragController.unregisterDropController(s.getDropController());
               }
               this.remove(s);
               this.scenes.remove(s);
               this.numScenes--;
               i--;
            } else if(w instanceof Transition){
               Transition t = (Transition)w;
               this.remove(t);
               this.transitions.remove(t);
               i--;
            }
         }
      }
      if(this.getWidgetCount() == 0){
         this.add(this.spacerPanel);
      }
      this.sequenceMenu.hide();
   }
   
   
   public String getFromTime(){
      ScenicToolbar toolbar = ScenicToolbar.getInstance();
      return toolbar.getFromTime();
      
   }
   
   
   public String getToTime(){
      ScenicToolbar toolbar = ScenicToolbar.getInstance();
      return toolbar.getToTime();
   }
   
   
   public String getDays(){
      ScenicToolbar toolbar = ScenicToolbar.getInstance();
      return toolbar.getDays();
   }
   
   
   public Vector<Scene> getScenes(){
      
      Vector<Scene> scenes = new Vector<Scene>();
      
      for(int i = 0; i < this.getWidgetCount(); i++){
         Widget w = this.getWidget(i);
         if(w instanceof Scene){
            scenes.add( (Scene) this.getWidget(i) );
         }
      }
      return scenes;
   }
   
   
   public Vector<Transition> getTransitions(){
      Vector<Transition> transitions = new Vector<Transition>();
      
      for(int i = 0; i < this.getWidgetCount(); i++){
         Widget w = this.getWidget(i);
         if(w instanceof Transition){
            transitions.add((Transition)this.getWidget(i));
         }
      }
      return transitions;
   }
   
   
   /**
    * Gets the Scene of the specified number (indexed starting at 0).
    * @param i
    * @return
    */
   public Scene getScene(int i){
      Scene s        = null;
      int   sceneNum = 0;
      
      for(int j = 0; j < this.getWidgetCount(); j++){
         Widget w = this.getWidget(j);
         if(w instanceof Scene){
            if(sceneNum == i){
               s = (Scene)w;
               break;
            } else {
               sceneNum++;
            }
         }
      }
      return s;
   }
   
   
   /**
    * Returns the scene of the specified number (starting at 0).
    * @param i
    * @return
    */
   public Transition getTransition(int i){
      Transition t             = null;
      int        transitionNum = 0;
      
      for(int j = 0; j < this.getWidgetCount(); j++){
         Widget w = this.getWidget(j);
         if(w instanceof Transition){
            if(transitionNum == i){
               t = (Transition)w;
               break;
            } else {
               transitionNum++;
            }
         }
      }
      return t;
   }
   
   
   public void signalWidgetDeletion(Widget w){
      if(w instanceof IconWidget){
         for(Scene s : this.scenes){
            s.handleGlobalIconWidgetRemoval((IconWidget)w);
         }
      }
   }
   
   
   public void signalWidgetAddition(Widget w, Scene scene){
      if(w instanceof IconWidget){
         for(Scene s : this.scenes){
            s.handleGlobalIconWidgetAddition((IconWidget)w, scene);
         }
      }
   }
   
   
   public void onResize(ResizeEvent e){
      if(!this.thumbnail){
         this.currWidth = e.getWidth();
         this.setWidth( this.currWidth + "px");
         
         this.readjustCellWidths();
      }
   }
      
   
   public boolean okToScroll(){
       int sequenceWidth = this.sceneSpacing;
       
       for(int i = 0; i < (this.getWidgetCount() - 1); i++){
          Widget w = this.getWidget(i);
          if(w instanceof Scene){
             Scene s        = (Scene)w;
             sequenceWidth += s.getWidth() + this.sceneSpacing;
          } else if(w instanceof Transition){
             Transition t   = (Transition)w;
             sequenceWidth += t.getWidth() + this.sceneSpacing;
          }
       }
       
       return(sequenceWidth < this.currWidth);
   }
   
   
   //--//
   
   // TODO: Call this after changing the timing wedge...
   protected void readjustCellWidths(){
      if(this.getWidgetCount() > 0 && this.numScenes > 0){
         // Calculate width of all scenes but the last
         int prefixWidth = 0;

         for(int i = 0; i < (this.getWidgetCount() - 1); i++){
            Widget w = this.getWidget(i);

            if(w instanceof Scene){
               Scene s      = (Scene)w;
               prefixWidth += s.getWidth() + this.sceneSpacing;

               this.setCellWidth(s, (s.getWidth()) + "px");
               this.setCellHorizontalAlignment(s, HasHorizontalAlignment.ALIGN_LEFT);
            } else if(w instanceof Transition){
               Transition t = (Transition)w;
               prefixWidth += t.getWidth() + this.sceneSpacing;
               
               this.setCellWidth(t, t.getWidth() + "px");
               this.setCellHorizontalAlignment(t, HasHorizontalAlignment.ALIGN_LEFT);
            }
         }

         if(!thumbnail){
            // Set width of the last scene or transition cell to rest of the screen
            Widget w = this.getWidget(this.getWidgetCount() - 1);

            if(w instanceof Scene){
               Scene s       = (Scene) w;
               int sWidth    = s.getWidth();
               int lastWidth = this.currWidth - prefixWidth;
               lastWidth     = (lastWidth < sWidth)? sWidth : lastWidth;

               this.setCellWidth(s, lastWidth + "px");
               this.setCellHorizontalAlignment((Widget)s, HasHorizontalAlignment.ALIGN_LEFT);   
            } else if(w instanceof Transition){
               Transition t  = (Transition) w;
               int tWidth    = t.getWidth();
               int lastWidth = this.currWidth - prefixWidth;
               lastWidth     = (lastWidth < tWidth)? tWidth : lastWidth;

               this.setCellWidth(t, lastWidth + "px");
               this.setCellHorizontalAlignment(t, HasHorizontalAlignment.ALIGN_LEFT);  
            }
         } else {
            // Set width of the last scene or transition cell to its own width
            Widget w = this.getWidget(this.getWidgetCount() - 1);

            if(w instanceof Scene){
               Scene s       = (Scene) w;
               int sWidth    = s.getWidth();
               this.setCellWidth(s, s.getWidth() + "px");
               this.setCellHorizontalAlignment(s, HasHorizontalAlignment.ALIGN_LEFT);  
            } else if(w instanceof Transition){
               Transition t  = (Transition) w;
               int tWidth    = t.getWidth();
               
               this.setCellWidth(t, t.getWidth() + "px");
               this.setCellHorizontalAlignment(t, HasHorizontalAlignment.ALIGN_LEFT);
            }
         }
         
      }
   }
   
   
   //------------------------------- HANDLERS --------------------------------
   
   public void onMouseUp(MouseUpEvent e){
      
      Widget sender = (Widget)e.getSource();
      
      int x =   e.getClientX()
              - DOM.getAbsoluteLeft(sender.getElement())
              + DOM.getElementPropertyInt(sender.getElement(), "scrollLeft")
              + Window.getScrollLeft();

      int y =   e.getClientY()
              - DOM.getAbsoluteTop(sender.getElement())
              + DOM.getElementPropertyInt(sender.getElement(), "scrollTop")
              + Window.getScrollTop();
      
      // Only add a Scene in the event of a left mouse click
      if(e.getNativeButton() == NativeEvent.BUTTON_LEFT){
         
         if(this.numScenes == 0){
            this.add(SceneFactory.createScene(this));
         } else if(this.numScenes == 1){
            if(0 < x && x < this.sceneSpacing){
               this.insertScene(SceneFactory.createScene(this),0);
            } else {
               this.add(SceneFactory.createScene(this));
            }
         } else {
            // Compute where in the sequence the mouse click occurred
            int curXOffset = 0;
            int curSpacing = this.sceneSpacing;
            
            if(curXOffset < x && x < curSpacing){
               this.insertScene(SceneFactory.createScene(this),0);
            } else {
               curXOffset  += curSpacing;
               Transition t = this.getTransition(0);
               curSpacing   = this.sceneSpacing * 2 + t.getWidth();
               
               for(int i = 0; i < this.numScenes - 1; i++){
                  Scene nextScene = this.getScene(i);
                  curXOffset     += nextScene.getWidth();
                  if(curXOffset < x && x < curXOffset + curSpacing){
                     this.insertScene(SceneFactory.createScene(this),i+1);
                  }
                  curXOffset += curSpacing;
               }
               Scene nextScene = this.getScene(this.numScenes - 1);
               curXOffset     += nextScene.getWidth();
               if(curXOffset < x){
                  this.add(SceneFactory.createScene(this));
               }
            }
         }         
      } else if(e.getNativeButton() == NativeEvent.BUTTON_RIGHT){
         this.sequenceMenu.setPopupPosition(x + this.getAbsoluteLeft(), y + this.getAbsoluteTop());
         this.sequenceMenu.show();
      }
   }
   
   
   //------------------------------- COMMANDS --------------------------------
   
   protected static class CmdClear implements Command {
      
      protected SequencePanel sp;
      
      public CmdClear(SequencePanel sp){
         this.sp = sp;
      }
      
      public void execute(){
         this.sp.clearScenes();
      }
   }
   
   
   //------------------------------- ANIMATIONS --------------------------------
   
   protected class SceneInsertAnimation extends Animation {
      
      private HorizontalPanel spacer;
      private int             spacerWidthMax;
      private Scene           s;
      private int             insertIndex;
      
      //--//
      
      /**
       * Creates a new SceneInsertAnimation.
       * @param s
       * @param i
       * @param insertIndex
       */
      public SceneInsertAnimation(Scene s, int i, int insertIndex){
         this.spacer         = new HorizontalPanel();
         this.spacerWidthMax = s.getWidth();
         this.s              = s;
         this.insertIndex    = insertIndex;
         
         this.spacer.setWidth("0px");
         insert(this.spacer, insertIndex);
         scenes.insertElementAt( s, i );
         s.sequencePosition = i;
         
         for(i = i + 1; i < scenes.size(); i++){
            scenes.get(i).sequencePosition = i;
            scenes.get(i).refreshInfoPanel();
         }
      }
      
      @Override
      protected void onUpdate(double progress){
         int w = (int)(progress * this.spacerWidthMax);
         this.spacer.setWidth(w + "px");
         readjustCellWidths();
      }
      
      @Override
      protected void onComplete(){
         int insertAt = getWidgetIndex(this.spacer);
         remove(this.spacer);
         insert(this.s, insertAt);
         
         if(!this.s.thumbnail){
            dragController.registerDropController(this.s.getDropController());
         }
         readjustCellWidths();
         
         numScenes++;
         this.s.refreshConnectors();
         this.s.refreshInfoPanel();
         
         this.s.fadeIn(false);
      }
   }
   
   
   protected class SceneDeleteAnimation extends Animation {
      
      private HorizontalPanel spacer;
      private int             spacerWidthMax;
    
      public SceneDeleteAnimation(Scene s){
         this.spacerWidthMax = s.getWidth();
         this.spacer         = new HorizontalPanel();
         this.spacer.setWidth(this.spacerWidthMax + "px");
         int i = getWidgetIndex(s);
         remove(s);

         if(0 == numScenes)
            add(spacerPanel);
         
         insert(this.spacer,i);
      }
      
      @Override
      protected void onUpdate(double progress){
         int w = (int)((1.0 - progress) * this.spacerWidthMax);
         this.spacer.setWidth(w + "px");
         readjustCellWidths();
      }
      
      @Override
      protected void onComplete(){
         remove(this.spacer);
         readjustCellWidths();
      }
   }
}
