package edu.washington.cs.rfid.cascadia.scenic.client.ui.factory;

import java.util.HashMap;
import java.util.Vector;

import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Person;
import edu.washington.cs.rfid.cascadia.scenic.client.data.PersonGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Place;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Thing;
import edu.washington.cs.rfid.cascadia.scenic.client.data.ThingGrp;

/**
 * Downloads and maintains a personal library of actors for a user.
 * 
 * @author Evan
 */
public class ActorLibrary {
	
	protected Vector<Image>     imageCache;

	protected int               id;
	
	// Lists of people, places and things
	protected Vector<Person>    people;
	protected Vector<Person>    personTypes;
	protected Vector<PersonGrp> personGrps;
	protected Vector<Thing>     things;
	protected Vector<Thing>     thingTypes;
	protected Vector<ThingGrp>  thingGrps;
	protected Vector<Place>     places;
	protected Vector<Place>     placeTypes;
	
	// Tables of people, places and things
	protected HashMap<String,Person>    personTbl;
	protected HashMap<String,PersonGrp> personGrpTbl;
	protected HashMap<String,Thing>     thingTbl;
	protected HashMap<String,ThingGrp>  thingGrpTbl;
	protected HashMap<String,Place>     placeTbl;
	
	//--//
	
	/**
	 * Creates a new <code>ActorLibrary</code>
	 */
	public ActorLibrary(){
		
		this.imageCache  = this.buildImgCache();
		
		this.id          = 1;
		this.people      = this.buildPeople();		
		this.personTypes = this.buildPersonTypes();
		this.personGrps  = this.buildPersonGrps();
		this.things      = this.buildThings();
		this.thingTypes  = this.buildThingTypes();
		this.thingGrps   = this.buildThingGrp();
		this.places      = this.buildPlaces();
		this.placeTypes  = this.buildPlaceTypes();
		
		this.initTables();
	}
	
	public Vector<Person> getPersons(){
		return this.people;
	}
	
	public Vector<Person> getPersonTypes(){
	   return this.personTypes;
	}
	
	public Vector<PersonGrp> getPersonGrps(){
		return this.personGrps;
	}
	
	public Vector<Thing> getThings(){
		return this.things;
	}
	
	public Vector<Thing> getThingTypes(){
	   return this.thingTypes;
	}
	
	public Vector<ThingGrp> getThingGrps(){
		return this.thingGrps;
	}
	
	public Vector<Place> getPlaces(){
		return this.places;
	}
	
	public Vector<Place> getPlaceTypes(){
	   return this.placeTypes;
	}
	
	public Person getPerson(String name){
	   return this.personTbl.get(name);
	}
	
	public PersonGrp getPersonGrp(String name){
	   return this.personGrpTbl.get(name);
	}
	
	public Thing getThing(String name){
	   return this.thingTbl.get(name);
	}
	
	public ThingGrp getThingGrp(String name){
	   return this.thingGrpTbl.get(name);
	}
	
	public Place getPlace(String name){
	   return this.placeTbl.get(name);
	}
	
	//--//
	
	private Vector<Image> buildImgCache(){
	   Vector<Image> cache = new Vector<Image>();
	   
	   cache.add( new Image( "images/person-orange.png" ) );
      cache.add( new Image( "images/person-blue.png" ) );
      cache.add( new Image( "images/person-blue-green.png" ) );
      cache.add( new Image( "images/person-pink.png" ) );
      cache.add( new Image( "images/person-purple.png" ) );
      cache.add( new Image( "images/person-red.png" ) );
      cache.add( new Image( "images/person-yellow.png" ) );
      
      cache.add( new Image( "images/group-person-orange.png" ) );
      cache.add( new Image( "images/group-person-blue.png" ) );
      cache.add( new Image( "images/group-person-blue-green.png" ) );
      cache.add( new Image( "images/group-person-pink.png" ) );
      cache.add( new Image( "images/group-person-purple.png" ) );
      cache.add( new Image( "images/group-person-red.png" ) );
      cache.add( new Image( "images/group-person-yellow.png" ) );
      
      cache.add( new Image( "images/group-person-orange.png" ) );
      cache.add( new Image( "images/group-person-blue.png" ) );
      cache.add( new Image( "images/group-person-blue-green.png" ) );
      cache.add( new Image( "images/group-person-pink.png" ) );
      cache.add( new Image( "images/group-person-purple.png" ) );
      cache.add( new Image( "images/group-person-red.png" ) );
      cache.add( new Image( "images/group-person-yellow.png" ) );
      
      cache.add( new Image( "images/group-object-orange.png" ) );
      cache.add( new Image( "images/group-object-blue.png" ) );
      cache.add( new Image( "images/group-object-blue-green.png" ) );
      cache.add( new Image( "images/group-object-pink.png" ) );
      cache.add( new Image( "images/group-object-purple.png" ) );
      cache.add( new Image( "images/group-object-red.png" ) );
      cache.add( new Image( "images/group-object-yellow.png" ) );

      cache.add( new Image( "images/mug.png" ) );
      cache.add( new Image( "images/book.png" ) );
      cache.add( new Image( "images/backpack.png" ) );
      cache.add( new Image( "images/ipod.png" ) );
      cache.add( new Image( "images/keys.png" ) );
      cache.add( new Image( "images/laptop.png" ) );
      cache.add( new Image( "images/notebook.png" ) );
      cache.add( new Image( "images/umbrella.png" ) );
      cache.add( new Image( "images/wallet.png" ) );
      cache.add( new Image( "images/jacket.png" ) );
      cache.add( new Image( "images/pda.png" ) );
      cache.add( new Image( "images/bike-helmet.png" ) );
      cache.add( new Image( "images/camera.png" ) );
	   
	   return cache;
	}
	
	private Vector<Person> buildPeople(){
	   Vector<Person> people = new Vector<Person>();
      
      people.add( new Person("Alice"      , id++, "Alice"    , "images/person-orange.png", false) );
      people.add( new Person("James"      , id++, "James"    , "images/person-blue.png", false) );
      people.add( new Person("Nikhil"     , id++, "Nikhil", "images/person-red.png", false  ) );
      people.add( new Person("Prof. Chen" , id++, "Prof. Chen", "images/person-pink.png", false  ) );
//    people.add( new Person("Evan"       , id++, "Evan"       , "images/person-blue-green.png", false) );
//    people.add( new Person("Gaetano"    , id++, "Gaetano"    , "images/person-orange.png", false) );
//    people.add( new Person("Magda"      , id++, "Magda"      , "images/person-red.png", false) );
//    people.add( new Person("me"      , id++, "me"      , "images/person-orange.png", false) );
//    people.add( new Person("Custodian Dave"  , id++, "Custodian Dave"  , "images/person-red.png", false   ) );
//    people.add( new Person("Me"         , id++, "Me"         , "images/person-pink.png", false  ) );
//    people.add( new Person("Prof. Silverman", id++, "Prof. Silverman", "images/person-purple.png", false) );
//    people.add( new Person("Support Staff Member" , id++, "Support Staff Member" , "images/person-blue-green.png", false  ) );
      
      return people;
	}
	
	
	private Vector<Person> buildPersonTypes(){
	   Vector<Person> types = new Vector<Person>();
      
	   types.add( new Person("Student"      , id++, "Student"    , "images/person-orange.png", true) );
      types.add( new Person("Faculty"      , id++, "Faculty"    , "images/person-blue.png", true) );
      types.add( new Person("Staff"        , id++, "Staff", "images/person-red.png", true  ) );
      
      return types;
	}
	
	
	private Vector<PersonGrp> buildPersonGrps(){
	   Vector<PersonGrp> groups = new Vector<PersonGrp>();
	   
      groups.add( new PersonGrp("Professors", id++, "Professors", "images/group-person-orange.png", false) );
      groups.add( new PersonGrp("Students"  , id++, "Students", "images/group-person-blue.png", false) );
      groups.add( new PersonGrp("PhDs"      , id++, "PhDs", "images/group-person-blue-green.png", false) );
      groups.add( new PersonGrp("Database"  , id++, "Database", "images/group-person-pink.png", false) );
      
      return groups;
	}
	
	
	private Vector<Thing> buildThings(){
	   Vector<Thing> things = new Vector<Thing>();
      
      things.add( new Thing("Alice's algorithms notes" , id++, "Alice's algorithms notes", "images/notebook.png", false) );
      things.add( new Thing("James' algorithms notes" , id++, "James' algorithms notes", "images/notebook.png", false) );
      things.add( new Thing("Bike Helmet"       , id++, "Bike Helmet", "images/bike-helmet.png", false) );      
//    things.add( new Thing("Coffee mug"        , id++, "Coffee mug", "images/mug.png", false) );
//    things.add( new Thing("Prof. Chen's Book" , id++, "Prof. Chen's Book", "images/book.png", false) );
//    things.add( new Thing("Bob's Backpack"    , id++, "Backpack", "images/backpack.png", false) );
//    things.add( new Thing("Bob's iPod"        , id++, "iPod", "images/ipod.png", false) );
//    things.add( new Thing("Car keys"    , id++, "Car keys", "images/keys.png", false) );
//    things.add( new Thing("My Laptop"    , id++, "My Laptop", "images/laptop.png", false) );
//    things.add( new Thing("Custodian's Mop"   , id++, "Custodian's Mop", "images/mop.png", false) );
//    things.add( new Thing("Database notebook" , id++, "Database notebook", "images/notebook.png", false) );
//    things.add( new Thing("Umbrella"          , id++, "Umbrella", "images/umbrella.png", false) );
//    things.add( new Thing("Wallet"            , id++, "Wallet", "images/wallet.png", false) );
//    things.add( new Thing("Jacket"            , id++, "Jacket", "images/jacket.png", false) );
//    things.add( new Thing("Linux Workstation" , id++, "Linux Workstation", "images/pc.png", false) );
//    things.add( new Thing("Windows Workstation" , id++, "Windows Workstation", "images/pc.png", false) );
//    things.add( new Thing("PDA"               , id++, "PDA", "images/pda.png", false) );
//    things.add( new Thing("Bike Helmet"       , id++, "Bike Helmet", "images/bike-helmet.png", false) );
//    things.add( new Thing("Camera"            , id++, "Camera", "images/camera.png", false) );
      
      return things;
	}
	
	
	private Vector<Thing> buildThingTypes(){
	   Vector<Thing> types = new Vector<Thing>();
      
      types.add( new Thing("Book"  , id++, "Book", "images/book.png", true) );
      types.add( new Thing("Notes" , id++, "Notes", "images/notebook.png", true) );
      types.add( new Thing("Helmet", id++, "Helmet", "images/bike-helmet.png", true) );
      types.add( new Thing("Laptop", id++, "Laptop", "images/laptop.png", true) );
      types.add( new Thing("iPod", id++, "iPod", "images/ipod.png", true) );
      
      return types;
	}
	
	
	private Vector<ThingGrp> buildThingGrp(){
	   Vector<ThingGrp> groups = new Vector<ThingGrp>();
	   
      groups.add( new ThingGrp("Gym gear"        , id++, "Gym gear", "images/group-object-orange.png", false) ); 
      groups.add( new ThingGrp("Meeting supplies", id++, "Meeting supplies", "images/group-object-blue.png", false) );
      groups.add( new ThingGrp("Biking gear"     , id++, "Biking gear", "images/group-object-pink.png", false) );
      groups.add( new ThingGrp("Study materials" , id++, "Study materials", "images/group-object-red.png", false) );
      
      return groups;
	}

	
	private Vector<Place> buildPlaces(){
	   Vector<Place> places = new Vector<Place>();
            
      places.add( new Place("Alice and Nikhil's Office", id++, "Alice and Nikhil's Office", "images/place-pixel.png", false) );    
      places.add( new Place("James' Office"       , id++, "James' office"      , "images/place-pixel.png", false) );
      places.add( new Place("Prof. Chen's Office" , id++, "Prof. Chen's Office" , "images/place-pixel.png", false) );
      places.add( new Place("Lab"                 , id++, "Lab"                 , "images/place-pixel.png", false) );
//    places.add( new Place("Magda's Office"      , id++, "Magda's office"      , "images/place-pixel.png", false) );
//    places.add( new Place("Evan's Office"       , id++, "Evan's office"      , "images/place-pixel.png", false) );
//    places.add( new Place("Building"            , id++, "Building"            , "images/place-pixel.png", false) );
//    places.add( new Place("Alice's Office"      , id++, "Alice's Office"      , "images/place-pixel.png", false) );
//    places.add( new Place("Lab"                 , id++, "Lab"                 , "images/place-pixel.png", false) );
//    places.add( new Place("My Office"         , id++, "My Office"         , "images/place-pixel.png", false) );
//    places.add( new Place("Prof. Chen's Office" , id++, "Prof. Chen's Office" , "images/place-pixel.png", false) );
//    places.add( new Place("Prof. Silverman's Office", id++, "Prof. Silverman's Office", "images/place-pixel.png", false) );
//    places.add( new Place("Custodial Closet"  , id++, "Custodial Closet"  , "images/place-pixel.png", false) );
//    places.add( new Place("Linux Expert\'s Office"  , id++, "Linux Expert\'s Office"  , "images/place-pixel.png", false) );
//    places.add( new Place("Windows Expert\'s Office"  , id++, "Windows Expert\'s Office"  , "images/place-pixel.png", false) );
//    places.add( new Place("Support Office"  , id++, "Support Office"  , "images/place-pixel.png", false) );
//    places.add( new Place("Loading Dock"  , id++, "Loading Dock"  , "images/place-pixel.png", false) );
//    places.add( new Place("Conference Room"     , id++, "Conference Room"     , "images/place-pixel.png", false) );
//    places.add( new Place("Bike Room"     , id++, "Bike Room"     , "images/place-pixel.png", false) );
//    places.add( new Place("Lab"                 , id++, "Lab"                 , "images/place-pixel.png", false) );
//    places.add( new Place("Coffee Stand"        , id++, "Coffee Stand"        , "images/place-pixel.png", false) );
//    places.add( new Place("Student Lounge"      , id++, "Lounge"              , "images/place-pixel.png", false) );
//    places.add( new Place("Printer Room"        , id++, "Printer Room"        , "images/place-pixel.png", false) );     
//    places.add( new Place("Undergrad Lab", id++, "Undergrad Lab", "images/place-pixel.png", false) );
//    places.add( new Place("Lunch Room (4th floor)", id++, "Lunch Room (4th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Printer (4th floor)", id++, "Printer (4th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Coffee Room (4th floor)", id++, "Coffee Room (4th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Lunch Room (5th floor)", id++, "Lunch Room (5th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Printer (5th floor)", id++, "Printer (5th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Coffee Room (5th floor)", id++, "Coffee Room (5th floor)", "images/place-pixel.png", false) );
//    places.add( new Place("Mia's office (CSE 550)", id++, "Mia's office (CSE 550)", "images/place-pixel.png", false) );
      
      return places;
	}
	
	
	private Vector<Place> buildPlaceTypes(){
	   Vector<Place> types = new Vector<Place>();
	   
	   types.add( new Place("Conference Room", id++, "Conference Room", "images/place-pixel.png", true) );
	   types.add( new Place("Lounge", id++, "Lounge", "images/place-pixel.png", true) );
	   types.add( new Place("Office", id++, "Office", "images/place-pixel.png", true) );
	   types.add( new Place("Lab", id++, "Lab", "images/place-pixel.png", true) );
	   
	   return types;
	}
	
	
	private void initTables(){
	   this.personTbl    = new HashMap<String,Person>(); 
	   this.personGrpTbl = new HashMap<String,PersonGrp>();
	   this.thingTbl     = new HashMap<String,Thing>();
	   this.thingGrpTbl  = new HashMap<String,ThingGrp>();
	   this.placeTbl     = new HashMap<String,Place>();
	   
	   for(Person p : this.people){
         this.personTbl.put(p.getName(), p);
      }
	   for(Person p : this.personTypes){
         this.personTbl.put(p.getName(), p);
      }
	   
	   for(PersonGrp pg : this.personGrps){
	      this.personGrpTbl.put(pg.getName(), pg);
	   }
	   
	   for(Thing t : this.things){
	      this.thingTbl.put(t.getName(), t);
	   }
	   for(Thing t : this.thingTypes){
	      this.thingTbl.put(t.getName(), t);
	   }
	   
	   for(ThingGrp tg : this.thingGrps){
	      this.thingGrpTbl.put(tg.getName(), tg);
	   }
	   
	   for(Place p : this.places){
	      this.placeTbl.put(p.getName(), p);
	   }
	   for(Place p : this.placeTypes){
         this.placeTbl.put(p.getName(), p);
      }
	}
}
