package edu.washington.cs.rfid.cascadia.scenic.client.ui.factory;

import java.util.Vector;

import com.allen_sauer.gwt.dnd.client.PickupDragController;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Pair;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicActor;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPrimitive;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicScene;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.Scene;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ScenicToolbar;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.SequencePanel;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ActorWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ConnectorHandleWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PrimitiveWidget;

public class SceneFactory {

   // Normal sizing
   protected static final int WIDTH_SCENE        = 350;
   protected static final int HEIGHT_SCENE       = 340;
   protected static final int POSITIONING_HEIGHT = 240;
   protected static final int INFO_HEIGHT        = 95;
   
   //--//
   
   protected static int sceneNum = 0;
   
   //--//
   
   /**
    * Creates a new Scene.
    * @param id
    * @param sequence
    * @return
    */
   public static Scene createScene(int id, SequencePanel sequence){
      return SceneFactory.createScene(id, sequence, false);
   }
   
   /**
    * Creates a new Scene.
    * @param id
    * @param sequence
    * @return
    */
   public static Scene createScene(SequencePanel sequence){
      return SceneFactory.createScene(SceneFactory.sceneNum++, sequence, false);
   }   
   
   /**
    * Creates a new thumbnail Scene.
    * @param id
    * @param sequence
    * @return
    */
   public static Scene createThumbnailScene(int id, SequencePanel sequence){
      return SceneFactory.createScene(id, sequence, true);
   }   
   
   /**
    * Creates a new Scene.
    * @param id
    * @param sequence
    * @return
    */
   public static Scene createThumbnailScene(SequencePanel sequence){
      return SceneFactory.createScene(SceneFactory.sceneNum++, sequence, true);
   }
   
   //--//
   
   private static Scene createScene(int id, SequencePanel sequence, boolean thumbnail){
      int width             = (thumbnail)? 
                               (int)(SceneFactory.WIDTH_SCENE / SequenceFactory.THUMBNAIL_FACTOR) :
                                     SceneFactory.WIDTH_SCENE;
      int height            = (thumbnail)? 
                              (int)(SceneFactory.HEIGHT_SCENE / SequenceFactory.THUMBNAIL_FACTOR) : 
                                    SceneFactory.HEIGHT_SCENE;
      int positioningHeight = (thumbnail)? 
                              (int)(SceneFactory.POSITIONING_HEIGHT / SequenceFactory.THUMBNAIL_FACTOR) : 
                                    SceneFactory.POSITIONING_HEIGHT;
      int infoHeight        = (thumbnail)? 
                              (int)(SceneFactory.INFO_HEIGHT / SequenceFactory.THUMBNAIL_FACTOR) :
                                    SceneFactory.INFO_HEIGHT;
      return new Scene(id,
                       sequence,
                       width,
                       height,
                       positioningHeight,
                       infoHeight,
                       !thumbnail); // Whether or not to include info panel
   }
   
   //--//
   
   public static Scene createScene(ScenicScene   sceneBean, 
                                   SequencePanel sequence , 
                                   boolean       draggable){
      return SceneFactory.createScene(sceneBean, sequence, draggable, false);
   }
      
   public static Scene createThumbnailScene(ScenicScene   sceneBean, 
                                            SequencePanel sequence , 
                                            boolean       draggable){
      return SceneFactory.createScene(sceneBean, sequence, draggable, true);
   }
   
   //--//
   
   private static Scene createScene(ScenicScene   sceneBean, 
                                    SequencePanel sequence , 
                                    boolean       draggable,
                                    boolean       thumbnail){

      Scene                   s                = null;
      Vector<ScenicActor>     sceneActors      = sceneBean.getActors();
      Vector<ActorWidget>     actorWidgets     = new Vector<ActorWidget>();
      Vector<ScenicPrimitive> scenePrimitives  = sceneBean.getPrimitives();
      Vector<PrimitiveWidget> primitiveWidgets = new Vector<PrimitiveWidget>();

      s = SceneFactory.createScene(SceneFactory.sceneNum++, sequence, thumbnail);

      // Add actors
      for(int j = 0; j < sceneActors.size(); j++){
         ActorWidget aw = SceneFactory.loadActor(sceneActors.get(j), s, draggable, thumbnail);
         if(null != aw){
            actorWidgets.add(aw);
         } else {
            sceneActors.remove(j);
            j--;
         }
      }

      // Add primitives
      for(int j = 0; j < scenePrimitives.size(); j++){
         PrimitiveWidget pw = SceneFactory.loadPrimitive(scenePrimitives.get(j), s, draggable, thumbnail);
         if(null != pw){
            primitiveWidgets.add(pw);
         } else {
            scenePrimitives.remove(j);
            j--;
         }
      }

      // Extract and add connectors, modifying actors and primitives appropriately
      for(int j = 0; j < scenePrimitives.size(); j++){
         SceneFactory.loadConnectors(scenePrimitives.get(j), primitiveWidgets.get(j), actorWidgets, s);
      }
      return s;
   }
   
   
   private static ActorWidget loadActor(ScenicActor sa, 
                                        Scene       s, 
                                        boolean     draggable,
                                        boolean     thumbnail){
      
      ActorWidget aw = (thumbnail)? ActorFactory.createThumbnailWidget(sa) :
                                    ActorFactory.createWidget(sa);

      if(null != aw){
         if(draggable){
            PickupDragController dragCtrl = ScenicToolbar.getDragController();
            dragCtrl.makeDraggable(aw);
            dragCtrl.addDragHandler(aw.getDragHandler());
         }
         aw.setScene(s);

         int left = sa.getLeft();
         int top  = sa.getTop();
         
         if(thumbnail){
            int oldWidth   = (int)(aw.getIconMaxWidth() * SequenceFactory.THUMBNAIL_FACTOR);
            int oldHeight  = (int)(aw.getIconMaxHeight() * SequenceFactory.THUMBNAIL_FACTOR);
            int newCenterX = (int)((sa.getLeft() + oldWidth/2)/SequenceFactory.THUMBNAIL_FACTOR);
            int newCenterY = (int)((sa.getTop()  + oldHeight/2)/SequenceFactory.THUMBNAIL_FACTOR);
            left           = newCenterX - (aw.getIconMaxWidth()/2);
            top            = newCenterY - (aw.getIconMaxHeight()/2);
            if(left < 0) left = 0;
            if(top < 0) top = 0;
         }
         
         s.addActorWidget(aw, left, top);
         if(thumbnail)aw.setLabelVisible(false);
      }
      return aw;
   }
   
   
   private static PrimitiveWidget loadPrimitive(ScenicPrimitive sp, 
                                                Scene           s , 
                                                boolean         draggable,
                                                boolean         thumbnail){
      
      PrimitiveWidget pw = (thumbnail)? PrimitiveFactory.createThumbnailWidget(sp) :
                                        PrimitiveFactory.createWidget(sp);
      if(null != pw){
         if(draggable){
            PickupDragController dragCtrl = ScenicToolbar.getDragController();
            dragCtrl.makeDraggable(pw);
            dragCtrl.addDragHandler(pw.getDragHandler());
         }
         pw.setScene(s);
         
         int left = sp.getLeft();
         int top  = sp.getTop();
         
         if(thumbnail){
            int oldWidth   = (int)(pw.getIconMaxWidth() * SequenceFactory.THUMBNAIL_FACTOR);
            int oldHeight  = (int)(pw.getIconMaxHeight() * SequenceFactory.THUMBNAIL_FACTOR);
            int newCenterX = (int)((sp.getLeft() + oldWidth/2)/SequenceFactory.THUMBNAIL_FACTOR);
            int newCenterY = (int)((sp.getTop()  + oldHeight/2)/SequenceFactory.THUMBNAIL_FACTOR);
            left           = newCenterX - (pw.getIconMaxWidth()/2);
            top            = newCenterY - (pw.getIconMaxHeight()/2);
            if(left < 0) left = 0;
            if(top < 0) top = 0;
         }
         s.addPrimitiveWidget(pw, left, top);
         if(thumbnail)pw.setLabelVisible(false);
      }
      return pw;
   }
   
   
   private static void loadConnectors(ScenicPrimitive sp, 
                                      PrimitiveWidget pw, 
                                      Vector<ActorWidget> actorWidgets, 
                                      Scene s){
      Vector<Pair<ScenicActor,ScenicActor>> pairs          = sp.getRelation();
      Vector<ScenicActor>                   actorsIncluded = new Vector<ScenicActor>();
      
      // Get the referenced actor widgets
      for(Pair<ScenicActor,ScenicActor> pair : pairs){
         ScenicActor a1 = pair.getOne();
         ScenicActor a2 = pair.getTwo();
         // a1
         boolean contains = false;
         for(ScenicActor a : actorsIncluded){
            if(a.getId() == a1.getId() && a.getName().equals(a1.getName())){
               contains = true;
               break;
            }
         }
         if(!contains){
            actorsIncluded.add(a1);
         }
         // a2
         contains = false;
         for(ScenicActor a : actorsIncluded){
            if(a.getId() == a2.getId() && a.getName().equals(a2.getName())){
               contains = true;
               break;
            }
         }
         if(!contains){
            actorsIncluded.add(a2);
         }
      }
      
      Vector<ActorWidget> actorWidgetsIncluded = new Vector<ActorWidget>();
      
      for(ScenicActor a : actorsIncluded){
         for(ActorWidget aw : actorWidgets){
            if(!actorWidgetsIncluded.contains(aw)   &&
                a.getName().equals(aw.getActor().getName())){
               actorWidgetsIncluded.add(aw);
            }
         }
      }
      
      // For each actor widget, add a connector from that actor to this primitive
      for(ActorWidget aw : actorWidgetsIncluded){
         ConnectorHandleWidget chw = new ConnectorHandleWidget(aw,pw,0,0,0,0,false);
         aw.addConnectorHandle(chw);
         pw.addConnectorHandle(chw);
         s.addConnectorHandleWidget(chw);
      }
   }
}