package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Connector;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.Scene;


public class ConnectorHandleWidget extends AbsolutePanel {

   protected static final String URL_HANDLE_IMG  = "images/connector-handle.png";
   protected static final String STYLE_CONNECTOR = "connector-handle";
   
   //--//
   
   public Connector       connector;
   public ActorWidget     actorWidget;
   public PrimitiveWidget primitiveWidget;
   
   public int             primitiveX;
   public int             primitiveY;
   public int             actorX;
   public int             actorY;
   
   public boolean         dragging;
   
   //--//

   protected final Image  handleImg;
   
   //--//
   
   protected FocusPanel   shim;
   protected Scene        scene;            // Scene which contains this ConnectorHandleWidget
   
   //--//
   
   /**
    * Creates a new <code>ConnectorHandleWidget</code> with the specified parameters.
    * @param aw
    * @param pw
    * @param primitiveX
    * @param primitiveY
    * @param actorX
    * @param actorY
    */
   public ConnectorHandleWidget(ActorWidget     aw        ,
                                PrimitiveWidget pw        , 
                                int             primitiveX, 
                                int             primitiveY, 
                                int             actorX    ,
                                int             actorY    ,
                                boolean         deleteEnabled) {
      if(null != aw){
         this.connector = new Connector(aw.actor, null);
         this.scene     = aw.scene;
      } else if(null != pw){
         this.connector = new Connector(null, pw.primitive);
         this.scene     = pw.scene;
      } else {this.connector = new Connector(null,null);
         this.scene     = null; 
      }
      this.actorWidget     = aw;
      this.primitiveWidget = pw;
      
      this.primitiveX      = primitiveX;
      this.primitiveY      = primitiveY;
      this.actorX          = actorX;
      this.actorY          = actorY;
      
      this.dragging        = false;
      this.handleImg       = new Image(ConnectorHandleWidget.URL_HANDLE_IMG);      
      this.shim            = new FocusPanel();

      if(deleteEnabled){
         this.add(this.handleImg); 
         
         this.shim.addStyleName("icon-shim");
         this.shim.addStyleName(ConnectorHandleWidget.STYLE_CONNECTOR);
         this.shim.addMouseUpHandler(new MouseUpHandler(){
           public void onMouseUp(MouseUpEvent e){
              if(!dragging){
                 if(e.getNativeButton() == NativeEvent.BUTTON_LEFT){
                    delete();
                 }
                 e.stopPropagation();
              }
            }
         });
      }
   }
   
   
   public int getWidth(){
      return this.handleImg.getWidth();
   }
   
   
   public int getHeight(){
      return this.handleImg.getHeight();
   }
   
   
   public void delete(){
      this.scene.removeConnectorHandleWidget(this);
   }
   
   
   /**
    * Let shim size match our size.
    * 
    * @param width the desired pixel width
    * @param height the desired pixel height
    */
   @Override
   public void setPixelSize(int width, int height) {
      super.setPixelSize(width, height);
      
      this.shim.setPixelSize(width, height);
   }

   /**
    * Let shim size match our size.
    * 
    * @param width the desired CSS width
    * @param height the desired CSS height
    */
   @Override
   public void setSize(String width, String height) {
      super.setSize(width, height);
      
      this.shim.setSize(width, height);
   }
   
   
   /**
    * Adjust the shim size and attach once our widget dimensions are known.
    */
   @Override
   protected void onLoad() {
      super.onLoad();
      
      this.shim.setPixelSize(getOffsetWidth(), getOffsetHeight());
      this.add(this.shim,0,0);
   }

   
   /**
    * Remove the shim to allow the widget to size itself when reattached.
    */
   @Override
   protected void onUnload() {
      super.onUnload();
      
      this.shim.removeFromParent();
   }
}
