package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Actor;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Person;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ScenicToolbar;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorLibrary;

public class PersonWidget extends ActorWidget {

   protected static final String PROPERTIES_TITLE       = "<b>&nbsp;&nbsp;Person Properties</b>";
   protected static final String PROPERTIES_NAME_PREFIX = "Specific Person:&nbsp;&nbsp;";
   protected static final String PROPERTIES_TYPE_PREFIX = "Type of Person:&nbsp;&nbsp;";
   
   //--//
   
   public PersonWidget(LabeledIcon lblIco, Person person){
      super(lblIco, person);
      
      this.propTitle     = new HTML( PersonWidget.PROPERTIES_TITLE       );
      this.nameBoxPrefix = new HTML( PersonWidget.PROPERTIES_NAME_PREFIX );
      this.nameBoxPrefix.setStyleName(ActorWidget.STYLE_ACTOR_PROPERTIES);
      this.typeBoxPrefix = new HTML( PersonWidget.PROPERTIES_TYPE_PREFIX );
      this.typeBoxPrefix.setStyleName(ActorWidget.STYLE_ACTOR_PROPERTIES);
      
      this.buildPropertiesContent();
      this.propDialog.setContent(this.propContent);
   }
   
   
   public PersonWidget(LabeledIcon                   lblIco, 
                       Person                        person, 
                       Vector<ConnectorHandleWidget> connectors){
      this(lblIco, person);
      
      this.connectors = connectors;
   }

   
   @Override
   public IconWidget cloneWidget(){

      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  ,
                                            this.lblIco.defaultImg               , 
                                            new HTML(this.lblIco.label.getHTML()), 
                                            false );
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
         
      return (IconWidget) new PersonWidget(newIco                     , 
                                           (Person)this.actor         ,
                                           connectorClone);
   }
   
   
   @Override
   public IconWidget nextWidget() {
    
      ActorWidget next = ActorFactory.createPersonWidget();
      
      next.setLabelVisible(false);

      return (IconWidget) next;
   }
   
   
   protected ArrayList<String> getPropertiesNames(){
      
      ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
      Vector<Person>    people = lib.getPersons();
      ArrayList<String> names  = new ArrayList<String>();
      
      for(int i = 0; i < people.size(); i++){
         names.add( people.get(i).getName() );
      }
      return names;
   }
   
   
   protected ArrayList<String> getPropertiesTypes(){
      
      ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
      Vector<Person>    types  = lib.getPersonTypes();
      ArrayList<String> names  = new ArrayList<String>();
      
      for(int i = 0; i < types.size(); i++){
         names.add( types.get(i).getName() );
      }
      return names;
   }
   
   
   @Override
   protected Actor findActorByName(String name){
      ActorLibrary lib    = ScenicToolbar.getActorLibrary();
      return lib.getPerson(name);
   }
   
   
   /**
    * Target ok if it is a primitive.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof PrimitiveWidget){
         targetOk = true;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
}