package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Actor;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Place;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ScenicToolbar;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorLibrary;

public class PlaceWidget extends ActorWidget {

   protected static final String PROPERTIES_TITLE       = "<b>&nbsp;&nbsp;Place Properties</b>";
   protected static final String PROPERTIES_NAME_PREFIX = "Specific Place:&nbsp;&nbsp;";
   protected static final String PROPERTIES_TYPE_PREFIX = "Type of Place:&nbsp;&nbsp;";
   
   //--//
   
   public PlaceWidget(LabeledIcon lblIco, Place place){
      super(lblIco, place);
      
      this.propTitle     = new HTML( PlaceWidget.PROPERTIES_TITLE       );
      this.nameBoxPrefix = new HTML( PlaceWidget.PROPERTIES_NAME_PREFIX );
      this.typeBoxPrefix = new HTML( PlaceWidget.PROPERTIES_TYPE_PREFIX );

      this.buildPropertiesContent();
      this.propDialog.setContent(this.propContent);
   }

   
   public PlaceWidget(LabeledIcon                   lblIco, 
                      Place                         place,
                      Vector<ConnectorHandleWidget> connectors){
      this(lblIco, place);
      this.connectors = connectors;
   }
   
   
   @Override
   public IconWidget cloneWidget(){

      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  ,
                                            this.lblIco.defaultImg               , 
                                            new HTML(this.lblIco.label.getHTML()), 
                                            false );
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
         
      return (IconWidget) new PlaceWidget(newIco                    , 
                                          (Place)this.actor         ,
                                          connectorClone);
   }
   
   
   @Override
   public IconWidget nextWidget() {
      
      ActorWidget next =  ActorFactory.createPlaceWidget();
      
      next.setLabelVisible(false);

      return (IconWidget) next;
   }
   
   
   
   protected ArrayList<String> getPropertiesNames(){
      
      ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
      Vector<Place>     places = lib.getPlaces();
      ArrayList<String> names  = new ArrayList<String>();
      
      for(int i = 0; i < places.size(); i++){
         names.add( places.get(i).getName() );
      }
      return names;
   }
   
   
   protected ArrayList<String> getPropertiesTypes(){
      
      ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
      Vector<Place>     types  = lib.getPlaceTypes();
      ArrayList<String> names  = new ArrayList<String>();
      
      for(int i = 0; i < types.size(); i++){
         names.add( types.get(i).getName() );
      }
      return names;
   }
   
   
   @Override
   protected Actor findActorByName(String name){
      ActorLibrary lib = ScenicToolbar.getActorLibrary();
      return lib.getPlace(name);
   }
   
   
   /**
    * Target ok if it is a primitive, not a With or Without primitive, and it is not already
    * connected to a place.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof PrimitiveWidget && !(w instanceof WithWidget) && !(w instanceof WithoutWidget)){
         boolean connectedToPlace = false;
         
         for(ConnectorHandleWidget connector : this.connectors){
            if(connector.actorWidget instanceof PlaceWidget){
               connectedToPlace = true;
               break;
            }
         }
         targetOk = !connectedToPlace;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
}