package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.With;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;

public class WithWidget extends PrimitiveWidget {

	protected static final String PROPERTIES_TITLE = "<b>&nbsp;&nbsp;With Primitive Properties</b>";
	
	//--//

	
	public WithWidget(LabeledIcon lblIco, With primitive){
		super(lblIco, primitive);
		
		this.propTitle = new HTML( WithWidget.PROPERTIES_TITLE );
		
		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}
	
	
	public WithWidget(LabeledIcon                   lblIco, 
	                  With                          primitive,
	                  Vector<ConnectorHandleWidget> connectors){
      this(lblIco, primitive);
      this.connectors = connectors;
   }
	
	
	@Override
   public IconWidget cloneWidget(){
      
      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
                                            this.lblIco.defaultImg               ,
                                            new HTML(this.lblIco.label.getHTML()),
                                            false);
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
      
      return (IconWidget) new WithWidget(newIco        ,
                                        (With)this.primitive,
                                        connectorClone);
   }
	
	
	@Override
   public String getPrimitiveDescription(){
      String         desc       = new String();
      Vector<String> actorNames = new Vector<String>();
      
      for(ConnectorHandleWidget connector : this.connectors){
         if(null != connector.actorWidget){
            if(connector.actorWidget instanceof PersonWidget){
               actorNames.insertElementAt(connector.actorWidget.actor.getName(), 0);
            } else {
               actorNames.add(connector.actorWidget.actor.getName());
            }
         }
      }
      
      
      if(2 <= actorNames.size()){
         desc = "\"";
         if(2 == actorNames.size()){
            desc += actorNames.get(0) + " and " + actorNames.get(1) + " are together.\"";
         } else {
            for(int i = 0; i < actorNames.size() - 1; i++){
               desc += actorNames.get(i) + ", ";
            }
            desc += " and " + actorNames.get(actorNames.size() - 1) + " are together.\"";
         }
      }
      return desc;
   }
	
	
	/**
    * Target ok if it is not a place.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(!(w instanceof PlaceWidget)){
         targetOk = true;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
   
   
   /**
    * For now, valid if connected to at least 2 non-place actors.
    */
   public boolean isValid(){
      boolean valid = false;
      
      // Then check for at least one non-place actor
      if(this.connectors.size() > 1){
         valid = true;
      }
      return valid;
   }
   
   
   //--//
   
   
   protected void buildPropertiesDescription(){
      this.propDescriptionPanel = new HorizontalPanel();
      this.propDescription      = new HTML("&quot;X is with, next to, or holding Y&quot;");
      
      HorizontalPanel descContainer = new HorizontalPanel();
      
      descContainer.add( this.propDescription );
      
      this.propDescriptionPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.propDescriptionPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      this.propDescriptionPanel.add( descContainer );
   }
}
