package edu.washington.cs.rfid.cascadia.scenic.client.util;

import com.google.gwt.event.dom.client.LoadEvent;
import com.google.gwt.event.dom.client.LoadHandler;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Load listener to rescale image and tell the deferred command it has done so onLoad, or not
 * in the case of IE.
 */
public class RescalingLoadListener implements LoadHandler {
   
   public int                         maxWidth;
   public int                         maxHeight;
   public DeferredRescaleCheckCommand deferredCmd;
   
   public HorizontalPanel             hp;
   public int                         hpIndex;
   
   public VerticalPanel               vp;
   public int                         vpIndex;
   
   public AbsolutePanel               ap;
   public int                         x;
   public int                         y;
   
   //--//
   
   public RescalingLoadListener(int maxWidth, int maxHeight, DeferredRescaleCheckCommand deferredCmd){
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.deferredCmd = deferredCmd;
      this.hp          = null;
      this.hpIndex     = -1;
      this.vp          = null;
      this.vpIndex     = -1;
   }
   
   
   public RescalingLoadListener(int maxWidth, int maxHeight, DeferredRescaleCheckCommand deferredCmd, HorizontalPanel hp, int hpIndex){
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.deferredCmd = deferredCmd;
      this.hp          = hp;
      this.hpIndex     = hpIndex;
      this.vp          = null;
      this.vpIndex     = -1;
   }
   
   
   public RescalingLoadListener(int maxWidth, int maxHeight, DeferredRescaleCheckCommand deferredCmd, VerticalPanel vp, int vpIndex){
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.deferredCmd = deferredCmd;
      this.hp          = null;
      this.hpIndex     = -1;
      this.vp          = vp;
      this.vpIndex     = vpIndex;
   }
   
   
   public RescalingLoadListener(int maxWidth, int maxHeight, DeferredRescaleCheckCommand deferredCmd, AbsolutePanel ap, int x, int y){
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.deferredCmd = deferredCmd;
      this.hp          = null;
      this.hpIndex     = -1;
      this.ap          = ap;
      this.x           = x;
      this.y           = y;
   }
   
   
   public void onError(Widget sender) {
      
   }

   
   public void onLoad(LoadEvent e) {
      
      Image img = (Image)e.getSource();
      
      this.deferredCmd.imgRescaled = true;
      
      double curWidth  = (double) img.getWidth();
      double curHeight = (double) img.getHeight();
      
      if((curWidth > 25 && curHeight > 25) || 
              curWidth > this.maxWidth || curHeight > this.maxHeight){

         // Scale down by maximum dimension
         if(curHeight >= curWidth && curHeight > (double)this.maxHeight){
            
            double scaleFactor  = ((double)this.maxHeight) / curHeight;
            curHeight           = (double)this.maxHeight;
            curWidth           *= scaleFactor;
         
         } else if(curWidth > curHeight && curWidth > (double)this.maxWidth){
            
            double scaleFactor = ((double)this.maxWidth) / curWidth;
            curWidth           = (double) this.maxWidth;
            curHeight         *= scaleFactor;
         }
   
         img.setSize( ((int)curWidth) + "px", ((int) curHeight) + "px");
         
         // Now insert into a HorizontalPanel or VerticalPanel if requested
         if(null != this.hp){
            this.hp.insert(img, this.hpIndex);
         } else if(null != this.vp){
            this.vp.insert(img, this.vpIndex);
         } else if(null != this.ap){
            this.ap.remove(0);
            this.ap.add(img, 0, 0);
//            this.ap.setWidgetPosition(img, 0, 0);
            this.ap.setWidth(((int)curWidth) + "px");
            this.ap.setHeight(((int) curHeight) + "px");
         }
      } else {
         this.ap.add(img, 0, 0);
         this.ap.setWidth(((int)curWidth) + "px");
         this.ap.setHeight(((int) curHeight) + "px");
      }
   }
}
