package edu.washington.cs.rfid.cascadia.timeline.client.ui;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.netthreads.gwt.simile.timeline.client.TimeLine;
import com.netthreads.gwt.simile.timeline.client.TimeLineWidget;

import edu.washington.cs.rfid.cascadia.scenic.client.grammar.GrammarEncoder;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPrimitive;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicScene;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicSequence;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.SequencePanel;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.SequenceFactory;
import edu.washington.cs.rfid.cascadia.timeline.client.event.ScenicEventResult;
import edu.washington.cs.rfid.cascadia.timeline.client.event.ScenicEventResultBundle;
import edu.washington.cs.rfid.cascadia.timeline.client.event.ScenicTimelineEvent;
import edu.washington.cs.rfid.cascadia.timeline.client.filter.IScenicFilterMatcher;
import edu.washington.cs.rfid.cascadia.timeline.client.highlighter.IScenicHighlighter;
import edu.washington.cs.rfid.cascadia.timeline.client.popups.BusyPopup;
import edu.washington.cs.rfid.cascadia.timeline.client.popups.ScenicEventBubble;

public class ScenicTimeline extends VerticalPanel {

   private class ClosePopupCmd implements Command {
      public void execute(){
         loadingPopup.hide();
      }
   }
   
   //--//
   
   protected static final String DEFAULT_HEIGHT = "100%";
   protected static final String DEFAULT_WIDTH  = "100%";
   
   //--//
   
   protected static ScenicTimeline instance;
   
   //--//
   
   protected ScenicEventResultBundle                 bundle;
   protected ScenicSequence                          scenicSequence;
   protected TimeLineWidget                          simileWidget;
   protected int                                     width;
   protected int                                     height;
   protected DateTimeFormat                          gregorianFormatter;
   protected ScenicTimelineRender                    renderer;
   protected List<ScenicTimelineEvent>               events;
   protected Map<Integer,List<IScenicFilterMatcher>> bandToFilterMap;
   protected Map<Integer,Boolean>                    filterMatcherSet;
   protected List<IScenicHighlighter>                highlighters;
   protected Map<Integer,Boolean>                    highlighterSet;
   protected ScenicEventBubble                       eventInfoBubble;
   protected Map<ScenicTimelineEvent,ScenicEventBubbleInfo> eventToInfoBubbleMap;
   protected BusyPopup                               loadingPopup;
   
   //--//
   
   /**
    * Creates a new ScenicTimeline
    * 
    * @param h Timeline height
    * @param w Timeline width
    * @param renderer Renderer for the timeline
    */
   protected ScenicTimeline(String h, String w, ScenicEventResultBundle bundle){
      super();
      
      this.bundle = bundle;
      this.scenicSequence = new ScenicSequence(bundle.templateXml);
      
      int suffixLen = 0;
      if(h.endsWith("%")){
         suffixLen = 1;
      } else if(h.endsWith("px")){
         suffixLen = 2;
      }
      int height = Integer.parseInt(h.substring(0,h.length() - suffixLen));
      
      if(w.endsWith("%")){
         suffixLen = 1;
      } else if(w.endsWith("px")){
         suffixLen = 2;
      }
      int width = Integer.parseInt(w.substring(0,w.length() - suffixLen));
      
      this.gregorianFormatter     = DateTimeFormat.getFormat("EEE, dd MMM yyyy HH:mm:ss '+0000'");
      this.renderer               = new ScenicTimelineRender(width,height,bundle);
      this.width                  = this.renderer.width;
      this.height                 = this.renderer.height;
      this.simileWidget           = new TimeLineWidget(h,w,this.renderer);
      this.events                 = new ArrayList<ScenicTimelineEvent>();
      this.bandToFilterMap        = new HashMap<Integer,List<IScenicFilterMatcher>>();
      this.filterMatcherSet       = new HashMap<Integer,Boolean>();
      this.highlighters           = new ArrayList<IScenicHighlighter>();
      this.highlighterSet         = new HashMap<Integer,Boolean>();
      this.eventInfoBubble        = new ScenicEventBubble();
      this.eventToInfoBubbleMap   = new HashMap<ScenicTimelineEvent,ScenicEventBubbleInfo>();
      this.loadingPopup           = new BusyPopup();
      
      this.setShowBubble();
      
      this.add(this.simileWidget);
   }
   
   //--//

   /**
    * Returns the singleton instance of ScenicTimeline.
    * 
    * @param h
    * @param w
    * @return
    */
   public static ScenicTimeline getInstance(String h, String w, ScenicEventResultBundle bundle){
      if(null == ScenicTimeline.instance){
         ScenicTimeline.instance = new ScenicTimeline(h,w, bundle);
      } else {
         // HACK HACK HACK
         ScenicTimeline.instance = new ScenicTimeline(h,w, bundle);
      }
      return instance;
   }
   
   
   /**
    * Returns the singleton instance of ScenicTimeline.
    * 
    * @param h
    * @param w
    * @return
    */
   public static ScenicTimeline getInstance(){
      if(null == ScenicTimeline.instance){
         ScenicTimeline.instance = new ScenicTimeline(ScenicTimeline.DEFAULT_HEIGHT,ScenicTimeline.DEFAULT_WIDTH, null);
      }
      return instance;
   }
   
   //--//
   
   
   public List<SequencePanel> getBandThumbnails(){
      List<SequencePanel> thumbnails = new ArrayList<SequencePanel>();
      
      SequencePanel eventThumb = SequenceFactory.createThumbnailSequencePanel(this.scenicSequence);
      thumbnails.add(eventThumb);
      
      String fromTime = scenicSequence.getFromTime();
      String toTime   = scenicSequence.getToTime();
      String days     = scenicSequence.getDays();
      
      List<ScenicScene> scenes = this.scenicSequence.getScenes();
      for(ScenicScene scene : scenes){
         ScenicSequence seq   = GrammarEncoder.getSubEventScenicSequence(fromTime,toTime,days,scene);
         SequencePanel  thumb = SequenceFactory.createThumbnailSequencePanel(seq);
         thumbnails.add(thumb);
      }
      
      return thumbnails;
   }
   
   
   public List<ScenicEventResult> getBandEvents(){
      return this.renderer.getBandEvents();
   }
   
   
   public int[] getBandOffsets(){
      return this.renderer.getBandOffsets();
   }
   
   
   public String[] getBandLabels(){
      return this.renderer.getBandLabels();
   }
   
   
   public String[] getBandColors(){
      return this.renderer.getBandColors();
   }
   
   
   public List<SequencePanel> getSubEventThumbnails(int bandNum){
      List<SequencePanel> thumbnails = new ArrayList<SequencePanel>();
      String              fromTime   = this.scenicSequence.getFromTime();
      String              toTime     = this.scenicSequence.getToTime();
      String              days       = this.scenicSequence.getDays();
      List<ScenicScene>   scenes     = this.scenicSequence.getScenes();
      
      if(0 == bandNum){
         for(int i = scenes.size()-1; i >= 0; i--){
            // First create the sequence from scenes
            List<ScenicScene> subSeq = new ArrayList<ScenicScene>();
            for(int j = 0; j <= i; j++){
               subSeq.add(scenes.get(j));
            }
            ScenicSequence seq   = GrammarEncoder.getSubEventScenicSequence(fromTime,toTime,days,subSeq);
            
//            // Now set the transitions appropriately
//            List<ScenicTransition> newTransitions = seq.getTransitions();
//            List<ScenicTransition> transitions = this.scenicSequence.getTransitions();
//            for(int j = 0; j < i; j++){
//               ScenicTransition transition = transitions.get(j);
//               newTransitions.get(j).setType(transition.getType());
//            }
            SequencePanel  thumb = SequenceFactory.createThumbnailSequencePanel(seq);
            thumbnails.add(thumb);
         }
      } else if(0 < bandNum && bandNum < scenes.size() + 1){
         ScenicScene             scene      = scenes.get(bandNum - 1);
         Vector<ScenicPrimitive> primitives = scene.getPrimitives();
         
         for(ScenicPrimitive primitive : primitives){
            ScenicSequence seq   = GrammarEncoder.getSubEventScenicSequence(fromTime,toTime,days,primitive);
            SequencePanel  thumb = SequenceFactory.createThumbnailSequencePanel(seq);
            thumbnails.add(thumb);
         }
         
      }
      return thumbnails;
   }
   
   
   public int[] getSubEventOffsets(int bandNum){
      return this.renderer.getSubEventOffsets(bandNum);
   }
   
   
   public String[] getSubEventLabels(int bandNum){
      return this.renderer.getSubEventLabels(bandNum);
   }
   
   
   public List<ScenicEventResult> getSubEvents(int bandNum){
      return this.renderer.getSubEvents(bandNum);
   }
   
   
   /**
    * Adds an event to the timeline.
    * @param e
    */
   public void addEvent(ScenicTimelineEvent e){
      this.events.add(e);
   }
   
   
   /**
    * Loads all events dynamically into the timeline using their XML format.
    */
   public void loadAllEvents(){
      
      this.loadingPopup.show();
      
      StringBuilder xml = new StringBuilder();
      
      xml.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?> <data>");
      for(ScenicTimelineEvent e : this.events){
         xml.append(e.toXml());
      }
      xml.append("</data>");
      
      
      this.simileWidget.getEventSource().loadXMLText(xml.toString());
      
      // TODO: Is the below necessary?
      // We need to send a resize message to 'fix' the display window size.
      //TimeLineTest.getMainPanel().onWindowResized(Window.getClientWidth(), Window.getClientHeight());
      
      DeferredCommand.addCommand(new ClosePopupCmd());
   }
   
   
   public void populateInfoBubbles(){
      
//      this.simileWidget.
      
      for(String key : this.bundle.keyToStreamMap.keySet()){
         List<ScenicTimelineEvent> events = this.bundle.keyToStreamMap.get(key);
         for(ScenicTimelineEvent e : events){
            RootPanel root = RootPanel.get(e.eventId + "-info");
            root.add(new HTML("BONZAI!!"));
         }
      }
   }
   
   
   /**
    * 
    */
   public void addBubbleFiller(){
      this.setBubbleFiller();// TODO:
   }
   
   
   /**
    * Shows the event bubble with information on the containing event.
    * @param e
    * @param x
    * @param y
    */
   public void showEventInfoBubble(SequencePanel thumb,ScenicTimelineEvent e,int x, int y){
      VerticalPanel vp = new VerticalPanel();
      ScenicEventBubbleInfo info = new ScenicEventBubbleInfo(thumb);
      info.setEvent(e);
      vp.add(info);
      this.eventInfoBubble.setContent(vp);
      this.eventInfoBubble.positionRelativeTo(x, y);
      this.eventInfoBubble.show();
      
      // DEBUG BELOW!!
      //thumb.refreshConnectors();
   }
   
   
   /**
    * Hides the event bubble.
    */
   public void hideEventBubbleInfo(){
      this.eventInfoBubble.hide();
   }
   
   
   /**
    * Adds a new filter to the list.
    * @param f
    */
   public void addMatchingFilter(IScenicFilterMatcher f, int band){
      TimeLine timeline = this.simileWidget.getTimeLine();
      
      if(null == this.bandToFilterMap.get(band)){
         this.bandToFilterMap.put(band, new ArrayList<IScenicFilterMatcher>());
      }
      List<IScenicFilterMatcher> filters = this.bandToFilterMap.get(band);
      
      if(!filters.contains(f)){
         filters.add(f);
      }
      if(null == this.filterMatcherSet.get(band)){
         this.setFilterMatcher(timeline,band);
         this.filterMatcherSet.put(band, true);
      }
   }
   
   
   /**
    * Removes filter from the list.
    * @param f
    */
   public void removeMatchingFilter(IScenicFilterMatcher f, int band){
      
      if(null != this.bandToFilterMap.get(band)){
         List<IScenicFilterMatcher> filters = this.bandToFilterMap.get(band);
         while(filters.contains(f)){
            filters.remove(f);
         }   
      }
   }
   
   
   /**
    * Clears the current highlighter and sets a new highlighter for the list.
    * @param h
    */
   public void addHighlighter(IScenicHighlighter h,int band){
      TimeLine timeline = this.simileWidget.getTimeLine();
      this.highlighters.clear();
      this.highlighters.add(h);
      
      if(null == this.highlighterSet.get(band)){
         this.setHighlighter(timeline,band);
         this.highlighterSet.put(band, true);
      }
   }
   
   
   /**
    * Finds and returns an event using its eventID.
    * @param eventId
    * @return
    */
   public ScenicTimelineEvent getEventById(String eventId){
      ScenicTimelineEvent match = null;
      
      for(ScenicTimelineEvent e : this.events){
         if(null != e.eventId && e.eventId.equals(eventId)){
            match = e;
            break;
         }
      }
      return match;
   }
   
   
   /**
    * Sets the center of the timeline to a particular date.
    * @param date
    */
   public void setCenterVisibleDate(Date date){
      TimeLine timeline = this.simileWidget.getTimeLine();
      String   dateStr  = this.gregorianFormatter.format(date);
      
      this.setCenterVisibleDate(timeline,dateStr,0);
   }
   
   
   /**
    * Sets the center of the timeline to a particular date.
    * @param date
    */
   public void setCenterVisibleDate(String date){
      TimeLine timeline = this.simileWidget.getTimeLine();
      
      this.setCenterVisibleDate(timeline,date,0);
   }
   
   
   /**
    * Scrolls the center of the timeline to a particular date.
    * @param date
    */
   public void scrollToCenter(Date date){
      TimeLine timeline = this.simileWidget.getTimeLine();
      String   dateStr  = this.gregorianFormatter.format(date);
      
      this.scrollToCenter(timeline,dateStr,0);
   }
   
   
   /**
    * Scrolls the center of the timeline to a particular date.
    * @param date
    */
   public void scrollToCenter(String date){
      TimeLine timeline = this.simileWidget.getTimeLine();
      
      this.scrollToCenter(timeline,date,0);
   }
   
   
   /**
    * Filters a given event using the disjunction of all current filters.
    * Called by the JavaScript filtering function.
    * @param e
    * @return
    */
   public boolean filterEvent(ScenicTimelineEvent e, int band){
      boolean pass = false;
      
      List<IScenicFilterMatcher> filters = this.bandToFilterMap.get(band);
      
      if(null != filters){
         for(IScenicFilterMatcher filter : filters){
            if(filter.doFilter(e)){
               pass = true;
               break;
            }
         }
      }
      return pass;
   }
   
   
   /**
    * Highlights the given event using the disjunction of all current
    * highlighters - takes the highest matching highlighter index.
    * Only events which pass the filter are highlighted.
    * Called by the JavaScript highlighting function.
    * @param e
    * @return
    */
   public int highlightEvent(ScenicTimelineEvent e, int band){
      int index = -1;
      
      if(this.filterEvent(e,band)){
         for(IScenicHighlighter h : this.highlighters){
            int result = h.getHighlightingIndex(e);
            index = (index < result)? result : index;
         }
      }
      return index;
   }
   
   
   /**
    * Repaints the timeline.
    */
   public void repaintTimeline(){
      this.repaintTimeline(this.simileWidget.getTimeLine());
   }
   
   
   //--//
   
   
//   private void populateIdToBubbleMap(){
//      for(String key : this.bundle.keyToStreamMap.keySet()){
//         List<ScenicTimelineEvent> events = this.bundle.keyToStreamMap.get(key);
//         for(ScenicTimelineEvent e : events){
//            ScenicEventBubbleInfo info = new ScenicEventBubbleInfo(e);
//            this.eventToInfoBubbleMap.put(e, info);
//         }
//      }
//   }
   
   
   /**
    * Scrolls the center of the timeline in the specified band to the specified date.
    */
   private native void scrollToCenter(TimeLine timeline, String date, int band) /*-{
      timeline.getBand(band).scrollToCenter($wnd.Timeline.DateTime.parseGregorianDateTime(date));
   }-*/;
   
   
   /**
    * Sets the center of the timeline in the specified band to the specified date.
    */
   private native void setCenterVisibleDate(TimeLine timeline, String date, int band) /*-{
      timeline.getBand(band).setCenterVisibleDate($wnd.Timeline.DateTime.parseGregorianDateTime(date));
   }-*/;
   
   
   public native void setShowBubble() /*-{
     $wnd.Timeline.OriginalEventPainter.prototype._showBubble = function(x, y, evt) {
        @edu.washington.cs.rfid.cascadia.timeline.client.ui.ScenicTimeline::showBubble(Ljava/lang/String;II)(evt.getEventID(),x,y);
     }
   }-*/;
   
   
   /**
    * Sets the bubble filling function for this timeline.
    * NOTE: see sources.js line 522 for the original source filer function.
    * @param timeline
    */
   private native void setBubbleFiller() /*-{
      
      $wnd.Timeline.DefaultEventSource.Event.prototype.fillInfoBubble = 
         function(elmt, theme, labeller) {
            var doc       = elmt.ownerDocument;           
            var title     = this.getText();
            var divTitle  = doc.createElement("div");
            var textTitle = doc.createTextNode(title);
            var evId      = this.getEventID();
            
            divTitle.appendChild(textTitle);
            theme.event.bubble.titleStyler(divTitle);
            elmt.appendChild(divTitle);
           
            var eventObject = this;
            var div         = document.createElement("div");
            
            
            div.innerHTML = "<table align=center><tr><td id=\"" + evId + "-info\"></td></tr></table>";
//            div.innerHTML   = "<div id=\"" + evId + "-info\">Loading...</div>";
            //+ this._description;
            elmt.appendChild(div);
         };
   }-*/;
   
   
   /**
    * Adds a generic filterMatcher function to band 0's event painter.
    * 
    * NOTE: Look in source.js, line 425 for more on contents of the evt object
    * @param timeline
    */
   private native void setFilterMatcher(TimeLine timeline, int band) /*-{
      filterMatcher = function(evt) {
         return @edu.washington.cs.rfid.cascadia.timeline.client.ui.ScenicTimeline::filterEvent(Ljava/lang/String;I)(evt.getEventID(),band);
      };
      timeline.getBand(band).getEventPainter().setFilterMatcher(filterMatcher);
   }-*/;
   
   
   /**
    * Adds a generic highlighterMatcher function to band 0's event painter.
    * 
    * NOTE: Look in source.js, line 425 for more on contents of the evt object
    * @param timeline
    */
   private native void setHighlighter(TimeLine timeline, int band) /*-{
      myHighlightMatcher = function(evt){
         return @edu.washington.cs.rfid.cascadia.timeline.client.ui.ScenicTimeline::highlightEvent(Ljava/lang/String;I)(evt.getEventID(),band);
      };
      timeline.getBand(band).getEventPainter().setHighlightMatcher(myHighlightMatcher);
   }-*/;
   
   
   
   /**
    * Shows the event bubble for the specified event at the specified coordinates.
    * @param eventId
    * @param x
    * @param y
    */
   private static void showBubble(String eventId, int x, int y){
      ScenicTimelinePanel      timelinePanel = ScenicTimelinePanel.getInstance();
      ScenicTimelineEventLabel label         = timelinePanel.getEventLabelById(eventId);
      SequencePanel            thumb         = null;
      if(null != label){
         thumb = label.getThumbnail();
      }
      ScenicTimeline      timeline = ScenicTimeline.getInstance();
      ScenicTimelineEvent e        = timeline.getEventById(eventId);
      timeline.showEventInfoBubble(thumb, e, x, y);
      
   }
   
   
   /**
    * Maps an event ID to an event in the singleton's event set 
    * and filters that event using the singleton's current set of filters.
    * @param id
    * @param trackNum
    */
   private static boolean filterEvent(String eventId, int band){
      ScenicTimeline      timeline = ScenicTimeline.getInstance();
      ScenicTimelineEvent e        = timeline.getEventById(eventId);
      
      return timeline.filterEvent(e,band);
   }
   
   
   /**
    * Maps an event ID to an event in the singleton's event set 
    * and highlights that event using the singleton's current set of highlighters.
    * @param eventId
    * @return
    */
   private static int highlightEvent(String eventId, int band){
      ScenicTimeline      timeline = ScenicTimeline.getInstance();
      ScenicTimelineEvent e        = timeline.getEventById(eventId);
      
      return timeline.highlightEvent(e,band);
   }
   
   
   /**
    * Repaints the timeline, allows the result of new filters to take affect.
    * @param timeline
    */
   private native void repaintTimeline(TimeLine timeline) /*-{
      timeline.paint();
   }-*/;
}