package edu.washington.cs.rfid.cascadia.timeline.client.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.SequencePanel;
import edu.washington.cs.rfid.cascadia.timeline.client.event.ScenicTimelineEvent;
import edu.washington.cs.rfid.cascadia.timeline.client.filter.EventStreamFilter;
import edu.washington.cs.rfid.cascadia.timeline.client.filter.IScenicFilterMatcher;
import edu.washington.cs.rfid.cascadia.timeline.client.highlighter.EventTreeHighlighter;
import edu.washington.cs.rfid.cascadia.timeline.client.highlighter.IScenicHighlighter;

public class ScenicTimelineEventLabel extends HorizontalPanel {

   protected static final String STYLE_TIMELINE_OVERLAY = "timeline-overlay";
   
   //--//
   
   protected ScenicTimeline                        timeline;
   protected SequencePanel                         thumbnail;
   protected int                                   band;
   protected String                                streamKey;
   protected HashMap<String,ArrayList<ScenicTimelineEvent>> keyToStreamMap;
   protected IScenicFilterMatcher                  streamFilter;
   
   protected FocusPanel                            focusPanel;
   protected HTML                                  titleHtml;
   protected String                                description;
   protected CheckBox                              filterCb;
   
   
   
   public ScenicTimelineEventLabel(ScenicTimeline                        timeline      , 
                                   SequencePanel                         thumbnail     ,
                                   String                                streamKey     , 
                                   HashMap<String,ArrayList<ScenicTimelineEvent>> keyToStreamMap,
                                   String                                title         ,
                                   String                                description   ,
                                   int                                   band          ,
                                   String                                bandStyle){

      this.timeline        = timeline;
      this.thumbnail       = thumbnail;
      this.band            = band;
      this.streamKey       = streamKey;
      this.keyToStreamMap  = keyToStreamMap;
      this.streamFilter    = new EventStreamFilter(this.keyToStreamMap.get(this.streamKey));
      
      this.focusPanel      = new FocusPanel();
      
      HorizontalPanel hp   = new HorizontalPanel();
      int streamSize       = this.keyToStreamMap.get(this.streamKey).size();
      this.titleHtml       = new HTML(title + "&nbsp;&nbsp;( " + streamSize + " )");
      this.titleHtml.setStyleName("gwt-HTML-small");
      this.description     = description;
      this.filterCb        = new CheckBox();
      this.filterCb.addValueChangeHandler(new CbValueChangeHandler());
      
      hp.add(this.filterCb );
      hp.add(this.titleHtml);
      
      this.focusPanel.setWidget(hp);
      this.focusPanel.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            doHighlighting();
         }
      });
      
      this.filterCb.setStyleName(bandStyle);
      this.focusPanel.setStyleName(bandStyle);
      this.setStyleName(bandStyle);
      
      this.filterCb.addStyleName(ScenicTimelineEventLabel.STYLE_TIMELINE_OVERLAY);
      this.focusPanel.addStyleName(ScenicTimelineEventLabel.STYLE_TIMELINE_OVERLAY);
      this.addStyleName(ScenicTimelineEventLabel.STYLE_TIMELINE_OVERLAY);
      
      this.add(this.focusPanel);
   }
 
   
   /**
    * Sets value for check box (checked or not).
    * @param checked
    */
   public void setValue(boolean checked){
      if(checked){
         this.timeline.addMatchingFilter(this.streamFilter,this.band);
      } else {
         this.timeline.removeMatchingFilter(this.streamFilter,this.band);
      }
      this.timeline.repaintTimeline();
      this.filterCb.setValue(checked); 
   }
   
   
   public void setValueNoRepaint(boolean checked){
      if(checked){
         this.timeline.addMatchingFilter(this.streamFilter,this.band);
      } else {
         this.timeline.removeMatchingFilter(this.streamFilter,this.band);
      }
      this.filterCb.setValue(checked); 
   }
   
   
   /**
    * Triggers highlighting for events in the tree rooted at this item.
    */
   public void doHighlighting(){
      this.timeline.addHighlighter(this.getHighlighter(),this.band);
      this.timeline.repaintTimeline();
   }
   
   
   public SequencePanel getThumbnail(){
      return this.thumbnail;
   }
   
   
   /**
    * Gets the matching filter for this tree item's top-level event stream.
    * @return
    */
   public IScenicFilterMatcher getFilter(){
      return this.streamFilter;
   }
   
   
   /**
    * Gets a highlighter for the events in this ScenicEventTreeItem and its children.
    * @return
    */
   public IScenicHighlighter getHighlighter(){
      return this.buildHighlighter();
   }
   
   
   //--//
   
   
   /**
    * Builds a highlighter for this tree item that highlights all events
    * at this level as well as all events at all sub-levels.
    * @return
    */
   protected IScenicHighlighter buildHighlighter(){
      List<ScenicTimelineEvent> list        = this.getEventList();
      IScenicHighlighter        highlighter = new EventTreeHighlighter(list);
      return highlighter;
   }
   
   
   /**
    * Recursively retrieves a list of all ScenicTimelineEvents in the tree
    * rooted at this event.
    * @return
    */
   protected List<ScenicTimelineEvent> getEventList(){
      List<ScenicTimelineEvent> list    = new ArrayList<ScenicTimelineEvent>();
      List<ScenicTimelineEvent> toClone = this.keyToStreamMap.get(this.streamKey);      
      for(ScenicTimelineEvent e : toClone){
         list.add(e);
      }
      return list;
   }
   
   
   //--//
   
   
   /**
    * Custom ClickHandler for the TreeItem check box.
    * @author evan
    */
   private class CbValueChangeHandler implements ValueChangeHandler<Boolean> {
      
      public void onValueChange(ValueChangeEvent<Boolean> e){
         if(e.getValue()){
            timeline.addMatchingFilter(streamFilter,band);
         } else {
            timeline.removeMatchingFilter(streamFilter,band);
         }
         timeline.repaintTimeline();
      }
   }
}
