package edu.washington.cs.rfid.cascadia.timeline.server;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import edu.washington.cs.rfid.cascadia.scenic.client.data.EventTemplate;

public class TemplateManager {

   private static final String FILE_TEMPLATE_FILENAMES = "C:\\template-list.txt";
   private static final String FILE_TEMPLATE_HEADER    = "#";

   //--//
   
   private List<EventTemplate> templates;
   
   //--//
   
   /**
    * Creates a new template manager
    */
   public TemplateManager(){
      this.templates = new Vector<EventTemplate>();
   }
   

   /**
    * Loads the templates from file.
    */
   public void loadTemplates(){
      
      // Read in the template filenames.
      Vector<String> filenames = new Vector<String>();
      BufferedReader reader    = null;
      
      try {
         reader      = new BufferedReader(new FileReader(TemplateManager.FILE_TEMPLATE_FILENAMES));
         String line = null;
         
         while(null != (line = reader.readLine())){
            filenames.add(line);
         }
      } catch(FileNotFoundException e){
         e.printStackTrace();
      } catch(IOException e){
         e.printStackTrace();
      } finally {
         try {
            if(null != reader){
               reader.close();
            }
         } catch(IOException e){
            e.printStackTrace();
         }
      }
      
      // Now read in the templates
      for(String filename : filenames){
         try {
            EventTemplate template = this.readTemplateFile(filename);
            if(null != template){
               this.templates.add(template);
            }
         } catch(FileNotFoundException e){
            e.printStackTrace();
         } catch(IOException e){
            e.printStackTrace();
         }
      }
   }
   
   
   public EventTemplate getTemplate(int i){
      EventTemplate template = null;
      
      if(i >= 0 && this.templates.size() > i){
         template = this.templates.get(i);
      }
      return template;
   }
   
   //--//
   
   /**
    * Reads in the template from the specified file.
    * @param filename
    * @return
    * @throws FileNotFoundException
    * @throws IOException
    */
   private EventTemplate readTemplateFile(String filename) throws FileNotFoundException, IOException {
      
      EventTemplate  template = null;
      BufferedReader reader   = new BufferedReader(new FileReader(filename));
      String         line     = null;
         
      try {
         String title           = null;
         String description     = null;
         String exampleApp      = null;
         String exampleScenario = null;
         String xml             = new String();
         
         while(null != (line = reader.readLine())){
            if(line.startsWith(TemplateManager.FILE_TEMPLATE_HEADER)){
               if(null == title){
                  title = line.substring(1);
               } else if(null == description){
                  description = line.substring(1);
               } else if(null == exampleApp){
                  exampleApp = line.substring(1);
               } else {
                  exampleScenario = line.substring(1);
               }
            } else {
               xml += line;
            }
         }
         if(null != title && null != description && null != xml){
            template = new EventTemplate(title, description, exampleApp, exampleScenario, xml);
         }
      } finally {
         try {
            if(null != reader){
               reader.close();
            }
         } catch(IOException e){
            e.printStackTrace();
         }
      }
      return template;
   }   
}
