package edu.washington.cs.rfid.cascadia.playback.client;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.playback.client.data.PlaybackOptions;
import edu.washington.cs.rfid.cascadia.playback.client.event.FastForwardEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.PauseEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.PlayEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.RestartEvent;
import edu.washington.cs.rfid.cascadia.playback.client.event.SlowDownEvent;
import edu.washington.cs.rfid.cascadia.playback.client.handler.FastForwardHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.PauseHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.PlayHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.RestartHandler;
import edu.washington.cs.rfid.cascadia.playback.client.handler.SlowDownHandler;
import edu.washington.cs.rfid.cascadia.playback.client.ui.PlaybackPanel;
import edu.washington.cs.rfid.map.client.data.LocationData;
import edu.washington.cs.rfid.map.client.data.TrackingData;
import edu.washington.cs.rfid.map.client.data.cache.DefaultMapImages;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class PlaybackWidget { //implements EntryPoint {

   /**
    * Create a remote service proxy to talk to the server-side Greeting service.
    */
   private final GreetingServiceAsync greetingService = GWT.create(GreetingService.class);

   protected PlaybackPanel playbackPanel;
   protected PlaybackTimer timer;
   protected int           time;
   
   
   /**
    * This is the entry point method.
    */
   public void onModuleLoad() {
      
      VerticalPanel mainPanel = new VerticalPanel();
      this.playbackPanel      = new PlaybackPanel();
      this.timer              = new PlaybackTimer();
      time                    = 0;
      
      Button a = new Button("Initalize Maps");
      a.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            List<PlaybackOptions> playbackOptions = new ArrayList<PlaybackOptions>();
            PlaybackOptions       options         = new PlaybackOptions("evan-trace",
                                                                        "Evan",
                                                                        "Evan",
                                                                        DefaultMapImages.ORANGE_PERSON     ,
                                                                        DefaultMapImages.ICON_ORANGE_PERSON,
                                                                        DefaultMapImages.ORANGE_PARTICLE   ,
                                                                        DefaultMapImages.ICONS_URL_ORANGE_PARTICLES[0]);
            PlaybackOptions       optionsB        = new PlaybackOptions("jane-trace",
                                                                        "Jane",
                                                                        "Jane",
                                                                        DefaultMapImages.BLUE_PERSON     ,
                                                                        DefaultMapImages.ICON_BLUE_PERSON,
                                                                        DefaultMapImages.BLUE_PARTICLE   ,
                                                                        DefaultMapImages.ICONS_URL_BLUE_PARTICLES[0]);
            
            playbackOptions.add(options);
            playbackOptions.add(optionsB);
            playbackPanel.generatePlaybackWidgets(playbackOptions);
         }
      });
      
      Button b = new Button("Advance Timestep");
      b.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            greetingService.getLocationData(time, new AsyncCallback<LocationData>() {
               public void onFailure(Throwable caught){}
               
               public void onSuccess(LocationData data){
                  Map<String,LocationData> updates = new HashMap<String,LocationData>();
                  updates.put("Evan", data);
                  updates.put("Jane", data);
                  
                  playbackPanel.updateLocation(updates);
               }
            });
            time++;
         }
      });
      
      Button c = new Button("Launch Timer");
      c.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            timer.schedule(1000);
         }
      });
      
      mainPanel.add(playbackPanel);
      mainPanel.add(a);
      mainPanel.add(b);
      mainPanel.add(c);
      
      RootPanel.get().add(mainPanel);
   }
   
   
   
   
   
   // ---------------------------- PLAYBACK TIMER ---------------------------------
   
   public class PlaybackTimer extends Timer {
      
      private static final int REAL_TIME_TIMEOUT        = 1000;
      private static final int DEFAULT_PLAYBACK_TIMEOUT = 250;
      
      //--//
      
      private int                      curPlaybackIdx;
      private int                      curPlaybackTimeout;
      private boolean                  playing;
      private Map<String,TrackingData> trackingDataMap;
      
      //--//
      
      public PlaybackTimer(){
         this.curPlaybackIdx     = 0;
         this.playing            = false;
         this.curPlaybackTimeout = PlaybackTimer.REAL_TIME_TIMEOUT;
         
         // Create the map of keys (entity names) to tracking data for the maps
         this.trackingDataMap    = new HashMap<String,TrackingData>();
         
         playbackPanel.addSlowDownHandler(new SlowDownHandler(){
            public void onSlowDown(SlowDownEvent e){
               slowDown();
               String speed = getSpeedFactor();
               playbackPanel.setPlaybackSpeedLabel("( ~" + speed + " )");
            }
         });
         
         playbackPanel.addFastForwardHandler(new FastForwardHandler(){
            public void onFastForward(FastForwardEvent e){
               speedUp();
               String speed = getSpeedFactor();
               playbackPanel.setPlaybackSpeedLabel("( ~" + speed + " )");
            }
         });
         
         playbackPanel.addPauseHandler(new PauseHandler(){
            public void onPause(PauseEvent e){
               cancel();
               playing = false;
               
               playbackPanel.setPlaybackSpeedLabel("( paused ) ");
            }
         });
         
         playbackPanel.addPlayHandler(new PlayHandler(){
            public void onPlay(PlayEvent e){
               if(playing){
                  curPlaybackTimeout = 1000;
               } else {
                  playing = true;
                  schedule(100);
               }
               String speed = getSpeedFactor();
               playbackPanel.setPlaybackSpeedLabel("( ~" + speed + " )");
            }
         });
         
         playbackPanel.addRestartHandler(new RestartHandler(){
            public void onRestart(RestartEvent e){
               curPlaybackIdx = 0;
               cancel();
               playing = false;
               playbackPanel.setPlaybackSpeedLabel("( stopped )");
               playbackPanel.clearAllMaps();
            }
         });
         
      }
      
      
      public void run(){
         if(this.playing){
            boolean reschedule = true;
            Date    newDate    = null;
            
            if(this.curPlaybackIdx < 5){
               
               greetingService.getLocationData(curPlaybackIdx, new AsyncCallback<LocationData>() {
                  public void onFailure(Throwable caught){}
                  
                  public void onSuccess(LocationData data){
                     Map<String,LocationData> updates = new HashMap<String,LocationData>();
                     updates.put("Evan", data);
                     updates.put("Jane", data);
                     
                     playbackPanel.updateLocation(updates);
                     
                     Date date = new Date(System.currentTimeMillis());
                     playbackPanel.updateTimeLabel(date.toString() + " index : " + curPlaybackIdx);
                                          
                     curPlaybackIdx++;
                     schedule(curPlaybackTimeout);
                  }
               });
               
            } else {
               reschedule = false;
               this.playing = false;
               Window.alert("Playback has ended!");
            }
         }
      }
      
      
      public String speedUp(){
         String speed = "";
         
         int tempTimeout = this.curPlaybackTimeout;
         tempTimeout    /= 2;
         
         if(tempTimeout > 100){
            this.curPlaybackTimeout = tempTimeout;
         }
         return this.getSpeedFactor();
      }
   
   
      public String slowDown(){
         String speed = "";
         
         int tempTimeout = this.curPlaybackTimeout;
         tempTimeout    *= 2;
         
         if(tempTimeout > 100){
            this.curPlaybackTimeout = tempTimeout;
         }
         return this.getSpeedFactor();
      }
         
      
      public void stop(){
         this.cancel();
         this.curPlaybackIdx = 0;
         
         this.playing = false;
      }
      
      private String getSpeedFactor(){
         String speed = "";
         int factor = PlaybackTimer.REAL_TIME_TIMEOUT / this.curPlaybackTimeout;
         if(factor > 0){
            speed = factor + "x";
         } else {
            factor = this.curPlaybackTimeout / PlaybackTimer.REAL_TIME_TIMEOUT;
            speed  = "1/" + factor;
         }
         return speed;
      }
   }
   
}
