package edu.washington.cs.rfid.cascadia.playback.server;

import java.util.ArrayList;

import edu.washington.cs.rfid.cascadia.playback.client.GreetingService;
import edu.washington.cs.rfid.map.client.data.LatLon;
import edu.washington.cs.rfid.map.client.data.LocationData;
import edu.washington.cs.rfid.map.client.data.Particle;
import edu.washington.cs.rfid.map.client.data.TagReading;
import edu.washington.cs.rfid.map.client.data.cache.DefaultMapFloors;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class GreetingServiceImpl extends RemoteServiceServlet implements
      GreetingService {

   public String greetServer(String input) {
      String serverInfo = getServletContext().getServerInfo();
      String userAgent = getThreadLocalRequest().getHeader("User-Agent");
      return "Hello, " + input + "!<br><br>I am running " + serverInfo
            + ".<br><br>It looks like you are using:<br>" + userAgent;
   }
   
   public LocationData getLocationData(int timestep){
      
      TagReading          r         = this.getDataPoint(timestep);
      ArrayList<Particle> particles = this.getParticles(timestep);
      
      return new LocationData(r,particles);
   }
   
   
   public TagReading getDataPoint(int timestep){
      TagReading r = null;
      
      switch(timestep){
         case 1:
            r = new TagReading(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635));
            break;
         case 2:
            r = new TagReading(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307));
            break;
         case 3:
            r = new TagReading(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799));
            break;
         case 4:
            r = new TagReading(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664));
            break;
         default:
            break;
      }
      return r;
   }
   
   
   public ArrayList<Particle> getParticles(int timestep){
      ArrayList<Particle> particles = new ArrayList<Particle>();
      
      switch(timestep){
         case 1:
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.6));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.3));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.05));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.05));
            
            break;
         case 2:
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.4));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.27));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.04));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.23));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664), 0.002));
            
            break;
         case 3:
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.1));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.4));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.009));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664), 0.13));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.002));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.653154619282,-122.305763885379), 0.012));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.653186237951,-122.3059040308), 0.002));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6531496506323,-122.305802106857), 0.001));
            particles.add(new Particle(DefaultMapFloors.DEFAULT_FLOORS[3], new LatLon(47.6532205667699,-122.305940911174), 0.001));
            
            break;
         default:
            break;
      }
      return particles;
   }
}
