package edu.washington.cs.rfid.cascadia.scenic.client.grammar;

import java.util.ArrayList;

import com.google.gwt.user.client.rpc.IsSerializable;

import edu.washington.cs.rfid.cascadia.scenic.client.xml.Node;

public class ScenicActor implements IsSerializable {

   protected final static String ID     = "id";
   protected final static String NAME   = "name";
   protected final static String TYPE   = "type";
   protected final static String LEFT   = "left";
   protected final static String TOP    = "top";
   protected final static String IMGURL = "imgurl";
   
   //--//
   
   protected int     id;           // ID of this actor
   protected String  name;         // Name of this actor
   protected boolean type;         // True if this Actor is a type
   protected int     left;         // Left pixel coordinate of this actor's widget
   protected int     top;          // Top pixel coordinate of this actor's widget
   protected String  imgUrl;       // URL for the actor's icon image
   
   //--//
   
   public ScenicActor(){
      this.id     = -1;
      this.name   = null;
      this.type   = false;
      this.left   = -1;
      this.top    = -1;
      this.imgUrl = null;
   }
   
   public ScenicActor(ArrayList<Node> kids){
       
      for(int i = 0; i < kids.size(); i++){
	  
         Node kid = kids.get(i);
	  
         if(kid.getName().equals(ScenicActor.ID)){
            try {
               this.id = Integer.parseInt(((Node)kid.getChildNodes().get(0)).getValue());
            } catch(NumberFormatException e){
        	this.id = -1;
            }
         } else if(kid.getName().equals(ScenicActor.NAME)){
            this.name = ((Node)kid.getChildNodes().get(0)).getValue();
         } else if(kid.getName().equals(ScenicActor.TYPE)){
            this.type = Boolean.parseBoolean(((Node)kid.getChildNodes().get(0)).getValue());
         } else if(kid.getName().equals(ScenicActor.LEFT)){
            try {
               this.left = Integer.parseInt(((Node)kid.getChildNodes().get(0)).getValue());
            } catch(NumberFormatException e){
               this.left = 0;
            }
         } else if(kid.getName().equals(ScenicActor.TOP)){
            try {
               this.top = Integer.parseInt(((Node)kid.getChildNodes().get(0)).getValue());
            } catch(NumberFormatException e){
               this.top = 0;
            }
         } else if(kid.getName().equals(ScenicActor.IMGURL)){
            this.imgUrl = ((Node)kid.getChildNodes().get(0)).getValue();
         }
      }
   }
   
   public ScenicActor(int id, String name, boolean type, int left, int top, String imgUrl){
      this.id     = id;
      this.name   = name;
      this.type   = type;
      this.left   = left;
      this.top    = top;
      this.imgUrl = imgUrl;
   }
   
   public long getId(){
      return this.id;
   }
   
   public String getName(){
      return this.name;
   }
   
   public boolean getType(){
      return this.type;
   }
   
   public int getLeft(){
      return this.left;
   }
   
   public int getTop(){
      return this.top;
   }
   
   public String getImgUrl(){
       return this.imgUrl;
   }
   
   public void setId(int id){
      this.id = id;
   }
   
   public void setName(String name){
      this.name = name;
   }
   
   public void setLeft(int left){
      this.left = left;
   }
   
   public void setTop(int top){
       this.top = top;
   }
   
   public void setImgUrl(String imgUrl){
       this.imgUrl = imgUrl;
   }
   
   public String toXml(){
       return this.toXml(0);
   }
   
   /**
    * Generates an XML representation of this object.
    * @return
    */
   public String toXml(int level){
       
       String indent = "";
       
       for(int i = 0; i < level; i++){
	   indent += "\t";
       }
       
       String xml = indent + "<scenicactor>\n" + 
                    indent + "\t<id>"     + this.id     + "</id>\n"     +
                    indent + "\t<name>"   + this.name   + "</name>\n"   +
                    indent + "\t<type>"   + this.TYPE   + "</type>\n"   +
                    indent + "\t<left>"   + this.left   + "</left>\n"   + 
                    indent + "\t<top>"    + this.top    + "</top>\n"    +
                    indent + "\t<imgurl>" + this.imgUrl + "</imgurl>\n" +
                    indent + "</scenicactor>\n";
       return xml;
   }
}