package edu.washington.cs.rfid.cascadia.scenic.client.grammar;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.user.client.rpc.IsSerializable;

import edu.washington.cs.rfid.cascadia.scenic.client.xml.Document;
import edu.washington.cs.rfid.cascadia.scenic.client.xml.Node;

public class ScenicSequence implements IsSerializable {

   protected static final String SCENIC_SEQUENCE   = "scenicsequence";
   protected static final String SCENIC_ACTOR      = "scenicactor";
   protected static final String SCENIC_PERSON     = "scenicperson";
   protected static final String SCENIC_PERSON_GRP = "scenicpersongrp";
   protected static final String SCENIC_THING      = "scenicthing";
   protected static final String SCENIC_THING_GRP  = "scenicthinggrp";
   protected static final String SCENIC_PLACE      = "scenicplace";
   protected static final String SCENIC_SCENE      = "scenicscene";
   protected static final String SCENIC_TRANSITION = "scenictransition";
   
   //--//
   
   protected Vector<ScenicActor>      actors;         // Actors referenced in the specification
   protected Vector<ScenicScene>      scenes;         // Scenes in the specification
   protected Vector<ScenicTransition> transitions;    // Transitions in the specification
   
   //--//
   
   public ScenicSequence(){
      this.actors      = new Vector<ScenicActor>();
      this.scenes      = new Vector<ScenicScene>();
      this.transitions = new Vector<ScenicTransition>();
   }
   
   public ScenicSequence(String xml){
      this.actors             = new Vector<ScenicActor>();
      this.scenes             = new Vector<ScenicScene>();
      this.transitions        = new Vector<ScenicTransition>();
      Document        doc     = Document.xmlParse(xml);
      ArrayList<Node> topNode = doc.getChildren();
      ArrayList<Node> kids    = null;

      if(topNode.size() > 0 && topNode.get(0).getName().equals(ScenicSequence.SCENIC_SEQUENCE)){
         kids = topNode.get(0).getChildNodes();

         for(int i = 0; i < kids.size(); i++){
            Node   n    = kids.get(i);
            String name = n.getName();

            if(name.equals(ScenicSequence.SCENIC_ACTOR)){
               this.actors.add(new ScenicActor(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_PERSON)){
               this.actors.add(new ScenicPerson(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_PERSON_GRP)){
               this.actors.add(new ScenicPersonGrp(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_THING)){
               this.actors.add(new ScenicThing(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_THING_GRP)){
               this.actors.add(new ScenicThingGrp(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_PLACE)){
               this.actors.add(new ScenicPlace(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_SCENE)){
               this.scenes.add(new ScenicScene(n.getChildNodes()));
            } else if(name.equals(ScenicSequence.SCENIC_TRANSITION)){
               this.transitions.add(new ScenicTransition(n.getChildNodes()));
            }
         }
      } else {
         this.actors      = new Vector<ScenicActor>();
         this.scenes      = new Vector<ScenicScene>();
         this.transitions = new Vector<ScenicTransition>();
      }
   }
   
   public ScenicSequence(Vector<ScenicActor>      actors, 
                         Vector<ScenicScene>      scenes,
                         Vector<ScenicTransition> transitions){
      this.actors      = actors;
      this.scenes      = scenes;
      this.transitions = transitions;
   }
   
   public Vector<ScenicActor> getActors(){
      return this.actors;
   }
   
   public Vector<ScenicScene> getScenes(){
      return this.scenes;
   }
   
   public Vector<ScenicTransition> getTransitions(){
      return this.transitions;
   }
   
   public void addActor(ScenicActor actor){
      if (!this.actors.contains( actor )){
         this.actors.add( actor );
      }
   }
   
   public void addScene(ScenicScene scene){
      this.scenes.add(scene);
   }
   
   public void addTransition(ScenicTransition transition){
      this.transitions.add(transition);
   }
   
   public String toXml(){
       
       String xml = "<scenicsequence>\n";
       
       if(null != this.actors) {
          for(int i = 0; i < this.actors.size(); i++){
             xml += this.actors.get(i).toXml(1);
          }
       }
       
       if(null != this.scenes && null != this.transitions) {
          
          for(int i = 0; i < this.scenes.size(); i++){
             if(i < this.transitions.size()){
                xml += this.scenes.get(i).toXml(1);
                xml += this.transitions.get(i).toXml(1);
             } else {
                xml += this.scenes.get(i).toXml(1);
             }
          }
       }
       
       xml += "</scenicsequence>\n";
       
       return xml;
   }
}