package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class PropertiesDialog extends PopupPanel {
   
   protected static final String STYLE_PROPERTIES_DIALOG  = "propertiesDialog";
   public    static final String STYLE_PROPERTIES_CONTENT = "propertiesContent";
   protected static final String STYLE_CLOSE_X            = "propertiesCloseX";
   
   protected static final String IMG_DIALOG_POINTER_LEFT  = "images/dialog-pointer-left.png";
   protected static final String IMG_DIALOG_POINTER_RIGHT = "images/dialog-pointer-right.png";
   protected static final String IMG_DIALOG_CLOSE_X       = "images/dialog-close-x.png";
   
   public    static final int    DIALOG_CONTENT_WIDTH     = 300;
   protected static final int    DIALOG_CONTENT_HEIGHT    = 255;
   protected static final int    DIALOG_CONTENT_LEFT      = 5;
   protected static final int    DIALOG_CONTENT_TOP       = 5;
   protected static final int    POINTER_IMG_WIDTH        = 38;
   protected static final int    POINTER_IMG_HEIGHT       = 40;
   protected static final int    DIALOG_WIDTH             = PropertiesDialog.DIALOG_CONTENT_WIDTH  + PropertiesDialog.POINTER_IMG_WIDTH + 4;
   protected static final int    DIALOG_HEIGHT            = PropertiesDialog.DIALOG_CONTENT_HEIGHT;
   protected static final int    DIALOG_TOP               = 120;
   protected static final int    CLOSE_X_BUTTON_WIDTH     = 19;
   protected static final int    CLOSE_X_LEFT_OPEN_RIGHT  = PropertiesDialog.DIALOG_CONTENT_WIDTH + PropertiesDialog.POINTER_IMG_WIDTH - PropertiesDialog.CLOSE_X_BUTTON_WIDTH - 5;
   protected static final int    CLOSE_X_LEFT_OPEN_LEFT   = PropertiesDialog.DIALOG_CONTENT_WIDTH - PropertiesDialog.CLOSE_X_BUTTON_WIDTH - 5;
   protected static final int    CLOSE_X_TOP              = 5;
   
   //--//
   
   protected AbsolutePanel   panel;
   protected VerticalPanel   contentPanel;
   protected VerticalPanel   pointerPanel;
   protected Widget          content;
   
   protected Image           closeXImg;
   protected Image           iconImg;
   protected Image           pointerLeftImg;
   protected Image           pointerRightImg;

   protected PushButton      closeXButton;
   
   //--//
   
   public PropertiesDialog(){
      super(true, false);
      
      this.panel           = new AbsolutePanel();
      this.contentPanel    = new VerticalPanel();
      this.pointerPanel    = new VerticalPanel();
      this.content         = null;
 
      this.panel       .setStyleName( PropertiesDialog.STYLE_PROPERTIES_DIALOG      );
      this.panel       .setWidth    ( PropertiesDialog.DIALOG_WIDTH  + "px"         );
      this.panel       .setHeight   ( PropertiesDialog.DIALOG_HEIGHT + "px"         );
      this.contentPanel.setStyleName( PropertiesDialog.STYLE_PROPERTIES_CONTENT     );
      this.contentPanel.setWidth    ( PropertiesDialog.DIALOG_CONTENT_WIDTH  + "px" );
      this.contentPanel.setHeight   ( PropertiesDialog.DIALOG_CONTENT_HEIGHT + "px" );
      this.pointerPanel.setWidth    ( PropertiesDialog.POINTER_IMG_WIDTH + "px"     );
      this.pointerPanel.setHeight   ( PropertiesDialog.POINTER_IMG_HEIGHT + "px"    );
      
      this.closeXImg       = new Image( PropertiesDialog.IMG_DIALOG_CLOSE_X       );
      this.iconImg         = new Image( PropertiesDialog.IMG_DIALOG_POINTER_RIGHT );
      this.pointerLeftImg  = new Image( PropertiesDialog.IMG_DIALOG_POINTER_LEFT  );
      this.pointerRightImg = new Image( PropertiesDialog.IMG_DIALOG_POINTER_RIGHT );

      this.closeXButton    = new PushButton( this.closeXImg, new ClickHandler(){
         public void onClick(ClickEvent e){
            hide();
         }
      });
      
      this.closeXButton.setStyleName( PropertiesDialog.STYLE_CLOSE_X );
      
      this.pointerPanel.add( this.pointerLeftImg );

      this.panel.add       ( this.contentPanel, PropertiesDialog.POINTER_IMG_WIDTH, 0                                  );
      this.panel.add       ( this.pointerPanel, 0, 0                                                                   );
      this.panel.add       ( this.closeXButton, PropertiesDialog.CLOSE_X_LEFT_OPEN_RIGHT, PropertiesDialog.CLOSE_X_TOP );
      
      this.setWidget(this.panel);
      
      this.addCloseHandler(new CloseHandler<PopupPanel>() {
         public void onClose(CloseEvent<PopupPanel> e){
            // Do nothing for now
         }
      });
   }
   
   
   /**
    * Sets the content panel for this <code>PropertiesDialog</code>.
    * 
    * @param title
    * @param content
    */
   public void setContent(VerticalPanel content){
      
      if(null != this.content){
         this.contentPanel.remove(this.content);
      }
      this.content = content;
      
      this.contentPanel.insert( this.content, 0);
      this.contentPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
   }
   
   
   /** 
    * Positions the <code>PropertiesDialog</code> relative to the given coordinates.
    * This includes choice of left or right pointer as well as the vertical and horizontal
    * positioning of the pointer and the content panel.
    * 
    * @param x
    * @param y
    */
   public void positionRelativeTo(Widget w, int x, int y){

      int panelLeft        = 0;
      int panelTop         = PropertiesDialog.DIALOG_TOP;
      int panelWidth       = PropertiesDialog.DIALOG_WIDTH - 2;
      
      int closeXLeft       = 0;
      int pointerLeft      = 0;
      int pointerTop       = 0;
      int dialogLeft       = 0;
      int dialogTop        = 0;
      
      int windowWidth      = Window.getClientWidth();
      int scrollOffset     = 0;
      int windowBoundRight = windowWidth;
      int windowBoundLeft  = 0;
      
      // Check which direction the dialog should pop-up from
      int dialogRightSide = x + w.getOffsetWidth() + panelWidth;
      
      if(dialogRightSide <= windowBoundRight){ // Then pop-up on the right

         panelLeft   = x + w.getOffsetWidth();
         closeXLeft  = PropertiesDialog.CLOSE_X_LEFT_OPEN_RIGHT; 
         pointerLeft = 0;
         dialogLeft  = PropertiesDialog.POINTER_IMG_WIDTH - 2;
         this.pointerPanel.setSize(PropertiesDialog.POINTER_IMG_WIDTH + "px", PropertiesDialog.POINTER_IMG_HEIGHT + "px");
         this.pointerPanel.remove(0);
         this.pointerPanel.add( this.pointerLeftImg );
         
      } else { // Then pop-up on the left
         
         panelLeft   = x - panelWidth;
         closeXLeft  = PropertiesDialog.CLOSE_X_LEFT_OPEN_LEFT;
         pointerLeft = PropertiesDialog.DIALOG_CONTENT_WIDTH + 2;
         dialogLeft  = 0;
         this.pointerPanel.setSize(PropertiesDialog.POINTER_IMG_WIDTH + "px", PropertiesDialog.POINTER_IMG_HEIGHT + "px");
         this.pointerPanel.remove(0);
         this.pointerPanel.add( this.pointerRightImg );
      }
      
      
      // Now handle vertical positioning
      if(y - (PropertiesDialog.POINTER_IMG_HEIGHT / 2) < PropertiesDialog.DIALOG_TOP){
      
         pointerTop = 0;
         
      } else if(y + (PropertiesDialog.POINTER_IMG_HEIGHT / 2) > PropertiesDialog.DIALOG_TOP + PropertiesDialog.DIALOG_HEIGHT){
         
         pointerTop = PropertiesDialog.DIALOG_HEIGHT - PropertiesDialog.POINTER_IMG_HEIGHT;
         
      } else {
         int widgetHeight = w.getOffsetHeight();
         pointerTop       = y - PropertiesDialog.DIALOG_TOP + (widgetHeight / 2) - (PropertiesDialog.POINTER_IMG_HEIGHT / 2);
      }
      
      dialogTop = 0;
      
      this.panel.setWidgetPosition( this.pointerPanel, pointerLeft, pointerTop                   );
      this.panel.setWidgetPosition( this.contentPanel, dialogLeft , dialogTop                    );
      this.panel.setWidgetPosition( this.closeXButton, closeXLeft , PropertiesDialog.CLOSE_X_TOP );
      
      this.setPopupPosition(panelLeft, panelTop);
   
      this.show();
   }
   
   
   public void close(){
      this.hide();
   }
}
