package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;

public class SceneInfoPanel extends AbsolutePanel {

   protected static final String URL_STATUS_UNDER_CONSTRUCTION = "images/under-construction.png";
   protected static final String URL_STATUS_SCENE_OK           = "images/scene-ok.png";
   
   protected static final String STYLE_SCENE_INFO              = "scene-info";
   protected static final String STYLE_SCENE_INFO_TITLE        = "scene-info-title";
   protected static final String STYLE_SCENE_INFO_STATUS       = "scene-info-status-msg";
   protected static final String STYLE_SCENE_INFO_DESCRIPTION  = "scene-info-description";
   
   protected static final int    TITLE_PANEL_X_COORD           = 0;
   protected static final int    TITLE_PANEL_Y_COORD           = 2;
   protected static final int    STATUS_ICON_PANEL_X_COORD     = 2;
   protected static final int    STATUS_ICON_PANEL_Y_COORD     = 2;
   protected static final int    STATUS_MSG_PANEL_X_COORD      = 0;
   protected static final int    STATUS_MSG_PANEL_Y_COORD      = 19;
   protected static final int    DESCRIPTION_PANEL_X_COORD     = 0;
   protected static final int    DESCRIPTION_PANEL_Y_COORD     = 38;
   
   //--//
   
   protected Scene           scene;             // The subject Scene
   
   protected Image           statusIcon;        // Icon showing the status of the Scene
   protected HTML            sceneTitle;        // Title for the scene
   protected HorizontalPanel sceneTitlePanel;   // Panel holding the scene title
   protected HTML            statusMsg;         // Message containing status of the scene
   protected HorizontalPanel statusMsgPanel;    // Panel holding the status message
   protected HTML            description;       // A description of the scene
   protected HorizontalPanel descriptionPanel;  // Panel holding the scene description
   
   //--//
   
   /**
    * Creates a new SceneInfoPanel for the given Scene.
    * @param scene
    */
   public SceneInfoPanel(Scene scene){
      super();
      
      this.scene            = scene;
      this.statusIcon       = new Image(SceneInfoPanel.URL_STATUS_UNDER_CONSTRUCTION);
      
      this.sceneTitle       = new HTML(this.getTitle());
      this.sceneTitle.addStyleName(SceneInfoPanel.STYLE_SCENE_INFO_TITLE);
      
      this.sceneTitlePanel  = new HorizontalPanel();
      this.sceneTitlePanel.add(this.sceneTitle);
      this.sceneTitlePanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
      this.sceneTitlePanel.setCellHorizontalAlignment(this.sceneTitle, HasHorizontalAlignment.ALIGN_CENTER);
      
      this.statusMsg        = new HTML(this.getStatusMsg());
      this.statusMsg.addStyleName(SceneInfoPanel.STYLE_SCENE_INFO_STATUS);
      //this.statusMsg.setVisible(false);

      this.statusMsgPanel   = new HorizontalPanel();      
      this.statusMsgPanel.add(this.statusMsg);
      this.statusMsgPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
      this.statusMsgPanel.setCellHorizontalAlignment(this.statusMsg, HasHorizontalAlignment.ALIGN_CENTER);
      
      this.description      = new HTML("New Scene");
      this.description.addStyleName(SceneInfoPanel.STYLE_SCENE_INFO_DESCRIPTION);
      
      this.descriptionPanel = new HorizontalPanel();
      this.descriptionPanel.add(this.description);
      this.descriptionPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
      this.descriptionPanel.setCellHorizontalAlignment(this.description, HasHorizontalAlignment.ALIGN_CENTER);
      
      this.add(this.sceneTitlePanel , SceneInfoPanel.TITLE_PANEL_X_COORD      , SceneInfoPanel.TITLE_PANEL_Y_COORD      );
      this.add(this.statusIcon      , SceneInfoPanel.STATUS_ICON_PANEL_X_COORD, SceneInfoPanel.STATUS_ICON_PANEL_Y_COORD);
      this.add(this.statusMsgPanel  , SceneInfoPanel.STATUS_MSG_PANEL_X_COORD , SceneInfoPanel.STATUS_MSG_PANEL_Y_COORD );
      this.add(this.descriptionPanel, SceneInfoPanel.DESCRIPTION_PANEL_X_COORD, SceneInfoPanel.DESCRIPTION_PANEL_Y_COORD);
      this.addStyleName(SceneInfoPanel.STYLE_SCENE_INFO);
   }
   
   
   public void setWidth(String w, int wPix){
      super.setWidth(w);
      
      this.sceneTitlePanel .setWidth(w);
      this.statusMsgPanel  .setWidth(w);
      this.descriptionPanel.setWidth(w);
   }
   
   
   public void setHeight(String h, int hPix){
      super.setHeight(h);
   }
   
   
   public String getTitle(){
      return "Scene " + (this.scene.sequencePosition + 1);
   }
   
   
   public String getStatusMsg(){
      return this.scene.getHint();
   }
   
   
   public String getDescription(){
      return this.scene.getDescription();
   }
   
   
   public void updateInfo(){
      if(this.scene.isValid()){
         this.statusIcon.setUrl(SceneInfoPanel.URL_STATUS_SCENE_OK);
      } else {
         this.statusIcon.setUrl(SceneInfoPanel.URL_STATUS_UNDER_CONSTRUCTION);
      }
      this.sceneTitle .setHTML(this.getTitle()      );
      this.statusMsg  .setHTML(this.getStatusMsg  ());
      this.description.setHTML(this.getDescription());
   }
}
