package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import java.util.Vector;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ActorWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PrimitiveWidget;
import gwt.canvas.client.Canvas;

public class ScenePositioningPanel extends AbsolutePanel {
   
   protected Scene  scene;
   protected Canvas canvas;
   
   //--//
   
   public ScenePositioningPanel(Scene scene){
      super();
      
      this.scene  = scene;
      this.canvas = new Canvas();
      
      this.add(this.canvas,0,0);
   }
   

   public void setWidth(String w, int wPix){
      super.setWidth(w);
      this.canvas.setWidth(wPix);
   }
   
   
   public void setHeight(String h, int hPix){
      super.setHeight(h);
      this.canvas.setHeight(hPix);
   }
   
   
   public void drawSegments(Vector<LineSegment> segments){
      this.canvas.clear();
      
      for(LineSegment segment : segments){
         if(segment.dotted){
            this.canvas.beginPath();
            
            // Use alternating moveTo and lineTo to draw the path
            double len           = segment.getLength();
            double numDashes     = len / LineSegment.DASH_LEN;
            double numDashSpace  = len / (LineSegment.DASH_LEN + LineSegment.DASH_LEN);
            double xIncDash      = (segment.x2 - segment.x1) / numDashes;
            double yIncDash      = (segment.y2 - segment.y1) / numDashes;
            double xIncDashSpace = (segment.x2 - segment.x1) / numDashSpace;
            double yIncDashSpace = (segment.y2 - segment.y1) / numDashSpace;
            int    counter       = 0;
            
            for(double i = 0; i < (len - LineSegment.DASH_LEN); i += (LineSegment.DASH_LEN * 2)){
               this.canvas.moveTo(segment.x1 + xIncDashSpace*counter, 
                                  segment.y1 + yIncDashSpace*counter);
               this.canvas.lineTo(segment.x1 + xIncDashSpace*counter + xIncDash, 
                                  segment.y1 + yIncDashSpace*counter + yIncDash);
               counter++;
            }           
            this.canvas.setLineWidth(segment.width);
            this.canvas.setStrokeStyle(segment.color); // lighter: f5c9b1
            this.canvas.stroke();
            this.canvas.closePath();
         } else {
            this.canvas.beginPath();
            this.canvas.moveTo(segment.x1, segment.y1);
            this.canvas.lineTo(segment.x2, segment.y2);
            this.canvas.setLineWidth(segment.width);
            this.canvas.setStrokeStyle(segment.color); // lighter: f5c9b1
            this.canvas.stroke();
            this.canvas.closePath();
         }
      }
   }
   
   
   /**
    * Removes widgets that are instances of {@link ActorWidget} and {@link PrimitiveWidget}.
    * 
    * @param w the widget to remove
    */
   @Override
   public boolean remove(Widget w) {
      
      return super.remove(w);
   }
}
