package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import java.util.ArrayList;
import java.util.List;

import com.allen_sauer.gwt.dnd.client.PickupDragController;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.IncompatibleRemoteServiceException;
import com.google.gwt.user.client.rpc.InvocationException;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.GreetingService;
import edu.washington.cs.rfid.cascadia.scenic.client.GreetingServiceAsync;
import edu.washington.cs.rfid.cascadia.scenic.client.data.EventTask;
import edu.washington.cs.rfid.cascadia.scenic.client.data.EventTemplate;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.GrammarEncoder;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicSequence;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorLibrary;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.PrimitiveFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.SequenceFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ActorWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PrimitiveWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;


public class ScenicToolbar extends VerticalPanel implements ResizeHandler {

   protected static final String TOOLBAR_WIDTH          = "100%";
   protected static final String TOOLBAR_HEIGHT         = "300px";
   protected static final String TOOLBAR_STYLE          = "toolbarPanel";

   protected static final int    SEQUENCE_VIEWER_X_POS  = 5;
   protected static final int    SEQUENCE_VIEWER_Y_POS  = 75;
        
   protected static final int    ICON_PANEL_SPACING     = 15;
        
   protected static final String ACTOR_PANEL_WIDTH      = "300px"; //"210px";
   protected static final String ACTOR_PANEL_HEIGHT     = "100px";
   protected static final int    ACTOR_X_POS            = 5;
   protected static final int    ACTOR_Y_POS            = 5;

   protected static final String PRIMITIVE_PANEL_WIDTH  = "420px";
   protected static final String PRIMITIVE_PANEL_HEIGHT = "100px";
   protected static final int    PRIMITIVE_X_POS        = 245;
   protected static final int    PRIMITIVE_Y_POS        = 5;
   
   protected static final int    MODE_TEMPLATES         = 0;
   protected static final int    MODE_FORMATIVE         = 1;
   protected static final int    MODE_TRANSLATE         = 2;

   //--//
   
   protected static ScenicToolbar        singleton;
   protected static ActorLibrary         actorLibrary;
   protected static PickupDragController dragController;
   
   //--//

   // Determines what mode we are operating in (templates, formative study w/tasks, or translation)
   protected int                   mode;
   
   // Client for AJAX
   protected final GreetingServiceAsync    clientProxy = GWT.create(GreetingService.class);
   
   // DnD controllers
   protected DefaultDropController dropController;
   protected DefaultDropController rootDropController;   

   // Top strip with icons
   protected HorizontalPanel       iconPanel;
   protected IconPanel             actorPanel;
   protected IconPanel             primitivePanel;
        
   // Middle sequence strip
   protected SequenceViewer        sequenceViewer;
   protected SequencePanel         sequencePanel;
   protected ScenicSequence        grammarSequence;
   
   // Buttons beneath the sequence
   protected HorizontalPanel       buttonPanel;
   protected Button                nextTaskButton;
   protected Button                templateButton;
      
   // Explains the currently displayed template (if there is one)
   protected VerticalPanel         templatePanel;
   protected int                   curTemplateNum;
   protected HTML                  templateTitle;
   protected HTML                  templateDescription;
   protected HTML                  templateExampleTitle;
   protected HTML                  templateApplication;
   protected HTML                  templateScenario;

   // Task info displayed at the bottom - needed only for the formative study !
   protected VerticalPanel         taskPanel;
   protected int                   curTaskNum;
   protected HTML                  taskNumber;
   protected HTML                  taskApplication;
   protected HTML                  taskScenario;
      
   
   //--//
   
   
   public static ScenicToolbar getInstance(){
      if(null == ScenicToolbar.singleton){
         ScenicToolbar.singleton = new ScenicToolbar(ScenicToolbar.MODE_FORMATIVE);  //ScenicToolbar.MODE_TEMPLATES);
      }
      return ScenicToolbar.singleton;
   }
   
   
   public static ActorLibrary getActorLibrary(){
      if(null == ScenicToolbar.actorLibrary){
         ScenicToolbar.actorLibrary = new ActorLibrary();
      }
      return ScenicToolbar.actorLibrary;
   }
   
   
   public static PickupDragController getDragController(){
      if(null == ScenicToolbar.dragController){
         ScenicToolbar.dragController = new PickupDragController( RootPanel.get(), true );
      }
      return ScenicToolbar.dragController;
   }
   
   //--//
   
   
   private ScenicToolbar(int mode){
      super();

      this.setHeight   ( ScenicToolbar.TOOLBAR_HEIGHT );
      this.setWidth    ( ScenicToolbar.TOOLBAR_WIDTH  );
      this.setStyleName( ScenicToolbar.TOOLBAR_STYLE  );

      ScenicToolbar.actorLibrary = ScenicToolbar.getActorLibrary();

      this.mode        = ScenicToolbar.MODE_TEMPLATES;
//      this.mode        = ScenicToolbar.MODE_FORMATIVE;

      // GET RID OF AJAX FOR DEMO!
//      this.clientProxy = ScenicService.Util.getInstance();

      this.initDndControls();
      this.initIconPanel  ();
      this.initSequenceViewer(SequenceFactory.createSequencePanel());
      this.initButtonPanel();
      this.initTemplatePanel();
      this.initTaskPanel();
      
      if(this.mode == ScenicToolbar.MODE_TEMPLATES){
         this.add( this.templatePanel );
      } else if(this.mode == ScenicToolbar.MODE_FORMATIVE){
         this.add( this.taskPanel );
      }
      
      //ScenicToolbar.disableTextSelectInternal( this.getElement()  , true       );
      JavaScriptHelper.disableContextMenu( this.getElement()               , true );
      JavaScriptHelper.disableContextMenu( this.sequencePanel.getElement() , true );
      JavaScriptHelper.disableContextMenu( this.sequenceViewer.getElement(), true );
      
//      if(this.mode == ScenicToolbar.MODE_TEMPLATES){
//         this.clientProxy.getTemplate(this.curTemplateNum, this.loadTemplateCallback);
//      } else if(this.mode == ScenicToolbar.MODE_FORMATIVE){
//         this.clientProxy.nextEventTask(this.curTaskNum, this.loadTaskCallback);
//      }
   }
   
   
   private void initDndControls(){
      ScenicToolbar.dragController = ScenicToolbar.getDragController();
      ScenicToolbar.dragController.setBehaviorConstrainedToBoundaryPanel( true  );
      ScenicToolbar.dragController.setBehaviorMultipleSelection         ( false );
      ScenicToolbar.dragController.setBehaviorDragProxy                 ( false );
                
      this.dropController = new DefaultDropController( this );
      ScenicToolbar.dragController.registerDropController( this.dropController );
            
      this.rootDropController = new DefaultDropController( RootPanel.get() );
      ScenicToolbar.dragController.registerDropController( this.rootDropController );
   }
   
   
   private void initIconPanel(){
      this.iconPanel = new HorizontalPanel();
      this.iconPanel.setSpacing( ScenicToolbar.ICON_PANEL_SPACING );

      this.initActorPanel    ();
      this.initPrimitivePanel();
      
      this.iconPanel.add( this.actorPanel     );
      this.iconPanel.add( this.primitivePanel );
      
      JavaScriptHelper.disableTextSelection(this.primitivePanel.getElement(), true);
      JavaScriptHelper.disableTextSelection(this.actorPanel.getElement()    , true);

      JavaScriptHelper.disableContextMenu(this.actorPanel.getElement()    , true );
      JavaScriptHelper.disableContextMenu(this.primitivePanel.getElement(), true );
      
      this.add(this.iconPanel);
   }
   
   
   private void initActorPanel(){
      this.actorPanel       = new IconPanel( ScenicToolbar.dragController );

      ActorWidget person    = ActorFactory.createPersonWidget   ();
      ActorWidget personGrp = ActorFactory.createPersonGrpWidget();
      ActorWidget thing     = ActorFactory.createThingWidget    ();
      ActorWidget thingGrp  = ActorFactory.createThingGrpWidget ();
      ActorWidget place     = ActorFactory.createPlaceWidget    ();
      
      person   .setLabelVisible(false);
      personGrp.setLabelVisible(false);
      thing    .setLabelVisible(false);
      thingGrp .setLabelVisible(false);
      place    .setLabelVisible(false);
      
      this.actorPanel.add( person    );
      this.actorPanel.add( thing     );
      this.actorPanel.add( place     );
      //this.actorPanel.add( thingGrp  );
      //this.actorPanel.add( personGrp );
      
      this.actorPanel.setSize( ScenicToolbar.ACTOR_PANEL_WIDTH, ScenicToolbar.ACTOR_PANEL_HEIGHT );
   }
   
   
   private void initPrimitivePanel(){
      this.primitivePanel   = new IconPanel( dragController );
      
      PrimitiveWidget with    = PrimitiveFactory.createWithWidget   ();
      PrimitiveWidget without = PrimitiveFactory.createWithoutWidget();
      PrimitiveWidget near    = PrimitiveFactory.createInsideWidget ();
      PrimitiveWidget far     = PrimitiveFactory.createOutsideWidget();
      PrimitiveWidget inside  = PrimitiveFactory.createNearWidget   ();
      PrimitiveWidget outside = PrimitiveFactory.createFarWidget    ();
      PrimitiveWidget lasts   = PrimitiveFactory.createLastsWidget  ();

      with   .setLabelVisible( false );
      without.setLabelVisible( false );
      inside .setLabelVisible( false );
      outside.setLabelVisible( false );
      near   .setLabelVisible( false );
      far    .setLabelVisible( false );
      lasts  .setLabelVisible( false );
      
      this.primitivePanel.add( with    );
      this.primitivePanel.add( without );
      this.primitivePanel.add( inside  );
      this.primitivePanel.add( outside );
      this.primitivePanel.add( near    );
      this.primitivePanel.add( far     );
      //this.primitivePanel.add( lasts   );
      
      this.primitivePanel.setSize( ScenicToolbar.PRIMITIVE_PANEL_WIDTH, ScenicToolbar.PRIMITIVE_PANEL_HEIGHT );
   }
   
   
   private void initSequenceViewer(SequencePanel sequencePanel){
      if(null != this.sequenceViewer){
         this.remove(this.sequenceViewer);
      }
      this.sequencePanel  = sequencePanel;
      this.sequenceViewer = new SequenceViewer(this.sequencePanel);
      Window.addResizeHandler(this.sequencePanel);
      //ScenicToolbar.disableTextSelectInternal( this.sequencePanel.getElement() , true );
      //ScenicToolbar.disableTextSelectInternal( this.sequenceViewer.getElement(), true );
      this.insert(this.sequenceViewer,1);
   }
   
   
   private void initButtonPanel(){
      this.buttonPanel = new HorizontalPanel();
      this.buttonPanel.setWidth("100%");
      
      HorizontalPanel buttonHolder = new HorizontalPanel();
      
      if(this.mode == ScenicToolbar.MODE_TEMPLATES){
         this.templateButton = new Button("Next Template");
         this.templateButton.addClickHandler(new ClickHandler(){
            public void onClick(ClickEvent e){
//               clientProxy.getTemplate(curTemplateNum, loadTemplateCallback);
            }
         });
         buttonHolder.add(this.templateButton);
      } else if(this.mode == ScenicToolbar.MODE_FORMATIVE){
         this.nextTaskButton = new Button("Next Task");
         this.nextTaskButton.addClickHandler(new ClickHandler(){
            public void onClick(ClickEvent e){
               ScenicSequence scenicSequence = GrammarEncoder.getScenicSequence(sequencePanel);
               System.out.println(scenicSequence.toXml());
               sequencePanel.clearScenes();
               //clientProxy.nextEventTask(curTaskNum, loadTaskCallback);
            }
         });
         buttonHolder.add(this.nextTaskButton);
      } else if(this.mode == ScenicToolbar.MODE_TRANSLATE){
         Button translateButton = new Button("Translate");
         translateButton.addClickHandler(new ClickHandler(){
            public void onClick(ClickEvent e){
               ScenicSequence seq = GrammarEncoder.getScenicSequence(sequencePanel);
//               clientProxy.translateScenic(seq, translateCallback);
            }
         });
         buttonHolder.add(translateButton);
      }
      this.buttonPanel.add(buttonHolder);
      this.buttonPanel.setCellHorizontalAlignment(buttonHolder, HasHorizontalAlignment.ALIGN_CENTER);
      
      this.add(this.buttonPanel);
   }
   
   
   /**
    * Initializes template panel - used only in the formative study.
    */
   private void initTemplatePanel(){
      this.curTemplateNum       = 1;
      this.templatePanel        = new VerticalPanel();
      
      VerticalPanel templateHolderPanel = new VerticalPanel();
      templateHolderPanel.setWidth("75%");
      
      this.templateTitle        = new HTML();
      this.templateDescription  = new HTML();
      this.templateExampleTitle = new HTML();
      this.templateApplication  = new HTML();
      this.templateScenario     = new HTML();
      
      DOM.setStyleAttribute(this.templatePanel.getElement(), "backgroundColor", "#FFFFFF");
      this.templatePanel.setWidth( "100%" );
      
      templateHolderPanel.add( new HTML("&nbsp;")        );
      templateHolderPanel.add( this.templateTitle        );
      templateHolderPanel.add( this.templateDescription  );
      templateHolderPanel.add( this.templateExampleTitle );
      templateHolderPanel.add( this.templateApplication  );
      templateHolderPanel.add( this.templateScenario     );
      
      this.templatePanel.add(templateHolderPanel);
      this.templatePanel.setCellHorizontalAlignment(templateHolderPanel, HorizontalPanel.ALIGN_CENTER);
   }
   
   
   /**
    * Initializes task panel - used only in the formative study
    */
   private void initTaskPanel(){
      this.curTaskNum = 1;
      this.taskPanel  = new VerticalPanel();
      
      VerticalPanel taskHolderPanel = new VerticalPanel();
      taskHolderPanel.setWidth("75%");
      
      this.taskNumber      = new HTML();
      this.taskApplication = new HTML();
      this.taskScenario    = new HTML();
      
      DOM.setStyleAttribute( this.taskPanel.getElement(), "backgroundColor", "#FFFFFF");
      this.taskPanel.setWidth( "100%" );
      
      taskHolderPanel.add( new HTML("&nbsp;")   );
      taskHolderPanel.add( this.taskNumber      );
      taskHolderPanel.add( this.taskApplication );
      taskHolderPanel.add( this.taskScenario    );
      
      this.taskPanel.add(taskHolderPanel);
      this.taskPanel.setCellHorizontalAlignment(taskHolderPanel, HorizontalPanel.ALIGN_CENTER);
   }
   
   
   public void loadTemplateFromXml(String xml){
      ScenicSequence sequence = new ScenicSequence(xml);
      this.sequencePanel.clearScenes();
      SequenceFactory.createSequencePanel(sequence, this.sequencePanel);
   }
   
   
   public void addThumbnail(SequencePanel thumbnail){
      this.add(thumbnail);
   }
   
   //------------------------------- HANDLER METHODS -----------------------------------//
   
   public void onResize(ResizeEvent e){
      int w = e.getWidth();
      this.buttonPanel.setWidth(w + "px");
   }
   
   //------------------------------- CALLBACKS -----------------------------------//
   
   private final AsyncCallback<EventTemplate> loadTemplateCallback = new AsyncCallback<EventTemplate>() {
      
      public void onSuccess(EventTemplate result) {
         if(null != result){
            templateTitle       .setHTML("<b>Template Event #" + curTemplateNum + ":</b>&nbsp;&nbsp;" + result.title + "<br>&nbsp;<br>");
            templateDescription .setHTML("<b>Description:</b><br>" + result.description + "<br><br>");
            templateExampleTitle.setHTML("<b>Example:</b><br>----------<br>");
            templateApplication .setHTML("<b>Application:</b><br>" + result.exampleApp + "<br><br>");
            templateScenario    .setHTML("<b>Usage Scenario:</b><br>" + result.exampleScenario + "<br>&nbsp;<br>");
            
            curTemplateNum = (curTemplateNum % 4 == 0)? 1 : (curTemplateNum + 1);
            
            
            
            ScenicSequence sequence = new ScenicSequence(result.xml);
            sequencePanel.clearScenes();
            SequenceFactory.createSequencePanel(sequence, sequencePanel);
            
            // Now crank out the sub-event thumbnails!
            List<SequencePanel>  thumbnails = new ArrayList<SequencePanel>(); 
            SequencePanel        thumbnail  = SequenceFactory.createThumbnailSequencePanel(sequence);
            thumbnails.add(thumbnail);
            List<ScenicSequence> subEvents  = GrammarEncoder.getSubEvents(sequence);
            for(ScenicSequence subEvent : subEvents){
               thumbnail = SequenceFactory.createThumbnailSequencePanel(subEvent);
               thumbnails.add(thumbnail);
            }
            for(SequencePanel panel : thumbnails){
               addThumbnail(panel);
            }
            
//            initSequenceViewer(SequenceFactory.createSequencePanel(sequence));
            
         } else {
            templateTitle.setHTML("Sorry, invalid template request!");
            templateDescription .setHTML("");
            templateExampleTitle.setHTML("");
            templateApplication .setHTML("");
            templateScenario    .setHTML("");
         }
      }

      public void onFailure(Throwable caught) {
         // Convenient way to find out which exception was thrown.
         try {
            throw caught;
         } catch (IncompatibleRemoteServiceException e) {
            // this client is not compatible with the server; cleanup and refresh the browser
         } catch (InvocationException e) {
            // the call didn't complete cleanly
         } catch (Throwable e) {
            // last resort -- a very unexpected exception
         }
      }
   };
   
   
   private final AsyncCallback<EventTask> loadTaskCallback = new AsyncCallback<EventTask>() {
      
      public void onSuccess(EventTask result) {
         if(null != result){
            taskNumber     .setHTML("<b>Task Number:&nbsp;&nbsp;" + result.eventNum + "</b><br>&nbsp;<br>");
            taskApplication.setHTML("<b>Application:</b><br>"     + result.application + "<br>&nbsp;<br>");
            taskScenario   .setHTML("<b>Usage Scenario:</b><br>"  + result.usageScenario + "<br>&nbsp;<br>");
            curTaskNum++;
         } else {
            System.out.println("result was null!");
         }
      }

      public void onFailure(Throwable caught) {
         // Convenient way to find out which exception was thrown.
         try {
            throw caught;
         } catch (IncompatibleRemoteServiceException e) {
            // this client is not compatible with the server; cleanup and refresh the browser
         } catch (InvocationException e) {
            // the call didn't complete cleanly
         } catch (Throwable e) {
            // last resort -- a very unexpected exception
         }
      }
   };
   
   
       
   private final AsyncCallback translateCallback = new AsyncCallback() {
        
      public void onSuccess(Object result) {
         System.out.println("As XML:\n" + result.toString());
      }

      public void onFailure(Throwable caught) {
         // Convenient way to find out which exception was thrown.
         try {
            throw caught;
         } catch (IncompatibleRemoteServiceException e) {
            // this client is not compatible with the server; cleanup and refresh the browser
         } catch (InvocationException e) {
            // the call didn't complete cleanly
         } catch (Throwable e) {
            // last resort -- a very unexpected exception
         }
      }
   };
}