package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.VerticalPanel;

public class TimeConstraintsPanel extends HorizontalPanel {
   
   protected static final String   STYLE_PANEL = "timeConstraintsPanel";
   protected static final String   STYLE_BOX   = "timeConstraintsBox";
   
   protected static final String[] TIMES       = {"12a","12:30a","1a","1:30a","2a","2:30a","3a","3:30a","4a","4:30a","5a","5:30a",
                                                  "6a","6:30a","7a","7:30a","8a","8:30a","9a","9:30a","10a","10:30a","11a","11:30a",
                                                  "12p","12:30p","1p","1:30p","2p","2:30p","3p","3:30p","4p","4:30p","5p","5:30p",
                                                  "6p","6:30p","7p","7:30p","8p","8:30p","9p","9:30p","10p","10:30p","11p","11:30p"};
   
   protected static final String[] DAYS        = {"Weekdays","Weekends","M,W,F","T,R"};
   
   //--//
   
   protected ListBox fromBox;
   protected ListBox toBox;
   protected ListBox daysBox;

   //--//
   
   /**
    * Creates a new <code>TimeConstraintsPanel</code>.
    */
   public TimeConstraintsPanel(){
      this.fromBox = new ListBox();
      this.fromBox.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      this.toBox   = new ListBox();
      this.toBox.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      this.daysBox = new ListBox();
      this.daysBox.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      
      for(int i=0; i < TimeConstraintsPanel.TIMES.length; i++){
         this.fromBox.addItem(TimeConstraintsPanel.TIMES[i]);
         this.toBox.addItem(TimeConstraintsPanel.TIMES[i]);
      }
      for(int i=0; i < TimeConstraintsPanel.DAYS.length; i++){
         this.daysBox.addItem(TimeConstraintsPanel.DAYS[i]);
      }
      
      this.add(this.createBoxPanel());
      this.setStyleName(TimeConstraintsPanel.STYLE_PANEL);
   }
   
   
   private VerticalPanel createBoxPanel(){
      VerticalPanel vp = new VerticalPanel();
      
      HorizontalPanel top = new HorizontalPanel();
      top.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
      HTML from = new HTML("&nbsp;&nbsp;&nbsp;From:&nbsp;");
      from.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      top.add(from);
      top.add(this.fromBox);
      top.add(new HTML("&nbsp;&nbsp;"));
      HTML to = new HTML("To:&nbsp;");
      to.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      top.add(to);
      top.add(this.toBox);
      
      HorizontalPanel bot = new HorizontalPanel();
      bot.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
      HTML days = new HTML("&nbsp;&nbsp;&nbsp;Days:&nbsp;");
      days.setStyleName(TimeConstraintsPanel.STYLE_BOX);
      bot.add(days);
      bot.add(this.daysBox);
      
      vp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
      HTML spacer = new HTML("");
      spacer.setHeight("10px");
      vp.add(spacer);
      vp.add(top);
      vp.add(new HTML("&nbsp;"));
      vp.add(bot);
      
      return vp;
   }
   
   
   public void setFromTime(String time){
      int idx   = -1;
      int count = this.fromBox.getItemCount();
      
      for(int i=0; i < count; i++){
         if(this.fromBox.getItemText(i).equals(time)){
            idx = i;
            break;
         }
      }
      if(0 < idx){
         this.fromBox.setSelectedIndex(idx);
      }
   }
   
   public void setToTime(String time){
      int idx   = -1;
      int count = this.toBox.getItemCount();
      
      for(int i=0; i < count; i++){
         if(this.toBox.getItemText(i).equals(time)){
            idx = i;
            break;
         }
      }
      if(0 < idx){
         this.toBox.setSelectedIndex(idx);
      }
   }

   public void setDays(String time){
      int idx   = -1;
      int count = this.daysBox.getItemCount();
      
      for(int i=0; i < count; i++){
         if(this.daysBox.getItemText(i).equals(time)){
            idx = i;
            break;
         }
      }
      if(0 < idx){
         this.daysBox.setSelectedIndex(idx);
      }
   }
   
   public String getFromTime(){
      return this.fromBox.getValue(this.fromBox.getSelectedIndex());
   }
   
   
   public String getToTime(){
      return this.toBox.getValue(this.toBox.getSelectedIndex());
   }
   
   
   public String getDays(){
      return this.daysBox.getValue(this.daysBox.getSelectedIndex());
   }
}
