package edu.washington.cs.rfid.cascadia.scenic.client.ui;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.VerticalPanel;

import gwt.canvas.client.Canvas;

public class Transition extends VerticalPanel {

   protected static final String STYLE_TRANSITION    = "transition";
   protected static final double WIDTH_CHANGE_FACTOR = 0.5;
   
   protected static final int    STATE_INITIAL       = 0;
   protected static final int    STATE_NEXT          = 1;
   protected static final int    STATE_BEFORE        = 2;
   
   protected static final String STATE_INITIAL_NAME  = "initial";
   protected static final String STATE_NEXT_NAME     = "next";
   protected static final String STATE_BEFORE_NAME   = "before";
   
   protected static final String STATE_LABEL_NEXT    = "Immediately";
   protected static final String STATE_LABEL_BEFORE  = "Time passes";
   
   //--//

   protected SequencePanel     parentSequence;
   protected int               state;
   
   protected int               width;
   protected int               height;
   protected AbsolutePanel     containerPanel;
   protected PushButton        timeWedgeButton;
   protected Image             errorWarningImg;
   protected Canvas            transparentCanvas;
   protected List<LineSegment> lines;
   protected Button            timingChangeButton;
   
   protected boolean           thumbnail;
   
   //--//
   
   /**
    * Creates a new Transition with the specified sizing and buttons.
    * @param parentSequence
    * @param width
    * @param height
    * @param timeWedgeButton
    * @param errorWarningImg
    * @param thumbnail
    */
   public Transition(SequencePanel parentSequence    ,
                     int           width             ,
                     int           height            ,
                     PushButton    timeWedgeButton   ,
                     Image         errorWarningImg   ,
                     Button        timingChangeButton,
                     boolean       thumbnail){
      this.parentSequence     = parentSequence;
      this.state              = Transition.STATE_INITIAL;
      this.width              = width;
      this.height             = height;
      this.containerPanel     = new AbsolutePanel();
      this.timeWedgeButton    = timeWedgeButton;
      this.errorWarningImg    = errorWarningImg;
      this.lines              = new ArrayList<LineSegment>();
      this.timingChangeButton = timingChangeButton;
      this.thumbnail          = thumbnail;
      
      this.containerPanel.setPixelSize(this.width, this.height);

      this.initTransparentCanvas();
      this.initErrorWarningImg();
      this.initTimeWedgeButton();
      this.initTimeChangeButton();
      
      this.setState(Transition.STATE_INITIAL);
      this.containerPanel.setStyleName(Transition.STYLE_TRANSITION);
      this.setStyleName(Transition.STYLE_TRANSITION);
      
      this.add(this.containerPanel);
      
      DeferredCommand.addCommand(new AdaptToWidthCmd()); // updates positioning
   }
   
   
   /**
    * Returns the current width in pixels of this transition.
    * @return
    */
   public int getWidth(){
      return this.width;
   }
   
   
   public int getState(){
      return this.state;
   }
   
   
   public String getType(){
      String type = null;
      if(this.state == Transition.STATE_INITIAL){
         type = Transition.STATE_INITIAL_NAME;
      } else if(this.state == Transition.STATE_NEXT){
         type = Transition.STATE_NEXT_NAME;
      } else if(this.state == Transition.STATE_BEFORE){
         type = Transition.STATE_BEFORE_NAME;
      }
      return type;
   }
   
   
   public void setState(String state){
      if(null != state){
         if(state.equals(Transition.STATE_INITIAL_NAME)){
            this.setState(Transition.STATE_INITIAL);
         } else if(state.equals(Transition.STATE_NEXT_NAME)){
            this.setState(Transition.STATE_NEXT);
         } else if(state.equals(Transition.STATE_BEFORE_NAME)){
            this.setState(Transition.STATE_BEFORE);
         }
      }
   }
   
   
   public void setState(int state){
      if(this.state == 0 || this.state != state){
         this.lines.clear();
         switch(state){
            case Transition.STATE_INITIAL:
               this.state = Transition.STATE_INITIAL;
               this.lines.add(this.getCenterLine());
               this.errorWarningImg.setVisible(true);
               this.timeWedgeButton.setVisible(false);
               timingChangeButton.setText("Set!");
               break;
            case Transition.STATE_NEXT:
               if(this.state == Transition.STATE_INITIAL){
                  this.changeWidth(this.width);
               } else {
                  this.changeWidth((int)(this.width * Transition.WIDTH_CHANGE_FACTOR));
               }
               this.state = Transition.STATE_NEXT;
               this.lines.add(this.getCenterLine());
               this.lines.add(this.getTransitionLine(false));
               this.lines.add(this.getTransitionArrowTop());
               this.lines.add(this.getTransitionArrowBottom());
               
               this.errorWarningImg.setVisible(false);
               this.timeWedgeButton.setVisible(true);
               timingChangeButton.setText("Instantly");
               break;
            case Transition.STATE_BEFORE:
               if(this.state == Transition.STATE_INITIAL){
                  this.changeWidth((int)(this.width / Transition.WIDTH_CHANGE_FACTOR));
               } else {
                  this.changeWidth((int)(this.width / Transition.WIDTH_CHANGE_FACTOR));
               }
               this.state = Transition.STATE_BEFORE;
               this.lines.add(this.getCenterLine());
               this.lines.add(this.getTransitionLine(true));
               this.lines.add(this.getTransitionArrowTop());
               this.lines.add(this.getTransitionArrowBottom());
               
               this.errorWarningImg.setVisible(false);
               this.timeWedgeButton.setVisible(true);
               timingChangeButton.setText("Time\nPasses");
               break;
            default:
               break;
         }
         this.redrawLines();
         this.parentSequence.readjustCellWidths();
         DeferredCommand.addCommand(new AdaptToWidthCmd());
      }
   }
   
   
   public void switchState(){
      if(this.state == Transition.STATE_BEFORE){
         this.setState(Transition.STATE_NEXT);
      } else {
         this.setState(Transition.STATE_BEFORE);
      }
   }
   
   //--//
   
   
   protected void initTransparentCanvas(){
      this.transparentCanvas = new Canvas();
      this.transparentCanvas.setPixelSize(this.width, this.height);
      this.transparentCanvas.setBackgroundColor("#242424");
      
      this.containerPanel.add(this.transparentCanvas,0,0);
   }
   
   
   protected void initErrorWarningImg(){
      int left = this.width/2 - this.errorWarningImg.getWidth()/2;
      left     = (left > 0)? left : 0;
      
      this.containerPanel.add(this.errorWarningImg, left, 0);
      if(this.thumbnail){
         this.errorWarningImg.setVisible(false);
      }
   }
   
   
   protected void initTimeWedgeButton(){
      int left = this.width/2 - this.timeWedgeButton.getOffsetWidth()/2;
      left     = (left > 0)? left : 0;
      
      this.containerPanel.add(this.timeWedgeButton,left,0);
      this.timeWedgeButton.addMouseUpHandler(new MouseUpHandler(){
         public void onMouseUp(MouseUpEvent e){
            e.stopPropagation();
         }
      });
      if(!this.thumbnail){
         this.timeWedgeButton.setVisible(false);
      }
   }
   
   
   protected void initTimeChangeButton(){
      
      int left = this.width/2 - this.timingChangeButton.getOffsetWidth()/2;
      this.containerPanel.add(this.timingChangeButton, 
                              left, 
                              this.height - this.timingChangeButton.getOffsetHeight());
      this.timingChangeButton.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){
            switchState();
         }
      });
      this.timingChangeButton.addMouseUpHandler(new MouseUpHandler(){
         public void onMouseUp(MouseUpEvent e){
            e.stopPropagation();
         }
      });
      if(this.thumbnail){
         this.timingChangeButton.setVisible(false);
      }
   }
   
   
   protected void changeWidth(int width){
      this.width = width;
      this.containerPanel.setPixelSize(this.width, this.height);
      this.transparentCanvas.setPixelSize(this.width, this.height);
      
      int left = this.width/2 - this.errorWarningImg.getWidth()/2;
      this.containerPanel.setWidgetPosition(this.errorWarningImg, left, 0);
      left = this.width/2 - this.timeWedgeButton.getOffsetWidth()/2;
      this.containerPanel.setWidgetPosition(this.timeWedgeButton, left, 0);
      left = this.width/2 - this.timingChangeButton.getOffsetWidth()/2;
      this.containerPanel.setWidgetPosition(this.timingChangeButton, 
                                            left, 
                                            this.height - this.timingChangeButton.getOffsetHeight());
      this.redrawLines();
   }
   
   
   protected LineSegment getCenterLine(){
      int         startX       = (int)Math.ceil(this.width / 2.0);
      LineSegment dividingLine = new LineSegment(startX, 
                                                 0, 
                                                 startX, 
                                                 this.height, 
                                                 LineSegment.WIDTH_DIVIDING_LINE, 
                                                 LineSegment.COLOR_DIVIDING_LINE,
                                                 true);
      return dividingLine;
   }
   
   
   protected LineSegment getTransitionLine(boolean dashed){
      double      widthLine = (this.thumbnail)? LineSegment.WIDTH_TRANSITION_LINE_THUMBNAIL :
                                                LineSegment.WIDTH_TRANSITION_LINE;
      int         startY   = (int)Math.ceil(this.height/2.0);
      LineSegment transLine = new LineSegment(0,
                                              startY,
                                              this.width,
                                              startY,
                                              widthLine, 
                                              LineSegment.COLOR_DIVIDING_LINE,
                                              dashed);
      return transLine;
   }
   
   
   protected LineSegment getTransitionArrowTop(){
      double      widthLine = (this.thumbnail)? LineSegment.WIDTH_TRANSITION_LINE_THUMBNAIL :
                                                LineSegment.WIDTH_TRANSITION_LINE;
      int         startX    = (this.thumbnail)? this.width     : this.width - 1;
      int         endX      = (this.thumbnail)? this.width - 4 : this.width - 11;
      int         startY    = (int)Math.ceil(this.height/2.0);
      int         endY      = (this.thumbnail)? startY - 4 : startY - 10;
      LineSegment arrowLine = new LineSegment(startX,
                                              startY,
                                              endX,
                                              endY,
                                              widthLine, 
                                              LineSegment.COLOR_DIVIDING_LINE,
                                              false);
      return arrowLine;
   }
   
   
   protected LineSegment getTransitionArrowBottom(){
      double      widthLine = (this.thumbnail)? LineSegment.WIDTH_TRANSITION_LINE_THUMBNAIL :
                                                LineSegment.WIDTH_TRANSITION_LINE;
      int         startX    = (this.thumbnail)? this.width     : this.width - 1;
      int         endX      = (this.thumbnail)? this.width - 4 : this.width - 11;
      int         startY    = (int)Math.ceil(this.height/2.0);
      int         endY      = (this.thumbnail)? startY + 4 : startY + 10;
      LineSegment arrowLine = new LineSegment(startX,
                                              startY - 1,
                                              endX,
                                              endY,
                                              widthLine, 
                                              LineSegment.COLOR_DIVIDING_LINE,
                                              false);
      return arrowLine;
   }
   
   
   protected void redrawLines(){
      this.transparentCanvas.clear();
      
      double dashLen = (this.thumbnail)? LineSegment.DASH_LEN_THUMBNAIL :
                                         LineSegment.DASH_LEN;
      
      
      for(LineSegment segment : lines){
         if(segment.dotted){
            this.transparentCanvas.beginPath();
            
            // Use alternating moveTo and lineTo to draw the path
            double len           = segment.getLength();
            double numDashes     = len / dashLen;
            double numDashSpace  = len / (dashLen + dashLen);
            double xIncDash      = (segment.x2 - segment.x1) / numDashes;
            double yIncDash      = (segment.y2 - segment.y1) / numDashes;
            double xIncDashSpace = (segment.x2 - segment.x1) / numDashSpace;
            double yIncDashSpace = (segment.y2 - segment.y1) / numDashSpace;
            int    counter       = 0;
            
            for(double i = 0; i < (len - dashLen); i += (dashLen * 2)){
               this.transparentCanvas.moveTo(segment.x1 + xIncDashSpace*counter, 
                                             segment.y1 + yIncDashSpace*counter);
               this.transparentCanvas.lineTo(segment.x1 + xIncDashSpace*counter + xIncDash, 
                                             segment.y1 + yIncDashSpace*counter + yIncDash);
               counter++;
            }           
            this.transparentCanvas.setLineWidth(segment.width);
            this.transparentCanvas.setStrokeStyle(segment.color);
            this.transparentCanvas.stroke();
            this.transparentCanvas.closePath();
         } else {
            this.transparentCanvas.beginPath();
            this.transparentCanvas.moveTo(segment.x1, segment.y1);
            this.transparentCanvas.lineTo(segment.x2, segment.y2);
            this.transparentCanvas.setLineWidth(segment.width);
            this.transparentCanvas.setStrokeStyle(segment.color);
            this.transparentCanvas.stroke();
            this.transparentCanvas.closePath();
         }
      }
   }
   
   
   //-------------------------------- COMMANDS ----------------------------------
   
   
   private class AdaptToWidthCmd implements Command {
      
      public void execute(){
         changeWidth(width);
      }
   }
}
