package edu.washington.cs.rfid.cascadia.scenic.client.ui.factory;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Person;
import edu.washington.cs.rfid.cascadia.scenic.client.data.PersonGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Place;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Thing;
import edu.washington.cs.rfid.cascadia.scenic.client.data.ThingGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicActor;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPerson;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPersonGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPlace;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicThing;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicThingGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ActorWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PersonGrpWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PersonWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PlaceWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ThingGrpWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.ThingWidget;


/**
 * A factory for creating Actors.  Creates actors encapsulated by ActorWidgets.
 * 
 * @author Evan
 */
public abstract class ActorFactory {

   protected static final String ID_PREFIX_ACTOR  = "Actor";
   
   //--//
   
   protected static int actorNum = 0;

   //--//

   public static ActorWidget createWidget(ScenicActor actorBean){

      ActorWidget aw = null;

      if(actorBean instanceof ScenicPerson){
         aw = ActorFactory.createPersonWidget((ScenicPerson)actorBean);
      } else if(actorBean instanceof ScenicPersonGrp){
         aw = ActorFactory.createPersonGrpWidget((ScenicPersonGrp)actorBean);
      } else if(actorBean instanceof ScenicThing){
         aw = ActorFactory.createThingWidget((ScenicThing)actorBean);
      } else if(actorBean instanceof ScenicThingGrp){
         aw = ActorFactory.createThingGrpWidget((ScenicThingGrp)actorBean);
      } else if(actorBean instanceof ScenicPlace){
         aw = ActorFactory.createPlaceWidget((ScenicPlace)actorBean);
      }
      aw.setLabelVisible(true);
      return aw;
   }
   
   
   public static ActorWidget createThumbnailWidget(ScenicActor actorBean){

      ActorWidget aw = null;

      if(actorBean instanceof ScenicPerson){
         aw = ActorFactory.createThumbnailPersonWidget((ScenicPerson)actorBean);
      } else if(actorBean instanceof ScenicPersonGrp){
         aw = ActorFactory.createThumbnailPersonGrpWidget((ScenicPersonGrp)actorBean);
      } else if(actorBean instanceof ScenicThing){
         aw = ActorFactory.createThumbnailThingWidget((ScenicThing)actorBean);
      } else if(actorBean instanceof ScenicThingGrp){
         aw = ActorFactory.createThumbnailThingGrpWidget((ScenicThingGrp)actorBean);
      } else if(actorBean instanceof ScenicPlace){
         aw = ActorFactory.createThumbnailPlaceWidget((ScenicPlace)actorBean);
      }
      return aw;
   }
   
   
   //----------------------------- PERSON ----------------------------//
   
   protected static final String ID_PREFIX_PERSON = "Person";
   protected static final String[] personImgs = new String[] {
      "images/person-orange.png",
      "images/person-blue.png",
      "images/person-blue-green.png",
      "images/person-pink.png",
      "images/person-purple.png",
      "images/person-red.png",
      "images/person-yellow.png"
   };
   protected static int HEIGHT_PERSON_IMG = 29;
   protected static int WIDTH_PERSON_IMG  = 20;
   
   //--//
   
   protected static int personNum   = 0;
   protected static int personColor = 0;

   //--//

   public static PersonWidget createPersonWidget(){
      return ActorFactory.createPersonWidget(ActorFactory.createPerson(),true,false);
   }
   
   public static PersonWidget createPersonWidget(ScenicPerson personBean){
      return ActorFactory.createPersonWidget(ActorFactory.createPerson(personBean),false,false);
   }
   
   public static PersonWidget createThumbnailPersonWidget(ScenicPerson personBean){
      return ActorFactory.createPersonWidget(ActorFactory.createPerson(personBean),false,true);
   }
   
   //--//
   
   private static PersonWidget createPersonWidget(Person p, boolean useDefaultIcon, boolean thumbnail){
      int         width      = (thumbnail)? 
                               (int)(ActorFactory.WIDTH_PERSON_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.WIDTH_PERSON_IMG;
      int         height     = (thumbnail)?
                               (int)(ActorFactory.HEIGHT_PERSON_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.HEIGHT_PERSON_IMG;
      Image       personImg  = new Image(p.getImgUrl());
      Image       defaultImg = new Image(ActorFactory.personImgs[0]);
      LabeledIcon lblIco     = new LabeledIcon(personImg,
                                               defaultImg,
                                               width,
                                               height,
                                               new HTML("&nbsp;" + p.getName() + "&nbsp;"), 
                                               useDefaultIcon);
      return new PersonWidget(lblIco, p);
   }
   
   
   
   private static Person createPerson(){      
      Person p = new Person(ActorFactory.ID_PREFIX_PERSON + ActorFactory.actorNum ,
                            ActorFactory.actorNum                                 ,
                            ActorFactory.ID_PREFIX_PERSON + ActorFactory.personNum,
                            ActorFactory.personImgs[ActorFactory.personColor]);

      ActorFactory.actorNum++;
      ActorFactory.personNum++;
      ActorFactory.personColor = (ActorFactory.personColor + 1) % ActorFactory.personImgs.length;

      return p;
   }
   
   private static Person createPerson(ScenicPerson spec){
       return new Person(ActorFactory.ID_PREFIX_PERSON + spec.getId(),
                         (int)spec.getId()                           ,
                         spec.getName()                              ,
                         spec.getImgUrl());
   }
   
   
   //-------------------------- PERSON GROUP --------------------------//

   protected static final String ID_PREFIX_PERSON_GRP = "PersonGrp";
   protected static final String[] personGrpImgs = new String[] {
      "images/group-person-orange.png",
      "images/group-person-blue.png",
      "images/group-person-blue-green.png",
      "images/group-person-pink.png",
      "images/group-person-purple.png",
      "images/group-person-red.png",
      "images/group-person-yellow.png"
   };
   protected static final int HEIGHT_PERSON_GRP_IMG = 45;
   protected static final int WIDTH_PERSON_GRP_IMG  = 45;
   
   //--//
   
   protected static int personGrpNum   = 0;
   protected static int personGrpColor = 0;

   //--//
   
   public static PersonGrpWidget createPersonGrpWidget(){
      return ActorFactory.createPersonGrpWidget(ActorFactory.createPersonGrp(),true, false);
   }
   
   public static PersonGrpWidget createPersonGrpWidget(ScenicPersonGrp personGrpBean){
      return ActorFactory.createPersonGrpWidget(ActorFactory.createPersonGrp(personGrpBean),false,false);
   }
   
   public static PersonGrpWidget createThumbnailPersonGrpWidget(ScenicPersonGrp personGrpBean){
      return ActorFactory.createPersonGrpWidget(ActorFactory.createPersonGrp(personGrpBean),false,true);
   }
   
   //--//
   
   private static PersonGrpWidget createPersonGrpWidget(PersonGrp personGrp, boolean useDefaultIcon, boolean thumbnail){
      int         width      = (thumbnail)? 
                               (int)(ActorFactory.WIDTH_PERSON_GRP_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.WIDTH_PERSON_GRP_IMG;
      int         height     = (thumbnail)?
                               (int)(ActorFactory.HEIGHT_PERSON_GRP_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.HEIGHT_PERSON_GRP_IMG;
      Image       personGrpImg = new Image(personGrp.getImgUrl());
      Image       defaultImg   = new Image(ActorFactory.personGrpImgs[0]);
      LabeledIcon lblIco       = new LabeledIcon(personGrpImg,
                                                 defaultImg,
                                                 width,
                                                 height,
                                                 new HTML("&nbsp;" + personGrp.getName() + "&nbsp;"),
                                                 useDefaultIcon);
      return new PersonGrpWidget(lblIco, personGrp);
   }
   
   private static PersonGrp createPersonGrp(){
      PersonGrp personGrp = new PersonGrp(ActorFactory.ID_PREFIX_PERSON_GRP + ActorFactory.actorNum,
                                          ActorFactory.actorNum                                    ,
                                          "Person Group " + ActorFactory.personGrpNum              ,
                                          ActorFactory.personGrpImgs[ActorFactory.personGrpColor]);
      
      ActorFactory.actorNum++;
      ActorFactory.personGrpNum++;
      ActorFactory.personGrpColor = (ActorFactory.personGrpColor + 1) % ActorFactory.personGrpImgs.length;

      return personGrp;
   }
   
   private static PersonGrp createPersonGrp(ScenicPersonGrp spec){
      return new PersonGrp(ActorFactory.ID_PREFIX_PERSON_GRP + spec.getId(),
                           (int)spec.getId()                               ,
                           spec.getName()                                  ,
                           spec.getImgUrl() );
   }
   
   
   //-------------------------- THING --------------------------//

   protected static final String ID_PREFIX_THING = "Thing";
   protected static final String[] thingImgs = new String[] {
      "images/object-orange.png",
      "images/object-blue.png",
      "images/object-blue-green.png",
      "images/object-pink.png",
      "images/object-purple.png",
      "images/object-red.png",
      "images/object-yellow.png"
   };
   protected static final String[] specificThingImgs = new String[] {
      "images/backpack.png",  // 48 48
      "images/ball.png",   // 48 48
      "images/bike-helmet.png",//   48 48
      "images/binder.png", // 96 96
      "images/book.png",   // 48 48
      "images/camcorder.png", // 64 64
      "images/camera.png", // 48 48
      "images/idcard.png", // 48 48
      "images/ipod.png",   // 96 96
      "images/jacket.png", // 64 64
      "images/keys.png",   // 96 96
      "images/laptop.png", // 96 96
      "images/mop.png", // 50 76
      "images/mouse.png",  // 48 48
      "images/mug.png", // 96 96
      "images/notebook.png",  // 64 64
      "images/pants.png",  // 96 96
      "images/pc.png",  // 158   134
      "images/pda.png", // 64 64
      "images/phone.png",  // 96 96
      "images/power-battery.png",// 48 48
      "images/power-plug.png",// 48 48
      "images/shoe.png",   // 64 64
      "images/skateboard.png",// 48 48
      "images/tank-top.png",  // 96 96
      "images/teapot.png", // 96 96
      "images/umbrella.png",  // 96 96
      "images/wallet.png" // 64 64
   };
   protected static final int[] specificThingImgWidths = new int[] {
      48,
      48,
      48,
      96,
      48,
      64,
      48,
      48,
      96,
      64,
      96,
      96,
      50,
      48,
      96,
      64,
      96,
      158,
      64,
      96,
      48,
      48,
      64,
      48,
      96,
      96,
      96,
      64
   };
   protected static final int[] specificThingImgHeights = new int[] {
      48,
      48,
      48,
      96,
      48,
      64,
      48,
      48,
      96,
      64,
      96,
      96,
      76,
      48,
      96,
      64,
      96,
      134,
      64,
      96,
      48,
      48,
      64,
      48,
      96,
      96,
      96,
      64,
   };
   
   protected static final int HEIGHT_THING_IMG = 18;
   protected static final int WIDTH_THING_IMG  = 19;
   
   //--//
   
   protected static int thingNum;
   protected static int thingColor;

   //--//
   
   public static ThingWidget createThingWidget(){
      return ActorFactory.createThingWidget(ActorFactory.createThing(),true, false);
   }

   
   public static ThingWidget createThingWidget(ScenicThing spec){
      return ActorFactory.createThingWidget(ActorFactory.createThing(spec),false, false);
   }
   
   
   public static ThingWidget createThumbnailThingWidget(ScenicThing spec){
      return ActorFactory.createThingWidget(ActorFactory.createThing(spec),false, true);
   }
   
   //--//
   
   private static ThingWidget createThingWidget(Thing thing, boolean useDefaultIcon, boolean thumbnail){
   
      int width  = 0;
      int height = 0;
      
      if(!thing.getImgUrl().contains("object-")){
         int m = 0; // matching index
         for(int i = 0; i < ActorFactory.specificThingImgs.length; i++){
            if(thing.getImgUrl().equals(ActorFactory.specificThingImgs[i])){
               m = i;
               break;
            }
         }
         width  = (thumbnail)? 
                   (int)(ActorFactory.specificThingImgWidths[m]/SequenceFactory.THUMBNAIL_FACTOR) :
                         ActorFactory.specificThingImgWidths[m];
         height = (thumbnail)?
                   (int)(ActorFactory.specificThingImgHeights[m]/SequenceFactory.THUMBNAIL_FACTOR) :
                         ActorFactory.specificThingImgWidths[m];
      } else {
         width  = (thumbnail)? 
                   (int)(ActorFactory.WIDTH_THING_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                         ActorFactory.WIDTH_THING_IMG;
         height = (thumbnail)?
                   (int)(ActorFactory.HEIGHT_THING_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                         ActorFactory.HEIGHT_THING_IMG;
      }
      
      
      Image       thingImg   = new Image(thing.getImgUrl());
      Image       defaultImg = new Image(ActorFactory.thingImgs[0]);
      LabeledIcon lblIco     = new LabeledIcon(thingImg,
                                               defaultImg,
                                               width,
                                               height,
                                               new HTML("&nbsp;" + thing.getName() + "&nbsp;"),
                                               useDefaultIcon);
      return new ThingWidget(lblIco, thing);
   }
   
   private static Thing createThing(){
      Thing thing = new Thing(ActorFactory.ID_PREFIX_THING + ActorFactory.actorNum,
                              ActorFactory.actorNum                               ,
                              "Thing "+ ActorFactory.thingNum                     ,
                              ActorFactory.thingImgs[ActorFactory.thingColor]);
      ActorFactory.actorNum++;
      ActorFactory.thingNum++;
      ActorFactory.thingColor = (ActorFactory.thingColor + 1) % ActorFactory.thingImgs.length;

      return thing;
   }
   
   public static Thing createThing(ScenicThing spec){
      Thing thing = new Thing(ActorFactory.ID_PREFIX_THING + spec.getId(),
                              (int)spec.getId()                          ,
                              spec.getName()                             ,
                              spec.getImgUrl() );
      return thing;
   }
   
   
   //-------------------------- THING GROUP --------------------------//
   
   protected static final String ID_PREFIX_THING_GRP = "ThingGrp";
   protected static final String[] thingGrpImgs = new String[] {
      "images/group-object-orange.png",
      "images/group-object-blue.png",
      "images/group-object-blue-green.png",
      "images/group-object-pink.png",
      "images/group-object-purple.png",
      "images/group-object-red.png",
      "images/group-object-yellow.png"
   };
   protected static final int HEIGHT_THING_GRP_IMG = 40;
   protected static final int WIDTH_THING_GRP_IMG  = 28;
   
   
   
   //--//
   
   protected static int thingGrpNum   = 0;
   protected static int thingGrpColor = 0;
   
   //--//
   
   public static ThingGrpWidget createThingGrpWidget(){
      return ActorFactory.createThingGrpWidget(ActorFactory.createThingGrp(), true, false);
   }
   
   
   public static ThingGrpWidget createThingGrpWidget(ScenicThingGrp thingGrpBean){
      return ActorFactory.createThingGrpWidget(ActorFactory.createThingGrp(thingGrpBean), false, false);
   }
   
   
   public static ThingGrpWidget createThumbnailThingGrpWidget(ScenicThingGrp thingGrpBean){
      return ActorFactory.createThingGrpWidget(ActorFactory.createThingGrp(thingGrpBean), false, true);
   }
   
   
   //--//
   
   private static ThingGrpWidget createThingGrpWidget(ThingGrp thingGrp, boolean useDefaultIcon, boolean thumbnail){
      int         width      = (thumbnail)? 
                                 (int)(ActorFactory.WIDTH_THING_GRP_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                       ActorFactory.WIDTH_THING_GRP_IMG;
      int         height     = (thumbnail)?
                                 (int)(ActorFactory.HEIGHT_THING_GRP_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                       ActorFactory.HEIGHT_THING_GRP_IMG;
      Image       thingImg   = new Image(thingGrp.getImgUrl());
      Image       defaultImg = new Image(ActorFactory.thingGrpImgs[0]);
      LabeledIcon lblIco     = new LabeledIcon(thingImg,
                                               defaultImg,
                                               width,
                                               height,
                                               new HTML("&nbsp;" + thingGrp.getName() + "&nbsp;"),
                                               useDefaultIcon);
      
      return new ThingGrpWidget(lblIco, thingGrp); 
   }
   
   private static ThingGrp createThingGrp(){
      ThingGrp thingGrp = new ThingGrp(ActorFactory.ID_PREFIX_THING_GRP + ActorFactory.actorNum,
                                       ActorFactory.actorNum                                                     ,
                                       "Group of Things " + ActorFactory.thingGrpNum                             ,
                                       ActorFactory.thingGrpImgs[ActorFactory.thingGrpColor]);

      ActorFactory.actorNum++;
      ActorFactory.thingGrpNum++;
      ActorFactory.thingGrpColor = (ActorFactory.thingGrpColor + 1) % ActorFactory.thingGrpImgs.length;

      return thingGrp;
   }
   
   private static ThingGrp createThingGrp(ScenicThingGrp spec){
      return new ThingGrp(ActorFactory.ID_PREFIX_THING_GRP + spec.getId(),
                          (int)spec.getId()                              ,
                          spec.getName()                                 ,
                          spec.getImgUrl() );
   }
   
   
   //----------------------------- PLACE -----------------------------//
   
   protected static final String ID_PREFIX_PLACE = "Place";
   protected static final String[] placeImgs = new String[] {
      "images/place-pixel.png"
   };
   protected static final int HEIGHT_PLACE_IMG = 75;
   protected static final int WIDTH_PLACE_IMG  = 60;
   
   //--//
   
   protected static int placeNum;
   protected static int placeColor;
   
   //--//
   
   public static PlaceWidget createPlaceWidget(){
      return ActorFactory.createPlaceWidget(ActorFactory.createPlace(), true, false);
   }
   
   
   public static PlaceWidget createPlaceWidget(ScenicPlace placeBean){
      return ActorFactory.createPlaceWidget(ActorFactory.createPlace(placeBean), false, false);
   }
   
   
   public static PlaceWidget createThumbnailPlaceWidget(ScenicPlace placeBean){
      return ActorFactory.createPlaceWidget(ActorFactory.createPlace(placeBean), false, true);
   }
   
   //--//
   
   private static PlaceWidget createPlaceWidget(Place place, boolean useDefaultIcon, boolean thumbnail){
      int         width      = (thumbnail)? 
                               (int)(ActorFactory.WIDTH_PLACE_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.WIDTH_PLACE_IMG;
      int         height     = (thumbnail)?
                               (int)(ActorFactory.HEIGHT_PLACE_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
                                     ActorFactory.HEIGHT_PLACE_IMG;
      Image       placeImg   = new Image(place.getImgUrl());
      Image       defaultImg = new Image(ActorFactory.placeImgs[0]);
      LabeledIcon lblIco     = new LabeledIcon(placeImg,
                                               defaultImg,
                                               width,
                                               height,
                                               new HTML("&nbsp;" + place.getName() + "&nbsp;"), 
                                               useDefaultIcon);       
      return new PlaceWidget(lblIco, place);
   }
   
   private static Place createPlace(){
      
      Place place = new Place(ActorFactory.ID_PREFIX_PLACE + ActorFactory.actorNum,
                              ActorFactory.actorNum                               ,
                              "Place " + ActorFactory.placeNum                    ,
                              ActorFactory.placeImgs[ActorFactory.placeColor]);      
      ActorFactory.actorNum++;
      ActorFactory.placeNum++;
      ActorFactory.placeColor = (ActorFactory.placeColor + 1) % ActorFactory.placeImgs.length;

      return place;
   }
   
   private static Place createPlace(ScenicPlace spec){
      return new Place(ActorFactory.ID_PREFIX_PLACE + spec.getId(),
                       (int)spec.getId()                          ,
                       spec.getName()                             ,
                       spec.getImgUrl() );
   }
}
