package edu.washington.cs.rfid.cascadia.scenic.client.ui.factory;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Far;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Inside;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Lasts;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Near;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Outside;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Primitive;
import edu.washington.cs.rfid.cascadia.scenic.client.data.With;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Without;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicFar;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicInside;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicNear;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicOutside;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicPrimitive;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicWith;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicWithout;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.FarWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.InsideWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.LastsWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.NearWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.OutsideWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.PrimitiveWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.WithWidget;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.widget.WithoutWidget;

public class PrimitiveFactory {

	protected static final String ID_PREFIX_WITH    = "with";
	protected static final String ID_PREFIX_WITHOUT = "without";
	protected static final String ID_PREFIX_INSIDE  = "inside";
	protected static final String ID_PREFIX_OUTSIDE = "outside";
	protected static final String ID_PREFIX_NEAR    = "near";
	protected static final String ID_PREFIX_FAR     = "far";
	protected static final String ID_PREFIX_LASTS   = "lasts";
	
	protected static final String withImg           = "images/primitive-with.png";
	protected static final String withoutImg        = "images/primitive-without.png";
	protected static final String insideImg         = "images/primitive-inside.png";
	protected static final String outsideImg        = "images/primitive-outside.png";
	protected static final String nearImg           = "images/primitive-near.png";
	protected static final String farImg            = "images/primitive-far.png";
	protected static final String lastsImg          = "images/primitive-lasts.png";
	
	protected static final int    WIDTH_PRIMITIVE_IMG  = 60;
	protected static final int    HEIGHT_PRIMITIVE_IMG = 30;
	
	//--//
	
	protected static int primitiveNum   = 0;
	protected static int currWithNum    = 0;
	protected static int currWithoutNum = 0;
	protected static int currInsideNum  = 0;
	protected static int currOutsideNum = 0;
	protected static int currNearNum    = 0;
	protected static int currFarNum     = 0;
	protected static int currLastsNum   = 0;
	
	//--//
	
	public static PrimitiveWidget createWidget(ScenicPrimitive sp){
	   PrimitiveWidget pw = null;
      
      if(sp instanceof ScenicWith){
         pw = PrimitiveFactory.createWithWidget();
      } else if(sp instanceof ScenicWithout){
         pw = PrimitiveFactory.createWithoutWidget();
      } else if(sp instanceof ScenicInside){
         pw = PrimitiveFactory.createInsideWidget();
      } else if(sp instanceof ScenicOutside){
         pw = PrimitiveFactory.createOutsideWidget();
      } else if(sp instanceof ScenicNear){
         pw = PrimitiveFactory.createNearWidget();
      } else if(sp instanceof ScenicFar){
         pw = PrimitiveFactory.createFarWidget();
      }
      pw.setLabelVisible(true);
      return pw;
	}
	
	public static PrimitiveWidget createThumbnailWidget(ScenicPrimitive sp){
      PrimitiveWidget pw = null;
      
      if(sp instanceof ScenicWith){
         pw = PrimitiveFactory.createThumbnailWithWidget();
      } else if(sp instanceof ScenicWithout){
         pw = PrimitiveFactory.createThumbnailWithoutWidget();
      } else if(sp instanceof ScenicInside){
         pw = PrimitiveFactory.createThumbnailInsideWidget();
      } else if(sp instanceof ScenicOutside){
         pw = PrimitiveFactory.createThumbnailOutsideWidget();
      } else if(sp instanceof ScenicNear){
         pw = PrimitiveFactory.createThumbnailNearWidget();
      } else if(sp instanceof ScenicFar){
         pw = PrimitiveFactory.createThumbnailFarWidget();
      }
      return pw;
   }
	
	public static PrimitiveWidget createWithWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_WITH, false );
	}

	public static PrimitiveWidget createWithoutWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_WITHOUT, false );
	}
	
	public static PrimitiveWidget createInsideWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_INSIDE, false );
	}
	
	public static PrimitiveWidget createOutsideWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_OUTSIDE, false );
	}
	
	public static PrimitiveWidget createNearWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_NEAR, false );
	}
	
	public static PrimitiveWidget createFarWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_FAR, false );
	}
	
	public static PrimitiveWidget createLastsWidget(){
		return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_LASTS, false );
	}
	
	
	public static PrimitiveWidget createThumbnailWithWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_WITH, true );
   }

   public static PrimitiveWidget createThumbnailWithoutWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_WITHOUT, true );
   }
   
   public static PrimitiveWidget createThumbnailInsideWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_INSIDE, true );
   }
   
   public static PrimitiveWidget createThumbnailOutsideWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_OUTSIDE, true );
   }
   
   public static PrimitiveWidget createThumbnailNearWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_NEAR, true );
   }
   
   public static PrimitiveWidget createThumbnailFarWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_FAR, true );
   }
   
   public static PrimitiveWidget createThumbnailLastsWidget(){
      return PrimitiveFactory.createWidget( PrimitiveFactory.ID_PREFIX_LASTS, true );
   }
	
	
	//--//
	
	private static With createWith(){
		
		With with = new With(PrimitiveFactory.ID_PREFIX_WITH+ PrimitiveFactory.primitiveNum  ,
                           PrimitiveFactory.primitiveNum                                   ,
                           PrimitiveFactory.ID_PREFIX_WITH,
                           PrimitiveFactory.withImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currWithNum++;

		return with;
	}
	
	private static Without createWithout(){
		
		Without without = new Without(PrimitiveFactory.ID_PREFIX_WITHOUT+ PrimitiveFactory.primitiveNum,
                                    PrimitiveFactory.primitiveNum                                             ,
                                    PrimitiveFactory.ID_PREFIX_WITHOUT    ,
                                    PrimitiveFactory.withoutImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currWithoutNum++;

		return without;
	}
	
	private static Inside createInside(){
		
		Inside inside = new Inside(PrimitiveFactory.ID_PREFIX_INSIDE+ PrimitiveFactory.primitiveNum    ,
                                   PrimitiveFactory.primitiveNum                                       ,
                                   PrimitiveFactory.ID_PREFIX_INSIDE,
                                   PrimitiveFactory.insideImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currInsideNum++;

		return inside;
	}

	private static Outside createOutside(){
		
		Outside outside = new Outside(PrimitiveFactory.ID_PREFIX_OUTSIDE+ PrimitiveFactory.primitiveNum     ,
                                      PrimitiveFactory.primitiveNum                                         ,
                                      PrimitiveFactory.ID_PREFIX_OUTSIDE,
                                      PrimitiveFactory.outsideImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currOutsideNum++;

		return outside;
	}
	
	private static Near createNear(){
		
		Near near = new Near(PrimitiveFactory.ID_PREFIX_NEAR+ PrimitiveFactory.primitiveNum  ,
                             PrimitiveFactory.primitiveNum                                   ,
                             PrimitiveFactory.ID_PREFIX_NEAR,
                             PrimitiveFactory.nearImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currNearNum++;

		return near;
	}
	
	private static Far createFar(){
		
		Far far = new Far(PrimitiveFactory.ID_PREFIX_FAR+ PrimitiveFactory.primitiveNum ,
                          PrimitiveFactory.primitiveNum                                 ,
                          PrimitiveFactory.ID_PREFIX_FAR,
                          PrimitiveFactory.farImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currFarNum++;

		return far;
	}
	
	private static Lasts createLasts(){
		Lasts lasts = new Lasts(PrimitiveFactory.ID_PREFIX_LASTS+ PrimitiveFactory.primitiveNum   ,
                                PrimitiveFactory.primitiveNum                                     ,
                                PrimitiveFactory.ID_PREFIX_LASTS,
                                PrimitiveFactory.lastsImg);

		PrimitiveFactory.primitiveNum++;
		PrimitiveFactory.currLastsNum++;

		return lasts;
	}
	
	private static PrimitiveWidget createWidget(String type, boolean thumbnail){
		
		PrimitiveWidget primitiveWidget = null;
		Primitive       primitive       = null;
		
		if(type.equals(PrimitiveFactory.ID_PREFIX_WITH)){
			primitive = PrimitiveFactory.createWith();
		} else if(type.equals(PrimitiveFactory.ID_PREFIX_WITHOUT)){
			primitive = PrimitiveFactory.createWithout();
		} else if(type.equals(PrimitiveFactory.ID_PREFIX_INSIDE)){
			primitive = PrimitiveFactory.createInside();
		} else if(type.equals(PrimitiveFactory.ID_PREFIX_OUTSIDE)){
			primitive = PrimitiveFactory.createOutside();
		} else if(type.equals(PrimitiveFactory.ID_PREFIX_NEAR)){
			primitive = PrimitiveFactory.createNear();
		} else if(type.equals(PrimitiveFactory.ID_PREFIX_FAR)){
			primitive = PrimitiveFactory.createFar();
		} else {	
			primitive = PrimitiveFactory.createLasts();
		}
		int   width          = (thumbnail)? 
		                       (int)(PrimitiveFactory.WIDTH_PRIMITIVE_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
		                             PrimitiveFactory.WIDTH_PRIMITIVE_IMG;
		int   height         = (thumbnail)?
		                       (int)(PrimitiveFactory.HEIGHT_PRIMITIVE_IMG/SequenceFactory.THUMBNAIL_FACTOR) :
		                             PrimitiveFactory.HEIGHT_PRIMITIVE_IMG;
		Image primitiveImage = new Image(primitive.getImgUrl());
      Image     defaultImg = new Image(primitive.getImgUrl());
      LabeledIcon lblIco   = new LabeledIcon(primitiveImage,
                                             defaultImg,
                                             width,
                                             height,
                                             new HTML("&nbsp;" + primitive.getName() + "&nbsp;"),
                                             true);

      if(type.equals(PrimitiveFactory.ID_PREFIX_WITH)){
         primitiveWidget = new WithWidget(lblIco, (With) primitive);
      } else if(type.equals(PrimitiveFactory.ID_PREFIX_WITHOUT)){
         primitiveWidget = new WithoutWidget(lblIco, (Without) primitive);
      } else if(type.equals(PrimitiveFactory.ID_PREFIX_INSIDE)){
         primitiveWidget = new InsideWidget(lblIco, (Inside) primitive);
      } else if(type.equals(PrimitiveFactory.ID_PREFIX_OUTSIDE)){
         primitiveWidget = new OutsideWidget(lblIco, (Outside) primitive);
      } else if(type.equals(PrimitiveFactory.ID_PREFIX_NEAR)){
         primitiveWidget = new NearWidget(lblIco, (Near) primitive);
      } else if(type.equals(PrimitiveFactory.ID_PREFIX_FAR)){
         primitiveWidget = new FarWidget(lblIco, (Far) primitive);
      } else { 
         primitiveWidget = new LastsWidget(lblIco, (Lasts) primitive);
      }
		return primitiveWidget;
	}
}
