package edu.washington.cs.rfid.cascadia.scenic.client.ui.factory;

import java.util.List;

import com.google.gwt.user.client.DeferredCommand;

import edu.washington.cs.rfid.cascadia.scenic.client.command.ConnectorRedrawCmd;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicScene;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicSequence;
import edu.washington.cs.rfid.cascadia.scenic.client.grammar.ScenicTransition;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.Scene;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.SequencePanel;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.Transition;

public class SequenceFactory {

   public static final double THUMBNAIL_FACTOR = 3.5;
   
   //--//
   
   protected static final int SCENE_SPACING    = 10;
   protected static final int HEIGHT_SEQUENCE  = 350;   
   
   //--//
   
   public static SequencePanel createSequencePanel(){
      return new SequencePanel(SequenceFactory.HEIGHT_SEQUENCE + "px",
                               SequenceFactory.SCENE_SPACING,
                               true,
                               false);
   }
   
   
   public static SequencePanel createThumbnailSequencePanel(){
      return new SequencePanel((int)(SequenceFactory.HEIGHT_SEQUENCE / SequenceFactory.THUMBNAIL_FACTOR) + "px",
                               SequenceFactory.SCENE_SPACING,
                               false,
                               true);
   }
   
   
   public static SequencePanel createSequencePanel(ScenicSequence spec){
      //this.sequencePanel.clearScenes();
      SequencePanel          sequencePanel = SequenceFactory.createSequencePanel();
      List<ScenicScene>      scenicScenes  = spec.getScenes();
      List<ScenicTransition> transitions   = spec.getTransitions();
      
      for(int i = 0; i < scenicScenes.size(); i++){
         ScenicScene ss = scenicScenes.get(i);
         Scene       s  = SceneFactory.createScene(ss, sequencePanel,true);
         sequencePanel.insertScene(s, i);
         DeferredCommand.addCommand(new ConnectorRedrawCmd(s));
      }
      for(int i = 0; i < transitions.size(); i++){
         ScenicTransition st = transitions.get(i);
         Transition       t  = sequencePanel.getTransition(i);
         t.setState(t.getType());
      }
      
      return sequencePanel;
   }
   
   
   public static SequencePanel createSequencePanel(ScenicSequence spec, SequencePanel sequencePanel){
      sequencePanel.clearScenes();
      List<ScenicScene>      scenicScenes      = spec.getScenes();
      List<ScenicTransition> scenicTransitions = spec.getTransitions();
      
      for(int i = 0; i < scenicScenes.size(); i++){
         ScenicScene ss = scenicScenes.get(i);
         Scene       s  = SceneFactory.createScene(ss, sequencePanel,true);
         sequencePanel.insertScene(s, i);
         DeferredCommand.addCommand(new ConnectorRedrawCmd(s));
      }
      for(int i = 0; i < scenicTransitions.size(); i++){
         ScenicTransition st = scenicTransitions.get(i);
         Transition       t  = sequencePanel.getTransition(i);
         t.setState(st.getType());
      }
      return sequencePanel;
   }
   
   
   public static SequencePanel createThumbnailSequencePanel(ScenicSequence spec){
      SequencePanel          sequencePanel     = SequenceFactory.createThumbnailSequencePanel();
      List<ScenicScene>      scenicScenes      = spec.getScenes();
      List<ScenicTransition> scenicTransitions = spec.getTransitions();
      
      for(int i = 0; i < scenicScenes.size(); i++){
         ScenicScene ss = scenicScenes.get(i);
         Scene       s  = SceneFactory.createThumbnailScene(ss, sequencePanel,false);
         sequencePanel.insertScene(s, i);
         DeferredCommand.addCommand(new ConnectorRedrawCmd(s));
      }
      for(int i = 0; i < scenicTransitions.size(); i++){
         ScenicTransition st = scenicTransitions.get(i);
         Transition       t  = sequencePanel.getTransition(i);
         t.setState(st.getType());
      }
      return sequencePanel;
   }
}