package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.RadioButton;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Actor;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ContextMenu;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;


public class ActorWidget extends IconWidget {
   
   protected static final String PROPERTIES_TITLE         = "<b>Default Properties</b>";
   protected static final String PROPERTIES_NAME_PREFIX   = "Select actor:&nbsp;&nbsp;";
   protected static final String PROPERTIES_ICON_PREFIX   = "Current icon:&nbsp;&nbsp;";

   protected static final String BUTTON_SAVE_TXT          = "Save";
   protected static final String BUTTON_CANCEL_TXT        = "Cancel";
   protected static final String BUTTON_APPLY_TXT         = "Apply";
   
   protected static final int    BUTTON_SPACER_WIDTH      = 110;
   
   
   
   //--//
   
   protected Actor           actor;
   protected LabeledIcon     lblIco;
   
   protected HTML            nameBoxPrefix;
   protected HorizontalPanel nameBoxPanel;
   protected ListBox         namePropertyBox;
   protected HTML            iconPrefix;
   protected HorizontalPanel iconPanel;
   protected Image           icon;
   
   //--//
   
   public ActorWidget(LabeledIcon  lblIco ,
                      Actor        actor){
      super(lblIco);
      
      this.actor         = actor;
      this.lblIco        = lblIco;
      this.propTitle     = new HTML( ActorWidget.PROPERTIES_TITLE       );
      this.nameBoxPrefix = new HTML( ActorWidget.PROPERTIES_NAME_PREFIX );
      this.iconPrefix    = new HTML( ActorWidget.PROPERTIES_ICON_PREFIX );
      this.icon          = new Image( lblIco.img.getUrl() );
      this.iconMenu      = new ContextMenu();
      
      JavaScriptHelper.disableContextMenu(iconMenu.getElement(), true);
      this.iconMenu.addItem( "Make Connection", new CmdConnect(this));
      this.iconMenu.addItem( "Properties"  , new CmdProperties(this) );
      this.iconMenu.addItem( "Delete", new CmdDelete(this)     );
      
      this.iconMenu.hide();
      
      this.buildIconPanel();
      this.buildPropertiesContent();
      
      this.propDialog.setContent(this.propContent);
   }
   
   
   public ActorWidget(LabeledIcon                   lblIco ,
                      Actor                         actor  ,
                      Vector<ConnectorHandleWidget> connectors){
      this(lblIco,actor);
      this.connectors = connectors;
   }
   
   public void printLabledIconStats(){
      System.out.println("width abs: " + this.lblIco.absPanel.getOffsetWidth() + " height abs: " + this.lblIco.absPanel.getOffsetHeight());
      System.out.println("width img: " + this.lblIco.img.getOffsetWidth() + " height img: " + this.lblIco.img.getOffsetHeight());
      System.out.println("width total: "  +this.lblIco.getOffsetWidth() + " height total: " + this.lblIco.getOffsetHeight());
   }
   
   
   public Actor getActor(){
      return this.actor;
   }
   
   
   public String getImgUrl(){
       String url = null;
       Image  img = this.lblIco.getImage();
       
       if(null != img){
          url = img.getUrl();
       }
       return url;
   }
   
   
   public int getWidth(){
      return this.lblIco.img.getWidth();
   }
   
   
   public int getHeight(){
      return this.lblIco.img.getHeight();
   }
   
   
   public int getIconMaxWidth(){
      return this.lblIco.maxWidthImg;
   }
   
   
   public int getIconMaxHeight(){
      return this.lblIco.maxHeightImg;
   }
   
   
   public void setActor(Actor a){
      this.actor = a;
   }
   
   
   @Override
   public IconWidget cloneWidget(){

      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())       ,
                                            this.lblIco.defaultImg, //new Image(this.lblIco.defaultImg.getUrl()), 
                                            new HTML(this.lblIco.label.getHTML())     , 
                                            false );
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
         
      return (IconWidget) new ActorWidget(newIco      , 
                                          this.actor  ,
                                          connectorClone);
   }
   
   
   @Override
   public IconWidget nextWidget() {
      
      return null;
      // TODO: Below is the prototype for all other widget classes!
//      ActorWidget next =  this.factory.createWidget();
//      
//      next.setLabelVisible(false);
//
//      return (IconWidget) next;
   }

   @Override
   public void wigdetPickedUp(){
      this.pickedUp = true;
      this.lblIco.switchFromDefault();
   }
   
   
   public void switchFromDefaultImg(){
      this.lblIco.switchFromDefault();
   }
   
   public void setLabelVisible(boolean visible){
      this.lblIco.setLabelVisible(visible);
   }
   
   
   public boolean getLabelVisible(){
      return this.lblIco.getLabelVisible();
   }
   
   
   public int getRelativeXConnectorAnchor(){
      int anchorX = 0;
      if(this.getLabelVisible()){
         anchorX = this.lblIco.label.getOffsetWidth() / 2;
      } else {
         anchorX = this.lblIco.img.getWidth() / 2;
      }
      return anchorX;
   }
   
   
   public int getRelativeYConnectorAnchor(){
      return this.lblIco.img.getHeight() / 2;
   }
   
   
   //--//
   
   
   @Override
   protected void buildPropertiesContent(){

      HorizontalPanel titlePanel = new HorizontalPanel();

      titlePanel.add( this.propTitle );

      ArrayList<String> names = this.getPropertiesNames();
      names                   = this.getPropertiesNames();
      
      //Collections.sort(names);
      
      
      this.buildNameBoxPanel(names);
      this.buildPropertiesButtons();
      
      this.propContent = new VerticalPanel();
      
      this.propContent.add( titlePanel             );
      this.propContent.add( new HTML("<br>&nbsp;") );
      this.propContent.add( nameBoxPanel           );
      this.propContent.add( new HTML("<br>&nbsp;") );
      this.propContent.add( this.iconPanel         );
      this.propContent.add( new HTML("<br>&nbsp;") );
      this.propContent.add( this.buttonPanel       );
   }
   
   
   protected ArrayList<String> getPropertiesNames(){
      return new ArrayList<String>();
   }
   
   
   protected void buildNameBoxPanel(ArrayList<String> names){

      this.nameBoxPanel    = new HorizontalPanel();
      this.namePropertyBox = new ListBox();
      
      for(int i = 0; i < names.size(); i++){
         this.namePropertyBox.addItem( names.get(i) );
      }
      
      this.namePropertyBox.addChangeHandler(new ChangeHandler(){
         public void onChange(ChangeEvent e){
            // Do nothing for now
         }
      });
      
      HorizontalPanel nameBoxContainer = new HorizontalPanel();
      
      nameBoxContainer.add( this.nameBoxPrefix   );
      nameBoxContainer.add( this.namePropertyBox );
   
      this.nameBoxPanel.add( nameBoxContainer );
      this.nameBoxPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.nameBoxPanel.setCellHorizontalAlignment( nameBoxContainer, HasHorizontalAlignment.ALIGN_CENTER);
   }
   
   
   protected void buildIconPanel(){
      
      this.iconPanel = new HorizontalPanel();
      
      HorizontalPanel iconContainer = new HorizontalPanel();
      
      iconContainer.add( this.iconPrefix );
      iconContainer.add( this.icon       );
      
      this.iconPanel.add( iconContainer );
      this.iconPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.iconPanel.setCellHorizontalAlignment(  iconContainer, HasHorizontalAlignment.ALIGN_CENTER);
   }
   
   
   @Override
   protected void applyPropertiesChanges(){      
      String selectedName = this.namePropertyBox.getItemText( this.namePropertyBox.getSelectedIndex() );
      
      this.lblIco.maxWidthImg  = 48; // TODO: this is experimental 10/6/09, hack for demo
      this.lblIco.maxHeightImg = 48;

      this.actor          = this.findActorByName(selectedName); // Change Image called inside HERE

      this.lblIco.setLabel("&nbsp;" + selectedName + "&nbsp;");
      
// TODO: do we really need the call below??      
//      this.lblIco.changeImage(new Image(this.actor.getImgUrl())); // Change image called HERE
      
      // Adjust any connector end points accordingly
      DeferredCommand.addCommand(new CmdRefreshConnectors(this));
   }
   
   
   /**
    * Command to readjust the connector end point positions.
    * @author evan
    */
   protected static class CmdRefreshConnectors implements Command {
      
      protected IconWidget w;
      
      public CmdRefreshConnectors(IconWidget w){
         this.w = w;
      }
      
      public void execute(){
         w.scene.refreshConnectors(w);
      }
   }
   
   
   protected Actor findActorByName(String name){
      return null;
   }
}