package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TextBoxBase;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Far;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;

public class FarWidget extends PrimitiveWidget {

	protected static final String PROPERTIES_TITLE               = "<b>&nbsp;&nbsp;Far Primitive Properties</b>";
	
	protected static final String PROPERTIES_DEFAULT_DISTANCE    = "5";
	protected static final int    PROPERTIES_DISTANCE_BOX_LENGTH = 4;
	protected static final String PROPERTIES_DISTANCE_BOX_PREFIX = "Farther than&nbsp;&nbsp;";
	protected static final String PROPERTIES_DISTANCE_BOX_SUFFIX = "&nbsp;&nbsp;meters";
	
	//--//
	
	protected HorizontalPanel distanceBoxPanel;
	protected TextBox         distancePropertyBox;
	
	//--//
	
	public FarWidget(LabeledIcon lblIco, Far primitive){
		super(lblIco, primitive);
		
		this.propTitle = new HTML( FarWidget.PROPERTIES_TITLE );
		
		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}
	
	
	public FarWidget(LabeledIcon                   lblIco, 
	                 Far                           primitive,
	                 Vector<ConnectorHandleWidget> connectors){
      this(lblIco, primitive);
      this.connectors = connectors;
   }
	
	
	@Override
   public IconWidget cloneWidget(){
      
      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
                                            this.lblIco.defaultImg               ,
                                            new HTML(this.lblIco.label.getHTML()),
                                            false);
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
      
      return (IconWidget) new FarWidget(newIco        ,
                                        (Far)this.primitive,
                                        connectorClone);
   }
	

	/**
    * Target ok if it's an Actor of some sort.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof ActorWidget){
         targetOk = true;
      }
      
      return super.checkPotentialTarget(w) && targetOk;
   }
   
	
   @Override
   public String getPrimitiveDescription(){
      String         desc       = new String();
      Vector<String> actorNames = new Vector<String>();
      
      for(ConnectorHandleWidget connector : this.connectors){
         if(null != connector.actorWidget){
            actorNames.add(connector.actorWidget.actor.getName());
         }
      }
      
      if(2 <= actorNames.size()){
         desc = "\"";
         if(2 == actorNames.size()){
            desc += actorNames.get(0) + " and " + actorNames.get(1) + " are far each other.\"";
         } else {
            for(int i = 0; i < actorNames.size() - 1; i++){
               desc += actorNames.get(i) + ", ";
            }
            desc += " and " + actorNames.get(actorNames.size() - 1) + " are far each other.\"";
         }
      }
      return desc;
   }
	
   
   /**
    * For now, valid if connected to at least two actors.
    */
   public boolean isValid(){
      boolean valid = true;
      
      if(2 > this.connectors.size()){
         valid = false;
      }
      
      return valid;
   }
   
	
	//--//
	
	
	@Override
	protected void buildPropertiesContent(){

		HorizontalPanel titlePanel = new HorizontalPanel();

		titlePanel.add( this.propTitle );
	
		this.buildPropertiesDescription();
		this.buildDistanceBoxPanel     ();
		this.buildPropertiesButtons    ();
		
		this.propContent = new VerticalPanel();
		
		this.propContent.add( titlePanel                );
		this.propContent.add( new HTML("<br>&nbsp;")    );
		this.propContent.add( this.propDescriptionPanel );
		this.propContent.add( new HTML("<br>&nbsp;")    );
		this.propContent.add( this.distanceBoxPanel     );
		this.propContent.add( new HTML("<br>&nbsp;")    );
		this.propContent.add( this.buttonPanel          );
	}
	
	
	protected void buildPropertiesDescription(){
		this.propDescriptionPanel = new HorizontalPanel();	
		this.propDescription      = new HTML("&quot;X is far from Y&quot;");
		
		HorizontalPanel descContainer = new HorizontalPanel();
		
		descContainer.add( this.propDescription );
		
		this.propDescriptionPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
		this.propDescriptionPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
		this.propDescriptionPanel.add( descContainer );
	}
	
	
	protected void buildDistanceBoxPanel(){
		this.distanceBoxPanel    = new HorizontalPanel();
		
		this.distancePropertyBox = new TextBox();
		this.distancePropertyBox.setText         ( FarWidget.PROPERTIES_DEFAULT_DISTANCE    );
		this.distancePropertyBox.setVisibleLength( FarWidget.PROPERTIES_DISTANCE_BOX_LENGTH );
		this.distancePropertyBox.setTextAlignment( TextBoxBase.ALIGN_RIGHT                   );
		
		HorizontalPanel boxContainer = new HorizontalPanel();
		
		boxContainer.add( new HTML("&nbsp;&nbsp;")                            );
		boxContainer.add( new HTML(FarWidget.PROPERTIES_DISTANCE_BOX_PREFIX) );
		boxContainer.add( this.distancePropertyBox                            );
		boxContainer.add( new HTML(FarWidget.PROPERTIES_DISTANCE_BOX_SUFFIX) );
		
		this.distanceBoxPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
		this.distanceBoxPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
		this.distanceBoxPanel.add(boxContainer);
	}
	
	
	protected void createPropertiesButtons(){
      
      this.saveButton = new Button( IconWidget.BUTTON_SAVE_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkDistanceProperty()){
               applyPropertiesChanges();
               propDialog.close();
            } else {
               Window.alert("You must specify a numeric distance in meters!");
               distancePropertyBox.setText("");
            }
         }
      });
      
      this.cancelButton = new Button( IconWidget.BUTTON_CANCEL_TXT, new ClickHandler(){
         public void onClick(ClickEvent e){
            propDialog.close();
         }
      });
      
      this.applyButton = new Button( IconWidget.BUTTON_APPLY_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkDistanceProperty()){
               applyPropertiesChanges();
            } else {
               Window.alert("You must specify a numeric distance in meters!");
               distancePropertyBox.setText("");
            }
         }
      });
   }

	
	protected boolean checkDistanceProperty(){

	   boolean valid = true;
		String  txt   = this.distancePropertyBox.getText();
		
		if(null != txt){
			try {
				Integer.parseInt(txt);
			} catch(NumberFormatException e){
				valid = false;
			}
		} else {
			valid = false;
		}
		return valid;
	}
}