package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.allen_sauer.gwt.dnd.client.DragEndEvent;
import com.allen_sauer.gwt.dnd.client.DragHandler;
import com.allen_sauer.gwt.dnd.client.DragStartEvent;
import com.allen_sauer.gwt.dnd.client.HasDragHandle;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseDownEvent;
import com.google.gwt.event.dom.client.MouseDownHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.rfid.cascadia.scenic.client.ui.ContextMenu;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.Scene;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;


public abstract class IconWidget extends AbsolutePanel implements HasDragHandle {
   
   protected static final String PROPERTIES_TITLE         = "<b>Icon Properties</b>";
   protected static final String STYLE_PROPERTIES_CONTENT = "propertiesContent";
   
   protected static final String BUTTON_SAVE_TXT          = "Save";
   protected static final String BUTTON_CANCEL_TXT        = "Cancel";
   protected static final String BUTTON_APPLY_TXT         = "Apply";
   
   protected static final int    BUTTON_SPACER_WIDTH      = 160;
   
   //--//

   protected final Widget widget;
   
   //--//
   
   protected FocusPanel                    scenicShim;       // The drag handle
   protected Scene                         scene;            // The Scene that contains this IconWidget
   protected boolean                       pickedUp;         // True if this widget has been picked up

   protected Vector<ConnectorHandleWidget> connectors;       // List of the connectors attached to this widget
   protected ConnectorDragHandler          connectorDragHandler; 
   
   // Context menus
   protected ContextMenu                   iconMenu;         // Context menu for this IconWidget
   
   // Properties Dialog
   protected PropertiesDialog              propDialog;       // Dialog for checking and setting the properties of this IconWidget
   protected HTML                          propTitle;        // Title for the properties dialog
   protected VerticalPanel                 propContent;      // Content for the properties dialog
   
   protected HorizontalPanel               buttonPanel;
   protected HTML                          buttonCellSpacer;
   protected Button                        saveButton;
   protected Button                        cancelButton;
   protected Button                        applyButton;
   
   //--//
   
   public IconWidget(Widget widget){
      
      this.widget     = widget;
      this.scenicShim = new FocusPanel();      
      this.scene      = null;
      this.pickedUp   = false;
      this.connectors = new Vector<ConnectorHandleWidget>();

      
      // Add event handlers
      this.connectorDragHandler = new ConnectorDragHandler(this);
      this.scenicShim.addMouseUpHandler  (new ContextMenuMouseUpHandler());
      this.scenicShim.addMouseUpHandler  (new ConnectorUpHandler(this)   );      
      this.scenicShim.addMouseDownHandler(new CopyMouseDownHandler(this) );
      
      
      // Add context menu
      this.iconMenu   = new ContextMenu();
      JavaScriptHelper.disableContextMenu(iconMenu.getElement(), true);
      this.iconMenu.addItem( "Make Connection", new CmdConnect   (this));
      this.iconMenu.addItem( "Properties"     , new CmdProperties(this));
      this.iconMenu.addItem( "Delete Icon"    , new CmdDelete    (this));
      this.iconMenu.hide();

      //  Add properties dialog
      this.propDialog = new PropertiesDialog();
      JavaScriptHelper.disableContextMenu(this.propDialog.getElement(), true);
      this.propTitle = new HTML( IconWidget.PROPERTIES_TITLE );      
      this.propDialog.hide();

      // Add content and styling
      this.add(widget);       
      this.addStyleName("blank-fire-debug");
      this.scenicShim.addStyleName("icon-shim");
   }

   public abstract IconWidget cloneWidget();
   
   public abstract IconWidget nextWidget();

   public Widget getDragHandle() {
      return this.scenicShim;
   }
   
   public DragHandler getDragHandler(){
      return this.connectorDragHandler;
   }
   
   public abstract int getWidth();
   
   public abstract int getHeight();
   
   public abstract int getRelativeXConnectorAnchor();
   
   public abstract int getRelativeYConnectorAnchor();
   
   public Vector<ConnectorHandleWidget> getConnectorHandles(){
      return this.connectors;
   }
   
   public void setScene(Scene s){
      this.scene = s;
   }
   
   public abstract void setLabelVisible(boolean visible);
   public abstract boolean getLabelVisible();
   
   public void addConnectorHandle(ConnectorHandleWidget connector){
      this.connectors.add(connector);
   }
   
   public void removeConnectorHandle(ConnectorHandleWidget connector){
      if(this.connectors.contains(connector)){
         this.connectors.remove(connector);
      }
   }
   
   public void delete(){
      this.scene.removeIconWidget(this);
      this.iconMenu.hide();
   }
      
   public void showProperties(){
      int x = this.getAbsoluteLeft();
      int y = this.getAbsoluteTop();
      
      this.propDialog.positionRelativeTo(this, x, y);
      this.iconMenu.hide();
   }
   
   
   /**
    * Let shim size match our size.
    * 
    * @param width the desired pixel width
    * @param height the desired pixel height
    */
   @Override
   public void setPixelSize(int width, int height) {
      super.setPixelSize(width, height);
      
      this.scenicShim.setPixelSize(width, height);
      
      System.out.println("set Scenic shim size to w = " + width + " h = " + height);
   }

   /**
    * Let shim size match our size.
    * 
    * @param width the desired CSS width
    * @param height the desired CSS height
    */
   @Override
   public void setSize(String width, String height) {
      super.setSize(width, height);
      
      this.scenicShim.setSize(width, height);
   }

   
   public void wigdetPickedUp(){
      this.pickedUp = true;
   }
   
   
   /**
    * Checks if the specified IconWidget is equal to this IconWidget or
    * is already connected via some other connector.
    * @param w
    * @return
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w != this){
         boolean exists = false;
         for(ConnectorHandleWidget chw : this.connectors){
            if((chw.actorWidget == this && chw.primitiveWidget == w) ||
               (chw.actorWidget == w    && chw.primitiveWidget == this)){
               exists = true;
               break;
            }
         }
         targetOk = !exists;
      }
      return targetOk;
   }
   
   
   //--//
   
   
   /**
    * Adjust the shim size and attach once our widget dimensions are known.
    */
   @Override
   protected void onLoad() {
      super.onLoad();
      
      this.scenicShim.setPixelSize(getOffsetWidth(), getOffsetHeight());
      
     // System.out.println("IconWidget.onLoad: w = " + getOffsetWidth() + " h = " + getOffsetHeight());
      
      this.add(this.scenicShim,0,0);
   }

   
   /**
    * Remove the shim to allow the widget to size itself when reattached.
    */
   @Override
   protected void onUnload() {
      super.onUnload();
      
      this.scenicShim.removeFromParent();
   }
   
   
   //----------------------------- PROPERTIES MENU ------------------------------//
   
   
   protected void buildPropertiesContent(){

      HorizontalPanel titlePanel = new HorizontalPanel();

      titlePanel.add( this.propTitle );

      this.buildPropertiesButtons();
      
      this.propContent = new VerticalPanel();
      
      this.propContent.add( titlePanel             );
      this.propContent.add( new HTML("<br>&nbsp;") );
      this.propContent.add( this.buttonPanel       );
   }
   
   
   protected abstract void applyPropertiesChanges();
   
   
   protected void buildPropertiesButtons(){
      
      this.buttonPanel      = new HorizontalPanel();
      this.buttonCellSpacer = new HTML( "&nbsp;" );
      
      this.createPropertiesButtons();
      
      this.buttonPanel.add( this.buttonCellSpacer );
      this.buttonPanel.add( this.saveButton       );
      this.buttonPanel.add( this.cancelButton     );
      
      this.buttonPanel.setWidth    ( PropertiesDialog.DIALOG_CONTENT_WIDTH  + "px"                );
      this.buttonPanel.setCellWidth( this.buttonCellSpacer, IconWidget.BUTTON_SPACER_WIDTH + "px" );
      
      this.buttonPanel.setCellHorizontalAlignment( this.saveButton  , HasHorizontalAlignment.ALIGN_CENTER );
      this.buttonPanel.setCellHorizontalAlignment( this.cancelButton, HasHorizontalAlignment.ALIGN_CENTER );
   }
   
   
   protected void createPropertiesButtons(){
      
      this.saveButton = new Button( IconWidget.BUTTON_SAVE_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            applyPropertiesChanges();
            propDialog.close();
         }
      });
      
      this.cancelButton = new Button( IconWidget.BUTTON_CANCEL_TXT, new ClickHandler(){
         public void onClick(ClickEvent e){
            propDialog.close();
         }
      });
   }
   
   
   //----------------------------- EVENT HANDLERS ------------------------------//
   
   
   protected class ContextMenuMouseUpHandler implements MouseUpHandler {
      public void onMouseUp(MouseUpEvent e){
         
         int x =   e.getClientX()
                 - DOM.getAbsoluteLeft(scenicShim.getElement())
                 + DOM.getElementPropertyInt(scenicShim.getElement(), "scrollLeft")
                 + Window.getScrollLeft();
           
         int y =   e.getClientY()
                 - DOM.getAbsoluteTop(scenicShim.getElement())
                 + DOM.getElementPropertyInt(scenicShim.getElement(), "scrollTop")
                 + Window.getScrollTop();
           
         if(e.getNativeButton() == NativeEvent.BUTTON_RIGHT && null != scene){
            iconMenu.setPopupPosition( x + getAbsoluteLeft(), y + getAbsoluteTop());
            iconMenu.show();
         }
         e.stopPropagation();
       }
   }
   
   
   protected class ConnectorDragHandler implements DragHandler {
      
      public IconWidget w;
      
      public ConnectorDragHandler(IconWidget w){
         this.w = w;
      }
      
      public void onDragEnd(DragEndEvent e){
         if(this.w == e.getContext().draggable && null != this.w.scene){
            this.w.scene.dragEndIcon(this.w);
         }
      }
      
      public void onDragStart(DragStartEvent e){
         if(this.w == e.getContext().draggable && null != this.w.scene){
            this.w.scene.dragStartIcon(this.w);
         }
      }
      
      public void onPreviewDragEnd(DragEndEvent e){}
      
      public void onPreviewDragStart(DragStartEvent e) {
         if(this.w == e.getContext().draggable && null != this.w.scene){
            if(this.w.scene.getCopying()){
               this.w.scene.copyIcon(this.w);
            }
         }
      }
   }
   
   
   protected class ConnectorUpHandler implements MouseUpHandler {
      
      public IconWidget w;
      
      public ConnectorUpHandler(IconWidget w){
         this.w = w;
      }
      
      public void onMouseUp(MouseUpEvent e){
         if(e.getNativeButton() == NativeEvent.BUTTON_LEFT && null != w.scene){
            w.scene.mouseUpIcon(w);
         }
         e.stopPropagation();
      }
   }
   
   
   protected class CopyMouseDownHandler implements MouseDownHandler {
      
      IconWidget w;
      
      public CopyMouseDownHandler(IconWidget w){
         this.w = w;
      }
      
      public void onMouseDown(MouseDownEvent e){
         if(e.getNativeButton() == NativeEvent.BUTTON_LEFT && null != w.scene){
            if(e.isControlKeyDown()){
               // Then instantly replace this ActorWidget with a copy on the positioning panel
               w.scene.setCopying(true);
            }
         }
         e.stopPropagation();
      }
   }
   
   
   //----------------------------- MENU COMMANDS ------------------------------//
   
   
   protected static class CmdConnect implements Command {
      
      protected IconWidget w;
      
      public CmdConnect(IconWidget w){
         this.w = w;
      }
      
      public void execute(){
         if(null != this.w.scene){
            this.w.scene.setConnectorMode(true,w);
            this.w.iconMenu.hide();
         }
      }
   }
   
   
   protected static class CmdProperties implements Command {
      
      protected IconWidget w;
      
      public CmdProperties(IconWidget w){
         this.w = w;
      }
      
      public void execute(){
         this.w.showProperties();
      }
   }
   
   
   protected static class CmdDelete implements Command {
      
      protected IconWidget w;
      
      public CmdDelete(IconWidget w){
         this.w = w;
      }
      
      public void execute(){
         this.w.delete();
      }
   }
}