package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TextBoxBase;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Lasts;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;

public class LastsWidget extends PrimitiveWidget {

   protected static final String PROPERTIES_TITLE = "<b>&nbsp;&nbsp;Last Primitive Properties</b>";
   
   protected static final String PROPERTIES_DEFAULT_DISTANCE    = "5";
   protected static final int    PROPERTIES_DURATION_BOX_LENGTH = 4;
   protected static final String PROPERTIES_DURATION_BOX_PREFIX = "At least&nbsp;&nbsp;";
   protected static final String PROPERTIES_DURATION_BOX_SUFFIX = "&nbsp;&nbsp;minutes";
   
   //--//
   
   protected HorizontalPanel durationBoxPanel;
   protected TextBox         durationPropertyBox;
   
   //--//
   
   public LastsWidget(LabeledIcon lblIco, Lasts primitive){
      super(lblIco, primitive);
      
      this.propTitle = new HTML( LastsWidget.PROPERTIES_TITLE );
      
      this.buildPropertiesContent();
      this.propDialog.setContent(this.propContent);
   }
   
   
   public LastsWidget(LabeledIcon                   lblIco, 
                      Lasts                         primitive,
                      Vector<ConnectorHandleWidget> connectors){
      this(lblIco, primitive);
      this.connectors = connectors;
   }

   
   @Override
   public IconWidget cloneWidget(){
      
      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
                                            this.lblIco.defaultImg               ,
                                            new HTML(this.lblIco.label.getHTML()),
                                            false);
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
      
      return (IconWidget) new LastsWidget(newIco                        ,
                                          (Lasts)this.primitive         ,
                                          connectorClone);
   }
   
   
   
   @Override
   protected void buildPropertiesContent(){

      HorizontalPanel titlePanel = new HorizontalPanel();

      titlePanel.add( this.propTitle );
   
      this.buildPropertiesDescription();
      this.buildDurationBoxPanel     ();
      this.buildPropertiesButtons    ();
      
      this.propContent = new VerticalPanel();
      
      this.propContent.add( titlePanel                );
      this.propContent.add( new HTML("<br>&nbsp;")    );
      this.propContent.add( this.propDescriptionPanel );
      this.propContent.add( new HTML("<br>&nbsp;")    );
      this.propContent.add( this.durationBoxPanel     );
      this.propContent.add( new HTML("<br>&nbsp;")    );
      this.propContent.add( this.buttonPanel          );
   }
   
   
   protected void buildPropertiesDescription(){
      this.propDescriptionPanel = new HorizontalPanel();
      this.propDescription      = new HTML("&quot;This scene lasts for X minutes&quot;");
      
      HorizontalPanel descContainer = new HorizontalPanel();
      
      descContainer.add( this.propDescription );
      
      this.propDescriptionPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.propDescriptionPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      this.propDescriptionPanel.add( descContainer );
   }
   
   
   protected void buildDurationBoxPanel(){
      this.durationBoxPanel    = new HorizontalPanel();
      
      this.durationPropertyBox = new TextBox();
      this.durationPropertyBox.setText         ( LastsWidget.PROPERTIES_DEFAULT_DISTANCE    );
      this.durationPropertyBox.setVisibleLength( LastsWidget.PROPERTIES_DURATION_BOX_LENGTH );
      this.durationPropertyBox.setTextAlignment( TextBoxBase.ALIGN_RIGHT                    );
      
      HorizontalPanel boxContainer = new HorizontalPanel();
      
      boxContainer.add( new HTML("&nbsp;&nbsp;")                             );
      boxContainer.add( new HTML(LastsWidget.PROPERTIES_DURATION_BOX_PREFIX) );
      boxContainer.add( this.durationPropertyBox                             );
      boxContainer.add( new HTML(LastsWidget.PROPERTIES_DURATION_BOX_SUFFIX) );
      
      this.durationBoxPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.durationBoxPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      this.durationBoxPanel.add(boxContainer);
   }
   
   
  protected void createPropertiesButtons(){
      
      this.saveButton = new Button( IconWidget.BUTTON_SAVE_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkDistanceProperty()){
               applyPropertiesChanges();
               propDialog.close();
            } else {
               Window.alert("You must specify a numeric duration in minutes!");
               durationPropertyBox.setText("");
            }
         }
      });
      
      this.cancelButton = new Button( IconWidget.BUTTON_CANCEL_TXT, new ClickHandler(){
         public void onClick(ClickEvent e){
            propDialog.close();
         }
      });
      
      this.applyButton = new Button( IconWidget.BUTTON_APPLY_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkDistanceProperty()){
               applyPropertiesChanges();
            } else {
               Window.alert("You must specify a numeric duration in minutes!");
               durationPropertyBox.setText("");
            }
         }
      });
   }
     
   
   protected boolean checkDistanceProperty(){
      boolean valid = true;
      
      String txt = this.durationPropertyBox.getText();
      
      if(null != txt){
         try {
            double num = Double.parseDouble(txt);
         } catch(NumberFormatException e){
            valid = false;
         }
      } else {
         valid = false;
      }
      return valid;
   }
}