package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Outside;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;

public class OutsideWidget extends PrimitiveWidget {

	protected static final String PROPERTIES_TITLE = "<b>Outside Primitive Properties</b>";
	
	//--//

	public OutsideWidget(LabeledIcon lblIco, Outside primitive){
		super(lblIco, primitive);
		
		this.propTitle = new HTML( OutsideWidget.PROPERTIES_TITLE );
		
		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}
	
	
	public OutsideWidget(LabeledIcon                   lblIco, 
	                     Outside                       primitive,
	                     Vector<ConnectorHandleWidget> connectors){
      this(lblIco, primitive);
      this.connectors = connectors;
   }
	
	
	@Override
   public IconWidget cloneWidget(){
      
      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
                                            this.lblIco.defaultImg               ,
                                            new HTML(this.lblIco.label.getHTML()),
                                            false);
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
      
      return (IconWidget) new OutsideWidget(newIco        ,
                                            (Outside)this.primitive,
                                            connectorClone);
   }
	
	
	/**
    * Target ok if: 
    * 1) This widget is not connected to anything else and target is an Actor
    * 2) This widget is not connected to a place and target is an Actor
    * 3) This widget is connected to a place and target is a non-place Actor 
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof ActorWidget){
         if(0 == this.connectors.size()){
            targetOk = true;
         } else {
            boolean hasPlace = false;
            
            for(ConnectorHandleWidget connector : this.connectors){
               if(null != connector.actorWidget && connector.actorWidget instanceof PlaceWidget){
                  hasPlace = true;
                  break;
               }
            }
            if(!hasPlace){
               targetOk = true;
            } else {
               if(!(w instanceof PlaceWidget)){
                  targetOk = true;
               }
            }
         }
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
   
   
   @Override
   public String getPrimitiveDescription(){
      String         desc       = new String();
      String         placeName  = null;
      Vector<String> actorNames = new Vector<String>();
      
      for(ConnectorHandleWidget connector : this.connectors){
         if(null != connector.actorWidget){
            if(connector.actorWidget instanceof PlaceWidget){
               placeName = connector.actorWidget.actor.getName();
            } else {
               actorNames.add(connector.actorWidget.actor.getName());
            }
         }
      }
      
      if(null != placeName && 0 < actorNames.size()){
         desc = "\"";
         if(1 == actorNames.size()){
            desc += actorNames.get(0) + " is outside ";
         } else if(2 == actorNames.size()){
            desc += actorNames.get(0) + " and " + actorNames.get(1) + " are outside ";
         } else {
            for(int i = 0; i < actorNames.size() - 1; i++){
               desc += actorNames.get(i) + ", ";
            }
            desc += " and " + actorNames.get(actorNames.size() - 1) + " are outside ";
         }
         desc += placeName + ".\"";         
      }
      return desc;
   }
   
   
   /**
    * For now, valid if connected to one place and at least one other non-place actor.
    */
   public boolean isValid(){
      boolean valid = false;
      
      // Check for one place
      int places = 0;
      for(ConnectorHandleWidget connector : this.connectors){
         if(connector.actorWidget instanceof PlaceWidget){
            places++;
         }
      }
      // Then check for at least one non-place actor
      if(places == 1 && this.connectors.size() > 1){
         valid = true;
      }
      return valid;
   }
   
   
   //--//
   
   
   protected void buildPropertiesDescription(){
      this.propDescriptionPanel = new HorizontalPanel();
      this.propDescription      = new HTML("&quot;X is outside Y&quot;");
      
      HorizontalPanel descContainer = new HorizontalPanel();
      
      descContainer.add( this.propDescription );
      
      this.propDescriptionPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.propDescriptionPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      this.propDescriptionPanel.add( descContainer );
   }
}