package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Vector;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TextBoxBase;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Actor;
import edu.washington.cs.rfid.cascadia.scenic.client.data.PersonGrp;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ScenicToolbar;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorLibrary;

public class PersonGrpWidget extends ActorWidget {

   protected static final String PROPERTIES_TITLE           = "<b>&nbsp;&nbsp;Person Group Properties</b>";
   protected static final String PROPERTIES_NAME_PREFIX     = "Select person group:&nbsp;&nbsp;";
   
   protected static final String PROPERTIES_DEFAULT_SIZE    = "4";
   protected static final int    PROPERTIES_SIZE_BOX_LENGTH = 4;
   protected static final String PROPERTIES_SIZE_BOX_PREFIX = "Group size:&nbsp;&nbsp;";
   
   //--//

   protected HorizontalPanel sizeBoxPanel;
   protected TextBox         sizePropertyBox;
   
   //--//
   
   public PersonGrpWidget(LabeledIcon lblIco, PersonGrp personGrp){
      super(lblIco, personGrp);
      
      this.propTitle     = new HTML( PersonGrpWidget.PROPERTIES_TITLE       );
      this.nameBoxPrefix = new HTML( PersonGrpWidget.PROPERTIES_NAME_PREFIX );
      
      this.buildPropertiesContent();
      this.propDialog.setContent(this.propContent);
   }
   
   
   public PersonGrpWidget(LabeledIcon                   lblIco, 
                          PersonGrp                     personGrp,
                          Vector<ConnectorHandleWidget> connectors){
      this(lblIco, personGrp);
      
      this.connectors = connectors;
   }
   
   
   public int getSize(){
      if(this.checkSizeProperty()){
         String txt = this.sizePropertyBox.getText();
         return Integer.parseInt(txt);
      } else {
         return 1;
      }
   }
   
   
   @Override
   public IconWidget cloneWidget(){

      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  ,
                                            this.lblIco.defaultImg               , 
                                            new HTML(this.lblIco.label.getHTML()), 
                                            false );
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
         
      return (IconWidget) new PersonGrpWidget(newIco               , 
                                              (PersonGrp)this.actor,
                                              connectorClone);
   }
   
   
   @Override
   public IconWidget nextWidget() {      
      ActorWidget next =  ActorFactory.createPersonGrpWidget();
      
      next.setLabelVisible(false);

      return (IconWidget) next;
   }
   
   
   @Override
   protected void buildPropertiesContent(){

      HorizontalPanel titlePanel = new HorizontalPanel();

      titlePanel.add( this.propTitle );

      ArrayList<String> names = this.getPropertiesNames();
      names                   = this.getPropertiesNames();
      
      Collections.sort(names);
      
      this.buildNameBoxPanel(names);
      this.buildSizeBoxPanel();
      this.buildPropertiesButtons();
      
      this.propContent = new VerticalPanel();
      
      HTML spacer1 = new HTML("<br>&nbsp;");
      spacer1.setStyleName("spacer");
      HTML spacer2 = new HTML("<br>&nbsp;");
      spacer2.setStyleName("spacer");
      HTML spacer3 = new HTML("<br>&nbsp;");
      spacer3.setStyleName("spacer");
      HTML spacer4 = new HTML("<br>&nbsp;");
      spacer4.setStyleName("spacer");
      
      this.propContent.add( titlePanel             );
      this.propContent.add( spacer1 );
      this.propContent.add( nameBoxPanel           );
      this.propContent.add( spacer2 );
      this.propContent.add( this.iconPanel         );
      this.propContent.add( spacer3 );
      this.propContent.add( this.sizeBoxPanel      );
      this.propContent.add( spacer4 );
      this.propContent.add( this.buttonPanel       );
   }
   
   
   protected ArrayList<String> getPropertiesNames(){
      
      ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
      Vector<PersonGrp> people = lib.getPersonGrps();
      ArrayList<String> names  = new ArrayList<String>();
      
      for(int i = 0; i < people.size(); i++){
         names.add( people.get(i).getName() );
      }
      return names;
   }
   
   protected void buildSizeBoxPanel(){
      this.sizeBoxPanel    = new HorizontalPanel();
      
      this.sizePropertyBox = new TextBox();
      this.sizePropertyBox.setText         ( PersonGrpWidget.PROPERTIES_DEFAULT_SIZE    );
      this.sizePropertyBox.setVisibleLength( PersonGrpWidget.PROPERTIES_SIZE_BOX_LENGTH );
      this.sizePropertyBox.setTextAlignment( TextBoxBase.ALIGN_RIGHT                    );
      
      HorizontalPanel boxContainer = new HorizontalPanel();
      
      boxContainer.add( new HTML("&nbsp;&nbsp;")                             );
      boxContainer.add( new HTML(PersonGrpWidget.PROPERTIES_SIZE_BOX_PREFIX) );
      boxContainer.add( this.sizePropertyBox                                 );
      
      this.sizeBoxPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
      this.sizeBoxPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
      this.sizeBoxPanel.add(boxContainer);
   }
   
   
   protected void createPropertiesButtons(){
      
      this.saveButton = new Button( IconWidget.BUTTON_SAVE_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkSizeProperty()){
               applyPropertiesChanges();
               propDialog.close();
            } else {
               Window.alert("You must specify a group size no larger than 10");
               sizePropertyBox.setText("");
            }
         }
      });
      
      this.cancelButton = new Button( IconWidget.BUTTON_CANCEL_TXT, new ClickHandler(){
         public void onClick(ClickEvent e){
            propDialog.close();
         }
      });
      
      this.applyButton = new Button( IconWidget.BUTTON_APPLY_TXT , new ClickHandler(){
         public void onClick(ClickEvent e){
            if(checkSizeProperty()){
               applyPropertiesChanges();
            } else {
               Window.alert("You must specify a group size no larger than 10");
               sizePropertyBox.setText("");
            }
         }
      });
   }
   
   
   @Override
   protected Actor findActorByName(String name){
      ActorLibrary lib = ScenicToolbar.getActorLibrary();
      return lib.getPersonGrp(name);
   }
   
   
   protected boolean checkSizeProperty(){
      boolean valid = true;
      
      String txt = this.sizePropertyBox.getText();
      
      if(null != txt){
         try {
            double num = Integer.parseInt(txt);
            
            if(num > 10 || num <= 0){
               valid = false;
            }
         } catch(NumberFormatException e){
            valid = false;
         }
      } else {
         valid = false;
      }
      return valid;
   }
   
   
   /**
    * Target ok if it is a primitive.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof PrimitiveWidget){
         targetOk = true;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
}