package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Far;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Inside;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Lasts;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Near;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Outside;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Primitive;
import edu.washington.cs.rfid.cascadia.scenic.client.data.With;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Without;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ContextMenu;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.PrimitiveFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.util.JavaScriptHelper;

public class PrimitiveWidget extends IconWidget {

	protected static final String PROPERTIES_TITLE = "<b>Primitive Properties</b>";
	
	//--//
	
	protected Primitive        primitive;
	protected LabeledIcon      lblIco;
	
	protected HorizontalPanel  propDescriptionPanel;
	protected HTML             propDescription;
	
	//--//
	
	public PrimitiveWidget(LabeledIcon      lblIco   , 
	                       Primitive        primitive){
		super(lblIco);
		
		this.primitive = primitive;
		this.lblIco    = lblIco;
		this.propTitle = new HTML( PrimitiveWidget.PROPERTIES_TITLE );
		
		this.iconMenu  = new ContextMenu();
		JavaScriptHelper.disableContextMenu(iconMenu.getElement(), true);
		this.iconMenu.addItem( "Make Connection" , new CmdConnect(this)    );
		this.iconMenu.addItem( "Delete", new CmdDelete(this)     );
//		this.iconMenu.addItem( "Properties"      , new CmdProperties(this) );
		this.iconMenu.hide();
		
		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}
	
	
	public PrimitiveWidget(LabeledIcon                   lblIco   , 
                          Primitive                     primitive,
                          Vector<ConnectorHandleWidget> connectors){
	   this(lblIco,primitive);
	   this.connectors = connectors;
	}
	
	
	public Primitive getPrimitive(){
		return this.primitive;
	}
	
	
	@Override
	public IconWidget cloneWidget(){
		
		LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
		                                      this.lblIco.defaultImg               ,
		                                      new HTML(this.lblIco.label.getHTML()),
		                                      true);
		
		Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
		
		return (IconWidget) new PrimitiveWidget(newIco        ,
		                                        this.primitive,
		                                        connectorClone);
	}
	
	
	@Override
	public IconWidget nextWidget() {
		
		PrimitiveWidget next = null;

		if(this.primitive instanceof With){
			next = PrimitiveFactory.createWithWidget();
		} else if(this.primitive instanceof Without) {
			next = PrimitiveFactory.createWithoutWidget();
		} else if(this.primitive instanceof Inside) {
			next = PrimitiveFactory.createInsideWidget();
		} else if(this.primitive instanceof Outside) {
			next = PrimitiveFactory.createOutsideWidget();
		} else if(this.primitive instanceof Near) {
			next = PrimitiveFactory.createNearWidget();
		} else if(this.primitive instanceof Far){
			next = PrimitiveFactory.createFarWidget();
		} else if(this.primitive instanceof Lasts){
			next = PrimitiveFactory.createLastsWidget();
		} else {
			throw new IllegalStateException("Unhandled Widget class " + widget.getClass().getName());
		}
		next.setLabelVisible(false);
	   return (IconWidget) next;
	}

	
	@Override
	public void wigdetPickedUp(){
		this.pickedUp = true;
	}
	
	public void setLabelVisible(boolean visible){
		this.lblIco.setLabelVisible(visible);
	}
	
	public boolean getLabelVisible(){
      return this.lblIco.getLabelVisible();
   }
	
	public int getRelativeXConnectorAnchor(){
	   int anchorX = 0;
      if(this.getLabelVisible()){
         anchorX = this.lblIco.label.getOffsetWidth() / 2;
      } else {
//         anchorX = this.lblIco.img.getWidth() / 2;
         anchorX = this.lblIco.maxWidthImg / 2;
      }
      return anchorX;
   }
   
   
   public int getRelativeYConnectorAnchor(){
      int anchorY = 0;
      if(this.getLabelVisible()){
         anchorY = this.lblIco.img.getHeight() / 2;
      } else {
         anchorY = this.lblIco.maxHeightImg / 2;
      }
      return anchorY;
   }
   
   
   public int getWidth(){
      return this.lblIco.img.getWidth();
   }
   
   
   public int getHeight(){
      return this.lblIco.img.getHeight();
   }
   
   
   public int getIconMaxWidth(){
      return this.lblIco.maxWidthImg;
   }
   
   
   public int getIconMaxHeight(){
      return this.lblIco.maxHeightImg;
   }
   
   
   public String getPrimitiveDescription(){
      return "";
   }
   
   
   public boolean isValid(){
      return true;
   }
   
   
	//--//
	
	
	@Override
	protected void buildPropertiesContent(){

		HorizontalPanel titlePanel = new HorizontalPanel();

		titlePanel.add( this.propTitle );
	
		this.buildPropertiesDescription();
		this.buildPropertiesButtons    ();
		
		this.propContent = new VerticalPanel();
		
		this.propContent.add( titlePanel                );
		this.propContent.add( new HTML("<br>&nbsp;")    );
		this.propContent.add( this.propDescriptionPanel );
		this.propContent.add( new HTML("<br>&nbsp;")    );
		this.propContent.add( this.buttonPanel          );
	}
	
	
	protected void buildPropertiesDescription(){
		this.propDescriptionPanel = new HorizontalPanel();
		this.propDescription      = new HTML("An RFID Event Primitive");
		
		this.propDescriptionPanel.add( this.propDescription );
	}

	
	@Override
	protected void applyPropertiesChanges(){
		
	}	
}
