package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.ArrayList;
import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Actor;
import edu.washington.cs.rfid.cascadia.scenic.client.data.Thing;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.ScenicToolbar;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorFactory;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.factory.ActorLibrary;

public class ThingWidget extends ActorWidget {

	protected static final String PROPERTIES_TITLE       = "<b>&nbsp;&nbsp;Thing Properties</b>";
	protected static final String PROPERTIES_NAME_PREFIX = "Select thing:&nbsp;&nbsp;";
	
	//--//
	
	public ThingWidget(LabeledIcon lblIco, Thing thing){
		super(lblIco, thing);
		
		this.propTitle     = new HTML( ThingWidget.PROPERTIES_TITLE       );
		this.nameBoxPrefix = new HTML( ThingWidget.PROPERTIES_NAME_PREFIX );

		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}	
	
	public ThingWidget(LabeledIcon                   lblIco, 
	                   Thing                         thing, 
	                   Vector<ConnectorHandleWidget> connectors){
      this(lblIco, thing);
      this.connectors = connectors;
   }  
	
	
	@Override
   public IconWidget cloneWidget(){

      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  ,
                                            this.lblIco.defaultImg               , 
                                            new HTML(this.lblIco.label.getHTML()), 
                                            false );
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
         
      return (IconWidget) new ThingWidget(newIco           , 
                                          (Thing)this.actor, 
                                          connectorClone);
   }
	
   @Override
   public IconWidget nextWidget() {      
      ActorWidget next =  ActorFactory.createThingWidget();
      
      next.setLabelVisible(false);

      return (IconWidget) next;
   }

	

	protected ArrayList<String> getPropertiesNames(){
		
		ActorLibrary      lib    = ScenicToolbar.getActorLibrary();
		Vector<Thing>     things = lib.getThings();
		ArrayList<String> names  = new ArrayList<String>();

		for(int i = 0; i < things.size(); i++){
			names.add( things.get(i).getName() );
		}
		return names;
	}

	
	@Override
	protected Actor findActorByName(String name){
		
		Thing         thing  = null;
		ActorLibrary  lib    = ScenicToolbar.getActorLibrary();
		Vector<Thing> things = lib.getThings();
			
		for(int i = 0; i < things.size(); i++){
			
			if(things.get(i).getName().equals(name)){
				this.lblIco.changeImage( new Image(things.get(i).getImgUrl()) );
				thing = things.get(i);
				break;
			}
		}
		return thing;
	}
	
	
	/**
    * Target ok if it is a primitive.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(w instanceof PrimitiveWidget){
         targetOk = true;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
}