package edu.washington.cs.rfid.cascadia.scenic.client.ui.widget;

import java.util.Vector;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;

import edu.washington.cs.rfid.cascadia.scenic.client.data.Without;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.LabeledIcon;
import edu.washington.cs.rfid.cascadia.scenic.client.ui.PropertiesDialog;

public class WithoutWidget extends PrimitiveWidget {

	protected static final String PROPERTIES_TITLE = "<b>&nbsp;&nbsp;Without Primitive Properties</b>";
	
	//--//
	
	public WithoutWidget(LabeledIcon lblIco, Without primitive){
		super(lblIco, primitive);
		
		this.propTitle = new HTML( WithoutWidget.PROPERTIES_TITLE );
		
		this.buildPropertiesContent();
		this.propDialog.setContent(this.propContent);
	}
	
	
	public WithoutWidget(LabeledIcon                   lblIco, 
	                     Without                       primitive, 
	                     Vector<ConnectorHandleWidget> connectors){
      this(lblIco, primitive);
      this.connectors = connectors;
   }
	
	
	@Override
   public IconWidget cloneWidget(){
      
      LabeledIcon newIco = new LabeledIcon( new Image(this.lblIco.img.getUrl())  , 
                                            this.lblIco.defaultImg               ,
                                            new HTML(this.lblIco.label.getHTML()),
                                            false);
      
      Vector<ConnectorHandleWidget> connectorClone = (Vector<ConnectorHandleWidget>) this.connectors.clone();
      
      return (IconWidget) new WithoutWidget(newIco        ,
                                            (Without)this.primitive,
                                            connectorClone);
   }
	
	
	/**
    * Target ok if it is not a place.
    */
   public boolean checkPotentialTarget(IconWidget w){
      boolean targetOk = false;
      
      if(!(w instanceof PlaceWidget) && this.connectors.size() < 2){
         targetOk = true;
      }
      return super.checkPotentialTarget(w) && targetOk;
   }
	
   
   @Override
   public String getPrimitiveDescription(){
      String         desc       = new String();
      String         placeName  = null;
      Vector<String> actorNames = new Vector<String>();
      
      for(ConnectorHandleWidget connector : this.connectors){
         if(null != connector.actorWidget){
            actorNames.add(connector.actorWidget.actor.getName());
         }
      }
      
      if(2 == this.connectors.size()){
         ActorWidget aw1 = this.connectors.get(0).actorWidget;
         ActorWidget aw2 = this.connectors.get(1).actorWidget;
         
         String firstName = null;
         String otherName = null;
         
         if(aw2 instanceof PersonWidget && !(aw1 instanceof PersonWidget)){
            firstName = aw2.actor.getName();
            otherName = aw1.actor.getName();
         } else {
            firstName = aw1.actor.getName();
            otherName = aw2.actor.getName();
         }
         desc = "\"" + firstName + " is without " + otherName + "\"";
      }
      return desc;
   }
   
   
   /**
    * For now, valid if connected to exactly two non-place actor.
    */
   public boolean isValid(){
      boolean valid = false;
      
      if(this.connectors.size() == 2){
         valid = true;
      }
      return valid;
   }
   
   
   //--//
   
   
	protected void buildPropertiesDescription(){
		this.propDescriptionPanel = new HorizontalPanel();
		this.propDescription      = new HTML("&quot;X is not with, next to, or holding Y&quot;");
		
		HorizontalPanel descContainer = new HorizontalPanel();
		
		descContainer.add( this.propDescription );
		
		this.propDescriptionPanel.setWidth( PropertiesDialog.DIALOG_CONTENT_WIDTH + "px" );
		this.propDescriptionPanel.setHorizontalAlignment( HasHorizontalAlignment.ALIGN_CENTER );
		this.propDescriptionPanel.add( descContainer );
	}
}