package edu.washington.cs.rfid.cascadia.scenic.client.util;

import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Command to rescale image if the onload listener fails to fire (e.g. in IE).
 */
public class DeferredRescaleCheckCommand implements Command {
   
   public Image           img;
   public int             maxWidth;
   public int             maxHeight;
   public boolean         imgRescaled;
   
   public HorizontalPanel hp;
   public int             hpIndex;
   
   public VerticalPanel   vp;
   public int             vpIndex;

   public AbsolutePanel   ap;
   public int             x;
   public int             y;
   
   //--//
   
   public DeferredRescaleCheckCommand(Image img, int maxWidth, int maxHeight){
      this.img         = img;
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.imgRescaled = false;
      this.hp          = null;
      this.hpIndex     = -1;
      this.vp          = null;
      this.vpIndex     = -1;
   }
   
   
   public DeferredRescaleCheckCommand(Image img, int maxWidth, int maxHeight, HorizontalPanel hp, int hpIndex){
      this.img         = img;
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.imgRescaled = false;
      this.hp          = hp;
      this.hpIndex     = hpIndex;
      this.vp          = null;
      this.vpIndex     = -1;
   }
   
   
   public DeferredRescaleCheckCommand(Image img, int maxWidth, int maxHeight, VerticalPanel vp, int vpIndex){
      this.img         = img;
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.imgRescaled = false;
      this.hp          = null;
      this.hpIndex     = -1;
      this.vp          = vp;
      this.vpIndex     = vpIndex;
   }
   
   
   public DeferredRescaleCheckCommand(Image img, int maxWidth, int maxHeight, AbsolutePanel ap, int x, int y){
      this.img         = img;
      this.maxWidth    = maxWidth;
      this.maxHeight   = maxHeight;
      this.imgRescaled = false;
      this.hp          = null;
      this.hpIndex     = -1;
      this.vp          = null;
      this.vpIndex     = -1;
      this.ap          = ap;
      this.x           = x;
      this.y           = y;
   }
   
   
   public void execute(){
      
      if(!this.imgRescaled){
         
         // This is an ugly hack to get around the following problem:
         // The code below throws an IllegalArgumentException
         // because it says this.img is not a child of this.ap
         // It's not clear why it says this, but the if clause solves the
         // problem for now.
         // TODO: figure out a better solution to this.
         if(null != this.ap){
            Widget widgetContained = this.ap.getWidget(0);
            if(widgetContained instanceof Image){
               this.img = (Image)widgetContained;
            }
         }
         
         double curWidth  = (double) this.img.getWidth();
         double curHeight = (double) this.img.getHeight();

         if((curWidth < this.maxWidth && curWidth < 25) || 
               (curHeight < this.maxHeight && curHeight < 25)){

            // If the resizing is not done, then will the rescaler load resize in firefox for ProfilePanel?
            //this.img.setSize( ((int)this.maxWidth) + "px", ((int) this.maxHeight) + "px");
            
            // Now insert into a HorizontalPanel or VerticalPanel if requested - this triggers listener in Firefox!
            if(null != this.hp){
               this.hp.insert(this.img, this.hpIndex);
            } else if(null != this.vp){
               this.vp.insert(this.img, this.vpIndex);
            } else if(null != this.ap){
               this.ap.setWidgetPosition(this.img, 0, 0);
               this.ap.setWidth(((int)curWidth) + "px");
               this.ap.setHeight(((int) curHeight) + "px");
            }
            return;
         }
         
         // Scale down by maximum dimension
         if(curHeight >= curWidth && curHeight > (double)this.maxHeight){
            
            double scaleFactor  = ((double)this.maxHeight) / curHeight;
            curHeight           = (double)this.maxHeight;
            curWidth           *= scaleFactor;
            
         } else if(curWidth > curHeight && curWidth > (double)this.maxWidth){
            
            double scaleFactor = ((double)this.maxWidth) / curWidth;
            curWidth           = (double) this.maxWidth;
            curHeight         *= scaleFactor;
         }
         this.img.setSize( ((int)curWidth) + "px", ((int) curHeight) + "px");
         
         // Now insert into a HorizontalPanel or VerticalPanel if requested
         if(null != this.hp){
            this.hp.insert(this.img, this.hpIndex);
         } else if(null != this.vp){
            this.vp.insert(this.img, this.vpIndex);
         }  else if(null != this.ap){
            this.ap.remove(0);
            this.ap.add(img, 0, 0);
            this.ap.setWidth(((int)curWidth) + "px");
            this.ap.setHeight(((int) curHeight) + "px");
         }
         this.imgRescaled = true;
      }
   }
}