package edu.washington.cs.rfid.cascadia.timeline.client.popups;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class ScenicEventBubble extends PopupPanel {
   
   protected static final String STYLE_PROPERTIES_DIALOG  = "propertiesDialog";
   protected static final String STYLE_PROPERTIES_CONTENT = "propertiesContent";
   protected static final String STYLE_CLOSE_X            = "propertiesCloseX";
   protected static final String STYLE_TIMELINE_POPUP     = "timeline-popup";
   
   protected static final String IMG_DIALOG_POINTER_LEFT  = "../images/dialog-pointer-left.png";
   protected static final String IMG_DIALOG_POINTER_RIGHT = "../images/dialog-pointer-right.png";
   protected static final String IMG_DIALOG_CLOSE_X       = "../images/dialog-close-x.png";
   
   protected static final int    WIDTH_POSITIONING_PANEL  = 600;
   protected static final int    HEIGHT_POSITIONING_PANEL = 600;
   protected static final int    WIDTH_EVENT_MARKER       = 8;
   protected static final int    HEIGHT_EVENT_MARKER      = 10;
   protected static final int    WIDTH_POINTER            = 30;
   protected static final int    HEIGHT_POINTER           = 23;
      
   //--//
   
   protected AbsolutePanel   positioningPanel;
   protected VerticalPanel   contentPanel;
   protected VerticalPanel   pointerPanel;
   protected Widget          content;
   
   protected Image           closeXImg;
   protected Image           pointerLeftImg;
   protected Image           pointerRightImg;

   protected PushButton      closeXButton;
   
   //--//
   
   public ScenicEventBubble(){
      super(true, false);

      this.positioningPanel = new AbsolutePanel();
      this.positioningPanel.setStyleName(ScenicEventBubble.STYLE_PROPERTIES_DIALOG);
      this.positioningPanel.setWidth(ScenicEventBubble.WIDTH_POSITIONING_PANEL  + "px");
      this.positioningPanel.setHeight(ScenicEventBubble.HEIGHT_POSITIONING_PANEL + "px");
      this.positioningPanel.setStyleName(ScenicEventBubble.STYLE_TIMELINE_POPUP);
      
      this.contentPanel     = new VerticalPanel();
      this.contentPanel.setStyleName(ScenicEventBubble.STYLE_PROPERTIES_CONTENT);
      
      this.closeXImg       = new Image(ScenicEventBubble.IMG_DIALOG_CLOSE_X);
      this.closeXButton = new PushButton( this.closeXImg, new ClickHandler(){
         public void onClick(ClickEvent e){
            hide();
         }
      });
      this.closeXButton.setStyleName(ScenicEventBubble.STYLE_CLOSE_X);
//    this.contentPanel.add(this.closeXButton);
//    this.contentPanel.setCellHorizontalAlignment(this.closeXButton, HorizontalPanel.ALIGN_RIGHT);
      
      this.pointerPanel = new VerticalPanel();
      this.pointerPanel.setWidth(ScenicEventBubble.WIDTH_POINTER + "px");
      this.pointerPanel.setHeight(ScenicEventBubble.HEIGHT_POINTER + "px");
      
      this.pointerLeftImg = new Image(ScenicEventBubble.IMG_DIALOG_POINTER_LEFT);
      this.pointerLeftImg.setPixelSize(ScenicEventBubble.WIDTH_POINTER, 
                                       ScenicEventBubble.HEIGHT_POINTER);
      
      this.pointerRightImg = new Image(ScenicEventBubble.IMG_DIALOG_POINTER_RIGHT);
      this.pointerRightImg.setPixelSize(ScenicEventBubble.WIDTH_POINTER, 
                                        ScenicEventBubble.HEIGHT_POINTER);
      
      this.pointerPanel.add(this.pointerLeftImg);
      
      this.content = null;
      
      this.positioningPanel.add(this.contentPanel, ScenicEventBubble.WIDTH_POINTER, 0);
      this.positioningPanel.add(this.pointerPanel, 0, 0);
      
      this.setWidget(this.positioningPanel);
      this.setStyleName(ScenicEventBubble.STYLE_TIMELINE_POPUP);
   }
   
   
   // TODO: Take a ScenicThumbNail and get the size from that - until then just just default width
   /**
    * Sets the content panel for this <code>PropertiesDialog</code>.
    * 
    * @param title
    * @param content
    */
   public void setContent(VerticalPanel content){
      
      if(null != this.content){
         this.contentPanel.remove(this.content);
      }
      this.content = content;
      
      this.contentPanel.insert( this.content,0);
   }
   
   
   /** 
    * Positions the <code>PropertiesDialog</code> relative to the given coordinates.
    * This includes choice of left or right pointer as well as the vertical and horizontal
    * positioning of the pointer and the content panel.
    * 
    * @param x
    * @param y
    */
   public void positionRelativeTo(int x, int y){

      // Horizontal
      int panelLeft   = x + ScenicEventBubble.WIDTH_EVENT_MARKER;
      int pointerLeft = 0;
      int dialogLeft  = ScenicEventBubble.WIDTH_POINTER - 2;
      
      
      // Vertical
      int panelTop   = y - (40 + ScenicEventBubble.HEIGHT_POINTER/2);
      int pointerTop = y - panelTop - (ScenicEventBubble.HEIGHT_EVENT_MARKER / 2) 
                                    - (ScenicEventBubble.HEIGHT_POINTER / 2);
      int dialogTop  = 0;

//      this.pointerPanel.setSize("30px","23px");
      this.pointerPanel.setSize(ScenicEventBubble.WIDTH_POINTER  + "px", 
                                ScenicEventBubble.HEIGHT_POINTER + "px");
      this.pointerPanel.remove(0);
      this.pointerPanel.add(this.pointerLeftImg);

      this.positioningPanel.setWidgetPosition(this.pointerPanel,pointerLeft,pointerTop);
      this.positioningPanel.setWidgetPosition(this.contentPanel,dialogLeft ,dialogTop);
      
      this.setPopupPosition(panelLeft, panelTop);
   
      this.show();
   }
   
   
   public void close(){
      this.hide();
   }
}
