package edu.washington.cs.rfid.cascadia.timeline.server;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import edu.washington.cs.rfid.cascadia.scenic.client.data.EventTemplate;
import edu.washington.cs.rfid.cascadia.timeline.client.GreetingService;
import edu.washington.cs.rfid.cascadia.timeline.client.event.ScenicEventResultBundle;
import edu.washington.cs.rfid.cascadia.timeline.server.data.AliceHelmetResultLibrary;
import edu.washington.cs.rfid.cascadia.timeline.server.data.AliceOverSpecifiedMeetingLibrary;
import edu.washington.cs.rfid.cascadia.timeline.server.data.JamesAliceStudyResultLibrary;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class GreetingServiceImpl extends RemoteServiceServlet implements
      GreetingService {

   private TemplateManager templateMgr;
   private TaskManager     taskMgr;
   
   private int[] taskNumList;
   
   
   //--//
   
   /**
    * Reads in the templates from a file or from a database.
    */
   @Override
   public void init(ServletConfig config) throws ServletException {
      super.init(config);
    
      this.templateMgr        = new TemplateManager();
      this.taskMgr            = new TaskManager();
      
      this.taskNumList    = new int[5];
      this.taskNumList[0] = 0;
      this.taskNumList[1] = 1;
      this.taskNumList[2] = 2;
      this.taskNumList[3] = 0;
      this.taskNumList[4] = 0;
      
//      this.traceMgr.loadSyncAndStoreTraces();
      
      this.templateMgr.loadTemplates();
   }
   
   
   public String greetServer(String input) {
      String serverInfo = getServletContext().getServerInfo();
      String userAgent = getThreadLocalRequest().getHeader("User-Agent");
      return "Hello, " + input + "!<br><br>I am running " + serverInfo
            + ".<br><br>It looks like you are using:<br>" + userAgent;
   }
   
   
   public ScenicEventResultBundle getResultBundle(int taskNum){
      EventTemplate template = this.templateMgr.getTemplate(this.taskNumList[taskNum]);
      
      //ScenicEventResultBundle bundle = EventResultLibrary.getResultBundle(this.taskNumList[taskNum],template.xml);
      
      ScenicEventResultBundle bundle = null;
      if(this.taskNumList[taskNum] == 0){
         bundle = AliceHelmetResultLibrary.getResultBundle(this.taskNumList[taskNum],template.xml);
      } else if(this.taskNumList[taskNum] == 1){
         bundle = JamesAliceStudyResultLibrary.getResultBundle(this.taskNumList[taskNum],template.xml);
      } else if(this.taskNumList[taskNum] == 2){
         bundle = AliceOverSpecifiedMeetingLibrary.getResultBundle(this.taskNumList[taskNum],template.xml);
      }
      
      return bundle;
   }
}
