package edu.washington.cs.rfid.data;

import java.io.Serializable;


/**
 * The memory resident representation of antenna metadata.
 * 
 * @author Evan
 */
public class Ant implements Serializable {
	
	private static final long serialVersionUID = 1565078633L;

	//--//

	public int 	  aid;			// Antenna ID
	public String sid;			// Symbolic ID (ex: "above north Ubicomp lab door")
	public int 	  rid;			// Reader ID
	public int 	  num;			// Antenna number on the reader (1-4)
	public int	  floor;		// Floor number
	public double lat;			// Latitude
	public double lon;			// Longitude
	public double alt;          // Relative altitude in meters
	public int 	  assetNum;		// Asset number of the antenna

	//--//

	public Ant() {
		this(	-1,
				"",
				-1,
				-1,
				-1,
				-1,
				-1,
				-1,
				-1);
	}
	
	/**
	 * Creates a new <code>Antenna</code> object with the given parameters.
	 * 
	 * @param aid The antenna ID
	 * @param sid The symbolic ID (ex: "above north Ubicomp lab door")
	 * @param rid The reader ID
	 * @param num The antenna number on the reader (1-4)
	 * @param floor The floor number
	 * @param lat The latitude
	 * @param lon The longitude
	 * @param alt The altitude in meters
	 * @param assetNum The asset number of the antenna 
	 */
	public Ant(int    aid  , 
		       String sid  , 
		       int    rid  , 
		       int    num  , 
		       int    floor,
		       double lat  , 
		       double lon  , 
		       double alt  ,
		       int    assetNum){
		this.aid      = aid;
		this.sid      = sid;
		this.rid      = rid;
		this.num      = num;
		this.floor    = floor;
		this.lat      = lat;
		this.lon   	  = lon;
		this.alt      = alt;
		this.assetNum = assetNum;
	}
}