package edu.washington.cs.rfid.data;

import java.io.Serializable;

/**
 * The memory resident representation of an entity.
 * 
 * @author Evan
 */
public class Entity implements Serializable {

	private static final long serialVersionUID = 98675343424L;
	
	//--//
	
	private static final String DEFAULT_IMG_URL = "http://rfid.cs.washington.edu/dbicons/unknown.png";

	//--//
	
	public int 	   eid;			// The object ID
	public String  name;		// A description of this object
	public String  owner;		// The csnetid of the object's owner
	public int 	   typeId;		// The ID of the object's type
	public String  imgUrl;		// URL for an image of the object
	
	//--//

	/**
	 * Creates an <code>ObjectMetadata</code> with default values.
	 */
	public Entity(){
		this.eid    = -1;
		this.name   = null;
		this.owner  = null;
		this.typeId = -1;
		this.imgUrl = Entity.DEFAULT_IMG_URL;
	}
	
	
	/**
	 * Creates an <code>ObjectMetadata</code> with the specified parameters.
	 * 
	 * @param eid The object's ID
	 * @param name A description of this object
	 * @param owner The csnetid of the object's owner
	 * @param typeId The ID of the object's type
	 */
	public Entity(int eid, String name, String owner, int typeId){
		this.eid    = eid;
		this.name   = name;
		this.owner  = owner;
		this.typeId = typeId;
		this.imgUrl = Entity.DEFAULT_IMG_URL;
	}
	
	
	/**
	 * Creates an <code>ObjectMetadata</code> with the specified parameters.
	 * 
	 * @param eid The object's ID
	 * @param name A description of this object
	 * @param owner The csnetid of the object's owner
	 * @param typeId The ID of the object's type
	 * @param imgUrl URL for an image of the object
	 */
	public Entity(int eid, String name, String owner, int typeId, String imgUrl){
		this.eid    = eid;
		this.name   = name;
		this.owner  = owner;
		this.typeId = typeId;
		this.imgUrl = imgUrl;
	}
	
}