package edu.washington.cs.rfid.data;

import java.io.Serializable;


/**
 * The memory-resident representation of a tag read event.
 * 
 * @author Evan
 */
public class Tre implements Serializable {

	private static final long serialVersionUID = 1111111111111L;

	//--//
	
	public long tagID;			// The tag ID (i.e. hashed EPC)
	public int  antennaID;		// The antenna ID
	public long startTime;    	// The start timestamp for this TRE
	public long endTime;      	// The end timestamp for this TRE
    public int  numReads;     	// The number of reads compressed into this CompressedTRE
	public int  totalRSSI;		// The total RSSI over all TREs for this CompressedTRE
	
	//--//

	/**
	 * Creates a new <code>TRE</code> object with the specified parameters.
	 * 
	 * @param tagID The ID of the tag
	 * @param antennaID The ID of the antenna reading the tag
	 * @param startTime The start timestamp
	 * @param endTime The end timestamp
	 * @param numReads The number of reads
	 * @param totalRSSI The total RSSI
	 */
	public Tre(long tagID     , 
			   int  antennaID , 
			   long startTime , 
			   long endTime   , 
			   int  numReads  ,
			   int  totalRSSI){
		this.tagID     = tagID;
		this.antennaID = antennaID;
		this.startTime = startTime;
		this.endTime   = endTime;
		this.numReads  = numReads;
		this.totalRSSI = totalRSSI;
	}
	
	
	/**
	 * Returns a <code>String</code> representing this <code>CompressedTRE</code>.
	 * 
	 * @return A <code>String</code> representing this <code>CompressedTRE</code>
	 */
	public String toString(){
		return "( " + this.tagID + " , " + this.antennaID + " , " + this.startTime + 
		       " , " + this.endTime + " , " + this.numReads + " , " + this.totalRSSI + " )";
	}	
}