package edu.washington.cs.rfid.map.client;

import java.util.ArrayList;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import edu.washington.cs.rfid.map.client.data.LatLon;
import edu.washington.cs.rfid.map.client.data.Particle;
import edu.washington.cs.rfid.map.client.data.ParticleCloud;
import edu.washington.cs.rfid.map.client.data.TagReading;
import edu.washington.cs.rfid.map.client.data.TrackedEntity;
import edu.washington.cs.rfid.map.client.data.cache.DefaultMapImages;
import edu.washington.cs.rfid.map.client.ui.MercatorMapWidget;
import edu.washington.cs.rfid.map.client.ui.overlay.MarkerFactory;
import edu.washington.cs.rfid.map.client.ui.overlay.OverlayFilter;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 * 
 * This has been temporarily disabled in the interest of compiling a jar with
 * no entry point.
 */
public class EcosystemMap { //implements EntryPoint {

   public TrackedEntity     evan;
   public ParticleCloud     cloud;
   public int               time;
   public MercatorMapWidget map;
   public MarkerFactory     factory;
   
   /**
    * This is the entry point method.
    */
   public void onModuleLoad() {
      
      VerticalPanel vp = new VerticalPanel();
      
      this.map = new MercatorMapWidget("500px","500px",true);
      
      OverlayFilter filter = OverlayFilter.createDefaultFilter();
      filter.setAntVisible(false);
      filter.setRdrVisible(false);
      filter.setParticlesVisible(true);
      filter.setEntitiesVisible(true);
      filter.setConnectivityGraphVisible(true);
      
      map.setFilter(filter, true);
      
      this.factory = new MarkerFactory("http://rfid.cs.washington.edu/map-icons/");
      this.evan    = new TrackedEntity("Evan"                ,
                                       DefaultMapImages.BACKPACK,
                                       factory);
      this.cloud   = new ParticleCloud("Evan",DefaultMapImages.BLUE_PARTICLE,factory);
      this.time    = 1;
      
      vp.add(map);
      
      
      Button b = new Button("Advance Timestep");
      b.addClickHandler(new ClickHandler(){
         public void onClick(ClickEvent e){

            cloud.updateParticles(getParticles(time));
            map.processParticleCloud(cloud);
            // Change floors if necessary
            
            evan.updateLocation(getDataPoint(time));
            map.processTrackedEntity(evan);
            // Change floors if necessary
            
            time++;
         }
      });
      
      vp.add(b);
      
      RootPanel.get().add(vp);
   }
   
   
   public TagReading getDataPoint(int timestep){
      TagReading r = null;
      
      switch(timestep){
         case 1:
            r = new TagReading(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635));
            break;
         case 2:
            r = new TagReading(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307));
            break;
         case 3:
            r = new TagReading(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799));
            break;
         case 4:
            r = new TagReading(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664));
            break;
         default:
            break;
      }
      return r;
   }
   
   
   public ArrayList<Particle> getParticles(int timestep){
      ArrayList<Particle> particles = new ArrayList<Particle>();
      
      switch(timestep){
         case 1:
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.6));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.3));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.05));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.05));
            
            break;
         case 2:
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.4));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.27));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.04));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.23));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664), 0.002));
            
            break;
         case 3:
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532092743977,-122.305859103799), 0.1));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531654599707,-122.305787354708), 0.4));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531939167677,-122.305869832635), 0.009));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532137913469,-122.305818870664), 0.13));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532015955833,-122.305891290307), 0.002));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.653154619282,-122.305763885379), 0.012));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.653186237951,-122.3059040308), 0.002));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6531496506323,-122.305802106857), 0.001));
            particles.add(new Particle(MercatorMapWidget.DEFAULT_FLOORS[3], new LatLon(47.6532205667699,-122.305940911174), 0.001));
            
            break;
         default:
            break;
      }
      
      return particles;
   }
   
}
