package edu.washington.cs.rfid.map.client.ui.overlay;

import com.google.gwt.maps.client.geom.LatLng;
import com.google.gwt.maps.client.geom.Point;
import com.google.gwt.maps.client.geom.Size;
import com.google.gwt.maps.client.overlay.Icon;
import com.google.gwt.maps.client.overlay.Marker;
import com.google.gwt.maps.client.overlay.MarkerOptions;
import com.google.gwt.maps.client.overlay.Overlay;
import com.google.gwt.maps.client.overlay.Polyline;

/**
 * Arrow overlay class,
 * create an arrow to indicate historic movement.
 * @author gbc3
 *
 */
public class ArrowOverlay {

	private static final String ICON_BASE_URL  = "http://rfid.cs.washington.edu/dbicons";
	private static final int    ICON_SIZE      = 20;
	private static final int    DEFAULT_LENGTH = 40;
	
	private static Icon          defaultIcon;
	private static MarkerOptions optionsN;
	private static MarkerOptions optionsS;
	private static MarkerOptions optionsE;
	private static MarkerOptions optionsW;
	private static MarkerOptions optionsNW;
	private static MarkerOptions optionsNE;
	private static MarkerOptions optionsSW;
	private static MarkerOptions optionsSE;
	
	static {		
		Size size = Size.newInstance(20,20);		
		Icon icon;
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-0.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(0, (ICON_SIZE/2)));
		optionsE = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-45.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(0, ICON_SIZE));
		optionsNE = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-135.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(-(ICON_SIZE), ICON_SIZE));
		optionsNW = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-180.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(-(ICON_SIZE), (ICON_SIZE/2)));
		optionsW = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-90.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(-(ICON_SIZE/2), ICON_SIZE));
		optionsN = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-225.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(-(ICON_SIZE), 0));
		optionsSW = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-270.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(-(ICON_SIZE/2), 0));
		optionsS = MarkerOptions.newInstance(icon);
		
		icon = Icon.newInstance(ICON_BASE_URL + "/arrow-315.png");
		icon.setIconSize(size);
		icon.setIconAnchor(Point.newInstance(0, 0));
		optionsSE = MarkerOptions.newInstance(icon);
	}
	
	private ArrowOverlay(){};
	
	public static Overlay createArrow(int degrees, LatLng origin) {		
		return createIconArrow(degrees, origin);
	}
	
	private static Overlay createPolygonArrow(int degrees, LatLng origin) {
		int length = DEFAULT_LENGTH;
		double arrowEdgeLength = 0.25 * length;
		/*
		 * Point system should look like this..
		 * 
		 *    . _(p1)
		 *     \  - _   (p2)
		 *  (p0).    _- .
		 *     / _ -
		 *    '  (p3)
		 */				
		LatLng [] points = new LatLng[4];
		points[0] = LatLng.newInstance(origin.getLatitude(), origin.getLongitude());
		points[1] = LatLng.newInstance(origin.getLatitude() + (arrowEdgeLength * Math.cos(degrees + 135)),
				origin.getLongitude() + (arrowEdgeLength * Math.sin(degrees + 135)));
		points[2] = LatLng.newInstance(origin.getLatitude() + (length * Math.cos(degrees)),
				origin.getLongitude() + (length * Math.sin(degrees)));
		points[3] = LatLng.newInstance(origin.getLatitude() + (arrowEdgeLength * Math.cos(degrees + 225)),
				origin.getLongitude() + (arrowEdgeLength * Math.sin(degrees + 225)));
		
		return new Polyline(points);
	}
	
	private static Overlay createIconArrow(int degrees, LatLng origin) {
		// push everything up 22 (half of 45) deegrees and mod 360, so we can check per 45 intervals
		degrees = (degrees + 22) % 360;
		MarkerOptions options;
		if(degrees > 315) {			
			options = optionsSE;
		}
		else if(degrees > 270) {			
			options = optionsS;
		}
		else if(degrees > 225) {
			options = optionsSW;
		}
		else if(degrees > 180) {		
			options = optionsW;
		}
		else if(degrees > 135) {			
			options = optionsNW;
		}
		else if(degrees > 90) {			
			options = optionsN;
		}
		else if(degrees > 45) {			
			options = optionsNE;
		}
		else {
			options = optionsE;
		}
		
		return new Marker(origin, options);
	}	
}
