package edu.washington.cs.rfid.map.client.ui.overlay;

import com.google.gwt.maps.client.overlay.Overlay;

import edu.washington.cs.rfid.map.client.ui.MercatorMapWidget;

public abstract class OverlayFilter {

	private boolean           showAntennas;
	private boolean           showReaders;
	private boolean           showParticles;
	private boolean           showEntities;
	private boolean           showConnectivityGraph;
	private MercatorMapWidget map;
	
	//--//
	
	/**
	 * Creates a new OverlayFilter
	 */
	public OverlayFilter() {
		this.showAntennas          = false;
		this.showReaders           = false;
		this.showParticles         = false;
		this.showEntities          = false;
		this.showConnectivityGraph = false;
		this.map                   = null;
	}
	
	/**
    * Set the map for this filter
    * 
    * @param wrapperMap
    * @param realMap
    */
   public void setMap(MercatorMapWidget map) {
      this.map = map;
   }
	
	/**
	 * Set antennas visible or invisible in the map.
	 * @param visible
	 */
	public void setAntVisible(boolean visible) {
		if(this.showAntennas != visible) {
		   this.showAntennas = visible;
		   refresh();
		}
	}
	
	/**
	 * Set readers visible or invisible in the map
	 * @param visible
	 */
	public void setRdrVisible(boolean visible) {
		if(this.showReaders != visible) {
		   this.showReaders = visible;
			refresh();
		}
	}
	
	
	/**
	 * Set particles visible or invisible in the map.
	 * @param visible
	 */
	public void setParticlesVisible(boolean visible){
	   if(this.showParticles != visible){
	      this.showParticles = visible;
	      refresh();
	   }
	}
	
	
	/**
	 * Set entities visible of invisible in the map.
	 * @param visible
	 */
	public void setEntitiesVisible(boolean visible){
	   if(this.showEntities != visible){
	      this.showEntities = visible;
	      refresh();
	   }
	}
	
	
	/**
	 * Set connectivity graph edges to visible or invisible.
	 * @param visible
	 */
	public void setConnectivityGraphVisible(boolean visible){
	   if(this.showConnectivityGraph != visible){
	      this.showConnectivityGraph = visible;
	      refresh();
	   }
	}
	
	/**
	 * Will return true if this overlay passes the filter to draw on the map
	 * @param wrapper
	 * @return
	 */
	public boolean overlayAllowed(OverlayWrapper wrapper) {
	   boolean pass = false;
	   
		if(wrapper.typeID == OverlayWrapper.TYPE_ANTENNA) {
			pass = this.showAntennas;
		} else if(wrapper.typeID == OverlayWrapper.TYPE_READER) {
			pass = this.showReaders;
		} else if(wrapper.typeID == OverlayWrapper.TYPE_PARTICLE){
		   pass = this.showParticles;
		} else if(wrapper.typeID == OverlayWrapper.TYPE_ENTITY){
		   pass = this.showEntities;
		} else if(wrapper.typeID == OverlayWrapper.TYPE_CONNECTIVITY_EDGE){
		   pass = this.showConnectivityGraph;
		} else {
			pass = showCustom(wrapper.overlay, wrapper.typeID);
		}
		return pass;
	}
	
	//--//

	/**
    * Notify the map that it should redraw its icons
    */
   protected void refresh() {
      if(this.map != null) {
         this.map.refreshOverlays();
      }
   }

   
	/**
	 * Determine whether or not a non-antenna, non-reader icon should
	 * be displayed. Information of the type ID initially passed as well as
	 * the overlay is produced. 
	 * 
	 * Return statement says whether or not to draw it on the map
	 * 
	 * @param overlay
	 * @return
	 */
	protected abstract boolean showCustom(Overlay overlay, int typeID);		
	
	//--//
	
	/**
	 * Creates a default filter that shows all custom overlays.
	 * @return
	 */
	public static OverlayFilter createDefaultFilter() {
		return new OverlayFilter() {
		   @Override
			protected boolean showCustom(Overlay overlay, int typeid) {
				return true;
			}
		};
	}
}